=== modified file '.bzrignore'
--- .bzrignore	2009-04-01 14:40:05 +0000
+++ .bzrignore	2011-07-10 22:53:59 +0000
@@ -1,9 +1,12 @@
 ./dist
 ./loggerhead.egg-info
-./loggerhead.conf
 ./loggerhead.pid
 ./logs
 build
 *.log
 _trial_temp
 loggerhead-memprofile
+./docs/_build/
+tags
+.project
+.pydevproject

=== added file 'HACKING'
--- HACKING	1970-01-01 00:00:00 +0000
+++ HACKING	2011-02-10 23:05:31 +0000
@@ -0,0 +1,59 @@
+Loggerhead
+==========
+
+Overview
+--------
+
+This document attempts to give some hints for people that are wanting to work
+on Loggerhead.
+
+
+Testing
+-------
+
+You can run the loggerhead test suite as a bzr plugin. To run just the
+loggerhead tests::
+
+  bzr selftest -s bp.loggerhead
+
+
+Load Testing
+------------
+
+As a web service, Loggerhead will often be hit by multiple requests. We want
+to make sure that loggerhead can scale with many requests, without performing
+poorly or crashing under the load.
+
+There is a command ``bzr load-test-loggerhead`` that can be run to stress
+loggerhead. A script is given, describing what requests to make, against what
+URLs, and for what level of parallel activity.
+
+
+Load Testing Multiple Instances
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+One way that Launchpad provides both high availability and performance scaling
+is by running multiple instances of loggerhead, serving the same content. A
+proxy is then used to load balance the requests. This also allows us to shut
+down one instance for upgrading, without interupting service (requests are
+just routed to the other instance).
+
+However, multiple processes poses an even greater risk that caches will
+conflict. As such, it is useful to test that changes don't introduce coherency
+issues at load. ``bzr load-test-loggerhead`` can be configured with a script
+that will make requests against multiple loggerhead instances concurrently.
+
+To run multiple instances, it is often sufficient to just spawn multiple
+servers on different ports. For example::
+
+  $ bzr serve --http --port=8080 &
+  $ bzr serve --http --port=8081 &
+
+There is a simple example script already in the source tree::
+
+  $ bzr load-test-loggerhead load_test_scripts/multiple_instances.script
+
+
+
+.. vim: ft=rst tw=78
+

=== modified file 'MANIFEST.in'
--- MANIFEST.in	2008-06-22 18:04:15 +0000
+++ MANIFEST.in	2012-02-02 04:58:46 +0000
@@ -1,2 +1,6 @@
-include loggerhead.conf.example README.txt NEWS
+include NEWS
+include README.txt
+include apache-loggerhead.conf
+include bazaar.conf
+include loggerhead.conf.example
 recursive-include loggerhead/static *

=== modified file 'Makefile'
--- Makefile	2007-10-29 16:19:30 +0000
+++ Makefile	2011-03-14 08:47:03 +0000
@@ -8,5 +8,4 @@
 	rm -rf dist/
 
 check:
-	# XXX MichaelHudson 2007-10-29 bug=158361: We can't run the loggerhead
-	# tests in PQM yet :(
+	BZR_PLUGINS_AT=loggerhead@$$(pwd) bzr selftest -s bp.loggerhead

=== modified file 'NEWS'
--- NEWS	2009-08-20 13:30:37 +0000
+++ NEWS	2012-02-02 07:42:24 +0000
@@ -1,8 +1,107 @@
 What's changed in loggerhead?
 =============================
 
+NEXT
+----
+
+    - Add ``bzr load-test-loggerhead`` as a way to make sure loggerhead can
+      handle concurrent requests, etc. Scripts can be written that spawn
+      multiple threads, and issue concurrent requests.
+      (John Arbash Meinel)
+
+    - HEAD requests should not return body content. This is done by adding
+      another wsgi middleware that strips the body when the REQUEST_METHOD is
+      HEAD. Note that you have to add the middleware into your pipeline, and
+      it does not decrease the actual work done.
+      (John Arbash Meinel, #716201)
+
+    - If we get a HEAD request, there is no reason to expand the template, we
+      shouldn't be returning body content anyway.
+      (John Arbash Meinel, #716201, #716217)
+
+    - Merge the pqm changes back into trunk, after trunk was reverted to an old
+      revision. (John Arbash Meinel, #716152)
+
+    - Redirect ``/files/file.txt`` to ``/view/file.txt`` and ``/view/dir`` to
+      ``/files/dir``. (Jasper St. Pierre, #569358)
+
+    - Remove ``start-loggerhead`` and ``stop-loggerhead`` which were already
+      deprecated. (John Arbash Meinel)
+
+    - Show "Author(s)" as separate from "Committer". And label them
+      correctly. (John Arbash Meinel, #733015)
+
+    - The json module is no longer claimed to be supported as alternative for 
+      simplejson. (Jelmer Vernooij, #586611)
+
+    - Viewing the ``/changes`` page only iterates enough of the history to show
+      the actual revisions displayed, rather than walking the whole mainline.
+      Improves performance on projects with long histories like emacs.
+      (John Arbash Meinel)
+
+    - Fix support for displaying foreign revision ids.
+      (Jelmer Vernooij, #736026)
+
+    - Add hook 'controller' to BranchWSGIApp, allowing other bzr plugins
+      to provide custom branch-specific controllers. (Jelmer Vernooij, #585822)
+
+    - Add privacy notification code to loggerhead, allowing branches to be
+      marked as private and a notification ribbon to be displayed via
+      javascript on the view pages. (JC Sackett, #823471)
+
+    - Add a script and documentation for running under mod_wsgi.
+      (Stuart Colville, Toshio Kuratomi)
+
+    - Make tz calculations consistent and use UTC in the UI everywhere we show
+      a precise timestamp. (Robert Collins, #594591)
+
+
+1.18.1 [24Mar2011]
+------------------
+
+    - Fix escaping of filenames in revision views.
+      (William Grant, #740142)
+
+    - Add missing import to loggerhead.trace, allowing start-loggerhead
+      to run when a log.roll config option is set.
+      (Max Kanat-Alexander, #673999)
+
+
+1.18 [10Nov2010]
+----------------
+
+    - Syntax highlighting is no longer applied for files greater than 512K,
+      reducing codebrowse.launchpad.net overloading.
+      (Max Kanat-Alexander, #513044)
+
+    - Documentation added in the docs directory. README simplified
+      accordingly. (Tres Seaver).
+
+    - Show svn/git/hg revision ids in loggerhead revision view.
+      (Jelmer Vernooij)
+
+    - Fix .bzr/smart access to branches in shared repos. (You also need
+      a version of bzr with bug #348308 fixed.) (Andrew Bennetts)
+
+    - Support FastCGI, SCGI and AJP using flup. (Denis Martinez)
+
+    - Repository.get_revision_inventory() was removed in bzr 2.2; use
+      Repository.get_inventory() instead. (Matt Nordhoff, #528194)
+
+    - Ignore readonly+ prefix when checking if Loggerhead is serving a
+      local location. (Reported by Tres Seaver.) (Matt Nordhoff)
+
+    - Set Cache-Control and Expires headers on static pages.
+      (John Arbash Meinel)
+
+    - Generate relative links where possible (everywhere but HTTP
+      redirects and feed IDs). (Michael Hudson, Matt Nordhoff)
+
+    - Fix bad redirect when visiting "/download" or "/download/".
+      (Matt Nordhoff, #247992)
+
 1.17 [20Aug2009]
----------------
+----------------
 
     - Add bug links in revision informations (Alexandre Garnier, #314052)
 
@@ -106,7 +205,7 @@
 
     - Fix a file descriptor leak (Matt Nordhoff, #370845)
 
-    - Use transport API internally, so it is possible to specify a remote 
+    - Use transport API internally, so it is possible to specify a remote
       URL to serve-branches. (Jelmer Vernooij, #371787)
 
     - Fix internal server errors when using start-loggerhead. (Matt
@@ -155,7 +254,7 @@
 
 
 1.10 [22Dec2008]
----------------
+----------------
 
     - Add startup deamon script for Linux (Marius Kruger)
 

=== modified file 'README'
--- README	2009-10-02 09:18:10 +0000
+++ README	2010-04-23 01:30:29 +0000
@@ -1,124 +1,28 @@
-LOGGERHEAD
+Loggerhead
 ==========
 
-[ Version 1.17 for Bazaar 1.17 ]
-
-Loggerhead is a web viewer for Bazaar branches.  It can be used to
-navigate a branch history, annotate files, perform searches... all the
-usual things.
-
-
-GETTING STARTED
----------------
-
-Loggerhead depends on:
-
-1) SimpleTAL for templating.
-   On Ubuntu, `sudo apt-get install python-simpletal`
-   or download from http://www.owlfish.com/software/simpleTAL/download.html
-2) simplejson for producing JSON data.
-   On Ubuntu, `sudo apt-get install python-simplejson`
-   or use `easy_install simplejson`.
-3) Paste for the server. (You need version 1.2 or newer of Paste.)
-   On Ubuntu, `sudo apt-get install python-paste`
-   or use `easy_install Paste`
-4) Paste Deploy  (optional, needed when proxying through Apache)
-   On Ubuntu, `sudo apt-get install python-pastedeploy`
-   or use `easy_install PasteDeploy`
-
-Then simply run the 'serve-branches' with the branch you want to
-serve on the command line:
-
-    ./serve-branches ~/path/to/branch
-
-The script listens on port 8080 so head to http://localhost:8080/ in
-your browser to see the branch.  You can also pass a directory that
-contains branches to the script, and it will serve a very simple
-directory listing at other pages.
-
-You may update the Bazaar branches being viewed at any time.
-Loggerhead will notice and refresh, and Bazaar uses its own branch
-locking to prevent corruption.
-
-To run loggerhead as a linux daemon:
-1) Copy loggerheadd to /etc/init.d
-2) Edit the file to configure where your loggerhead is installed, and which
-   serve-branches options you would like.
-3) Register the service
-   cd /etc/init.d
-   a) on upstart based systems like Ubuntu run: 
-      update-rc.d loggerheadd defaults
-   b) on Sysvinit based systems like Centos or SuSE run:
-      chkconfig --add loggerheadd
-
-
-LOGGERHEAD AS A BAZAAR PLUGIN
------------------------------
-
-This branch contains experimental support for using Loggerhead as a Bazaar
-plugin.  To use it, place the top-level Loggerhead directory (the one
-containing this file) at ``~/.bazaar/plugins/loggerhead``.
-
-
-USING A CONFIG FILE
--------------------
-
-To hide branches from being displayed, add to ``~/.bazaar/locations.conf``,
-under the branch's section:
-
-    [/path/to/branch]
-    http_serve = False
-
-
-More configuration options to come soon.
-
-
-SERVING LOGGERHEAD FROM BEHIND APACHE
--------------------------------------
-
-If you want to view Bazaar branches from your existing Apache
-installation, you'll need to configure Apache to proxy certain
-requests to Loggerhead.  Adding lines like this to you Apache
-configuration is one way to do this::
-
-    <Location "/branches/">
-        ProxyPass http://127.0.0.1:8080/branches/
-        ProxyPassReverse http://127.0.0.1:8080/branches/
-    </Location>
-
-If Paste Deploy is installed, the 'serve-branches' script can be
-run behind a proxy at the root of a site, but if you're running it at
-some path into the site, you'll need to specify is using '--prefix=/some_path'.
-
-
-SEARCH
-------
-
-Search is currently supported by using the bzr-search plugin (available
-at: ``https://launchpad.net/bzr-search``
-You need to have the plugin installed and each branch indexed to allow
-searching on branches.
-
-SUPPORT
--------
-
-Loggerhead is loosely based on bazaar-webserve, which was loosely
-based on hgweb.  Discussion should take place on the bazaar-dev
-mailing list at bazaar@lists.canonical.com.  You can join the list at
-<https://lists.ubuntu.com/mailman/listinfo/bazaar>.  You don't need to
-subscribe to post, but your first post will be held briefly for manual
-moderation.
-
-Bugs are tracked on Launchpad; start at:
-
-    https://bugs.launchpad.net/loggerhead
-
-
-HACKING
--------
-
-To run loggerhead tests, you will need the package python-nose,
-and run in the loggerhead directory:
-
-    nosetests
-
+Overview
+--------
+
+Loggerhead is a web viewer for Bazaar branches. Its lets users do
+stuff like:
+
+* navigate through branch history
+* view the files in a given revision
+* annotate files (i.e. find out the origin of each line of a file)
+* perform searches.
+
+
+Documentation
+-------------
+
+See docs/index.rst for installation instructions and basic
+documentation.
+
+
+Licensing
+---------
+
+This software is (C) Copyright Canonical Limited 2006-10 under the
+GPL Version 2 or later. Please see the file COPYING.txt for the
+licence details.

=== modified file '__init__.py'
--- __init__.py	2009-10-17 06:35:33 +0000
+++ __init__.py	2011-04-12 10:18:52 +0000
@@ -1,4 +1,4 @@
-# Copyright 2009 Canonical Ltd
+# Copyright 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,52 +26,55 @@
 
 """Loggerhead web viewer for Bazaar branches.
 
-This provides a new option "--http" to the "bzr serve" command, that 
+This provides a new option "--http" to the "bzr serve" command, that
 starts a web server to browse the contents of a branch.
 """
 
-version_info = (1, 17, 0)
+import sys
+
+from info import (
+    bzr_plugin_version as version_info,
+    bzr_compatible_versions,
+    )
 
 if __name__ == 'bzrlib.plugins.loggerhead':
     import bzrlib
     from bzrlib.api import require_any_api
-
-    require_any_api(bzrlib, [
-        (1, 13, 0), (1, 15, 0), (1, 16, 0), (1, 17, 0), (1, 18, 0),
-        (2, 0, 0), (2, 1, 0)])
-
-    # NB: Normally plugins should lazily load almost everything, but this
-    # seems reasonable to have in-line here: bzrlib.commands and options are
-    # normally loaded, and the rest of loggerhead won't be loaded until serve
-    # --http is run.
-
-    # transport_server_registry was added in bzr 1.16. When we drop support for
-    # older releases, we can remove the code to override cmd_serve.
-
-    try:
-        from bzrlib.transport import transport_server_registry
-    except ImportError:
-        transport_server_registry = None
+    from bzrlib import commands
+
+    require_any_api(bzrlib, bzr_compatible_versions)
+
+    from bzrlib.transport import transport_server_registry
 
     DEFAULT_HOST = '0.0.0.0'
     DEFAULT_PORT = 8080
     HELP = ('Loggerhead, a web-based code viewer and server. (default port: %d)' %
             (DEFAULT_PORT,))
 
-    def serve_http(transport, host=None, port=None, inet=None):
-        from paste.httpexceptions import HTTPExceptionHandler
-        from paste.httpserver import serve
-
+    def _ensure_loggerhead_path():
+        """Ensure that you can 'import loggerhead' and get the root."""
         # loggerhead internal code will try to 'import loggerhead', so
         # let's put it on the path if we can't find it in the existing path
         try:
-            import loggerhead
+            import loggerhead.apps.transport
         except ImportError:
             import os.path, sys
             sys.path.append(os.path.dirname(__file__))
 
+    def serve_http(transport, host=None, port=None, inet=None):
+        # TODO: if we supported inet to pass requests in and respond to them,
+        #       then it would be easier to test the full stack, but it probably
+        #       means routing around paste.httpserver.serve which probably
+        #       isn't testing the full stack
+        from paste.httpexceptions import HTTPExceptionHandler
+        from paste.httpserver import serve
+
+        _ensure_loggerhead_path()
+
+        from loggerhead.apps.http_head import HeadMiddleware
         from loggerhead.apps.transport import BranchesFromTransportRoot
         from loggerhead.config import LoggerheadConfig
+        from loggerhead.main import setup_logging
 
         if host is None:
             host = DEFAULT_HOST
@@ -81,42 +84,39 @@
         if not transport.is_readonly():
             argv.insert(0, '--allow-writes')
         config = LoggerheadConfig(argv)
+        setup_logging(config, init_logging=False, log_file=sys.stderr)
         app = BranchesFromTransportRoot(transport.base, config)
+        # Bug #758618, HeadMiddleware seems to break HTTPExceptionHandler from
+        # actually sending appropriate return codes to the client. Since nobody
+        # desperately needs HeadMiddleware right now, just ignoring it.
+        # app = HeadMiddleware(app)
         app = HTTPExceptionHandler(app)
         serve(app, host=host, port=port)
 
-    if transport_server_registry is not None:
-        transport_server_registry.register('http', serve_http, help=HELP)
-    else:
-        import bzrlib.builtins
-        from bzrlib.commands import get_cmd_object, register_command
-        from bzrlib.option import Option
-
-        _original_command = get_cmd_object('serve')
-
-        class cmd_serve(bzrlib.builtins.cmd_serve):
-            __doc__ = _original_command.__doc__
-
-            takes_options = _original_command.takes_options + [
-                Option('http', help=HELP)]
-
-            def run(self, *args, **kw):
-                if 'http' in kw:
-                    from bzrlib.transport import get_transport
-                    allow_writes = kw.get('allow_writes', False)
-                    path = kw.get('directory', '.')
-                    port = kw.get('port', DEFAULT_PORT)
-                    # port might be an int already...
-                    if isinstance(port, basestring) and ':' in port:
-                        host, port = port.split(':')
-                    else:
-                        host = DEFAULT_HOST
-                    if allow_writes:
-                        transport = get_transport(path)
-                    else:
-                        transport = get_transport('readonly+' + path)
-                    serve_http(transport, host, port)
-                else:
-                    super(cmd_serve, self).run(*args, **kw)
-
-        register_command(cmd_serve)
+    transport_server_registry.register('http', serve_http, help=HELP)
+
+    class cmd_load_test_loggerhead(commands.Command):
+        """Run a load test against a live loggerhead instance.
+
+        Pass in the name of a script file to run. See loggerhead/load_test.py
+        for a description of the file format.
+        """
+
+        takes_args = ["filename"]
+
+        def run(self, filename):
+            from bzrlib.plugins.loggerhead.loggerhead import load_test
+            script = load_test.run_script(filename)
+            for thread_id in sorted(script._threads):
+                worker = script._threads[thread_id][0]
+                for url, success, time in worker.stats:
+                    self.outf.write(' %5.3fs %s %s\n'
+                                    % (time, str(success)[0], url))
+
+    commands.register_command(cmd_load_test_loggerhead)
+
+    def load_tests(standard_tests, module, loader):
+        _ensure_loggerhead_path()
+        standard_tests.addTests(loader.loadTestsFromModuleNames(
+            ['bzrlib.plugins.loggerhead.loggerhead.tests']))
+        return standard_tests

=== added file 'apache-loggerhead.conf'
--- apache-loggerhead.conf	1970-01-01 00:00:00 +0000
+++ apache-loggerhead.conf	2010-04-18 17:55:33 +0000
@@ -0,0 +1,19 @@
+### If you receive MemoryError tracebacks setting up loggerhead under mod_wsgi,
+### read /usr/share/doc/loggerhead*/README for help
+Alias /bzr/static /usr/share/loggerhead/static
+RewriteEngine On
+RewriteRule ^/bzr$ /bzr/ [R]
+
+WSGIDaemonProcess loggerhead user=apache group=apache maximum-requests=1000 display-name=loggerhead processes=4 threads=1
+WSGISocketPrefix run/wsgi
+WSGIRestrictStdout On
+WSGIRestrictSignal Off
+
+WSGIScriptAlias /bzr /usr/bin/loggerhead.wsgi
+
+<Location /bzr>
+    WSGIProcessGroup loggerhead
+    Order deny,allow
+    Allow from all
+</Location>
+

=== added file 'bazaar.conf'
--- bazaar.conf	1970-01-01 00:00:00 +0000
+++ bazaar.conf	2010-04-18 14:41:23 +0000
@@ -0,0 +1,9 @@
+# directory to serve bzr branches from
+# Non-bzr directories under this path will also be visible in loggerhead
+#http_root_dir = '/var/www/bzr'
+
+# The url prefix for the bzr branches.
+http_user_prefix = '/bzr'
+
+# Directory to put cache files in
+http_sql_dir = '/var/cache/loggerhead'

=== added directory 'docs'
=== added file 'docs/Makefile'
--- docs/Makefile	1970-01-01 00:00:00 +0000
+++ docs/Makefile	2010-03-23 15:44:51 +0000
@@ -0,0 +1,89 @@
+# Makefile for Sphinx documentation
+#
+
+# You can set these variables from the command line.
+SPHINXOPTS    =
+SPHINXBUILD   = sphinx-build
+PAPER         =
+BUILDDIR      = _build
+
+# Internal variables.
+PAPEROPT_a4     = -D latex_paper_size=a4
+PAPEROPT_letter = -D latex_paper_size=letter
+ALLSPHINXOPTS   = -d $(BUILDDIR)/doctrees $(PAPEROPT_$(PAPER)) $(SPHINXOPTS) .
+
+.PHONY: help clean html dirhtml pickle json htmlhelp qthelp latex changes linkcheck doctest
+
+help:
+	@echo "Please use \`make <target>' where <target> is one of"
+	@echo "  html      to make standalone HTML files"
+	@echo "  dirhtml   to make HTML files named index.html in directories"
+	@echo "  pickle    to make pickle files"
+	@echo "  json      to make JSON files"
+	@echo "  htmlhelp  to make HTML files and a HTML help project"
+	@echo "  qthelp    to make HTML files and a qthelp project"
+	@echo "  latex     to make LaTeX files, you can set PAPER=a4 or PAPER=letter"
+	@echo "  changes   to make an overview of all changed/added/deprecated items"
+	@echo "  linkcheck to check all external links for integrity"
+	@echo "  doctest   to run all doctests embedded in the documentation (if enabled)"
+
+clean:
+	-rm -rf $(BUILDDIR)/*
+
+html:
+	$(SPHINXBUILD) -b html $(ALLSPHINXOPTS) $(BUILDDIR)/html
+	@echo
+	@echo "Build finished. The HTML pages are in $(BUILDDIR)/html."
+
+dirhtml:
+	$(SPHINXBUILD) -b dirhtml $(ALLSPHINXOPTS) $(BUILDDIR)/dirhtml
+	@echo
+	@echo "Build finished. The HTML pages are in $(BUILDDIR)/dirhtml."
+
+pickle:
+	$(SPHINXBUILD) -b pickle $(ALLSPHINXOPTS) $(BUILDDIR)/pickle
+	@echo
+	@echo "Build finished; now you can process the pickle files."
+
+json:
+	$(SPHINXBUILD) -b json $(ALLSPHINXOPTS) $(BUILDDIR)/json
+	@echo
+	@echo "Build finished; now you can process the JSON files."
+
+htmlhelp:
+	$(SPHINXBUILD) -b htmlhelp $(ALLSPHINXOPTS) $(BUILDDIR)/htmlhelp
+	@echo
+	@echo "Build finished; now you can run HTML Help Workshop with the" \
+	      ".hhp project file in $(BUILDDIR)/htmlhelp."
+
+qthelp:
+	$(SPHINXBUILD) -b qthelp $(ALLSPHINXOPTS) $(BUILDDIR)/qthelp
+	@echo
+	@echo "Build finished; now you can run "qcollectiongenerator" with the" \
+	      ".qhcp project file in $(BUILDDIR)/qthelp, like this:"
+	@echo "# qcollectiongenerator $(BUILDDIR)/qthelp/loggerhead.qhcp"
+	@echo "To view the help file:"
+	@echo "# assistant -collectionFile $(BUILDDIR)/qthelp/loggerhead.qhc"
+
+latex:
+	$(SPHINXBUILD) -b latex $(ALLSPHINXOPTS) $(BUILDDIR)/latex
+	@echo
+	@echo "Build finished; the LaTeX files are in $(BUILDDIR)/latex."
+	@echo "Run \`make all-pdf' or \`make all-ps' in that directory to" \
+	      "run these through (pdf)latex."
+
+changes:
+	$(SPHINXBUILD) -b changes $(ALLSPHINXOPTS) $(BUILDDIR)/changes
+	@echo
+	@echo "The overview file is in $(BUILDDIR)/changes."
+
+linkcheck:
+	$(SPHINXBUILD) -b linkcheck $(ALLSPHINXOPTS) $(BUILDDIR)/linkcheck
+	@echo
+	@echo "Link check complete; look for any errors in the above output " \
+	      "or in $(BUILDDIR)/linkcheck/output.txt."
+
+doctest:
+	$(SPHINXBUILD) -b doctest $(ALLSPHINXOPTS) $(BUILDDIR)/doctest
+	@echo "Testing of doctests in the sources finished, look at the " \
+	      "results in $(BUILDDIR)/doctest/output.txt."

=== added directory 'docs/_build'
=== added directory 'docs/_static'
=== added directory 'docs/_templates'
=== added file 'docs/conf.py'
--- docs/conf.py	1970-01-01 00:00:00 +0000
+++ docs/conf.py	2010-03-25 10:25:13 +0000
@@ -0,0 +1,194 @@
+# -*- coding: utf-8 -*-
+#
+# loggerhead documentation build configuration file, created by
+# sphinx-quickstart on Tue Mar 23 10:49:50 2010.
+#
+# This file is execfile()d with the current directory set to its containing dir.
+#
+# Note that not all possible configuration values are present in this
+# autogenerated file.
+#
+# All configuration values have a default; values that are commented out
+# serve to show the default.
+
+import sys, os
+
+# If extensions (or modules to document with autodoc) are in another directory,
+# add these directories to sys.path here. If the directory is relative to the
+# documentation root, use os.path.abspath to make it absolute, like shown here.
+#sys.path.append(os.path.abspath('.'))
+
+# -- General configuration -----------------------------------------------------
+
+# Add any Sphinx extension module names here, as strings. They can be extensions
+# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
+extensions = []
+
+# Add any paths that contain templates here, relative to this directory.
+templates_path = ['_templates']
+
+# The suffix of source filenames.
+source_suffix = '.rst'
+
+# The encoding of source files.
+#source_encoding = 'utf-8'
+
+# The master toctree document.
+master_doc = 'index'
+
+# General information about the project.
+project = u'Loggerhead'
+copyright = u'2010, Loggerhead team (https://launchpad.net/~loggerhead-team)'
+
+# The version info for the project you're documenting, acts as replacement for
+# |version| and |release|, also used in various other places throughout the
+# built documents.
+#
+# The short X.Y version.
+version = '1.17'
+# The full version, including alpha/beta/rc tags.
+release = '1.17'
+
+# The language for content autogenerated by Sphinx. Refer to documentation
+# for a list of supported languages.
+#language = None
+
+# There are two options for replacing |today|: either, you set today to some
+# non-false value, then it is used:
+#today = ''
+# Else, today_fmt is used as the format for a strftime call.
+#today_fmt = '%B %d, %Y'
+
+# List of documents that shouldn't be included in the build.
+#unused_docs = []
+
+# List of directories, relative to source directory, that shouldn't be searched
+# for source files.
+exclude_trees = ['_build']
+
+# The reST default role (used for this markup: `text`) to use for all documents.
+#default_role = None
+
+# If true, '()' will be appended to :func: etc. cross-reference text.
+#add_function_parentheses = True
+
+# If true, the current module name will be prepended to all description
+# unit titles (such as .. function::).
+#add_module_names = True
+
+# If true, sectionauthor and moduleauthor directives will be shown in the
+# output. They are ignored by default.
+#show_authors = False
+
+# The name of the Pygments (syntax highlighting) style to use.
+pygments_style = 'sphinx'
+
+# A list of ignored prefixes for module index sorting.
+#modindex_common_prefix = []
+
+
+# -- Options for HTML output ---------------------------------------------------
+
+# The theme to use for HTML and HTML Help pages.  Major themes that come with
+# Sphinx are currently 'default' and 'sphinxdoc'.
+html_theme = 'default'
+
+# Theme options are theme-specific and customize the look and feel of a theme
+# further.  For a list of options available for each theme, see the
+# documentation.
+#html_theme_options = {}
+
+# Add any paths that contain custom themes here, relative to this directory.
+#html_theme_path = []
+
+# The name for this set of Sphinx documents.  If None, it defaults to
+# "<project> v<release> documentation".
+#html_title = None
+
+# A shorter title for the navigation bar.  Default is the same as html_title.
+#html_short_title = None
+
+# The name of an image file (relative to this directory) to place at the top
+# of the sidebar.
+#html_logo = None
+
+# The name of an image file (within the static path) to use as favicon of the
+# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
+# pixels large.
+#html_favicon = None
+
+# Add any paths that contain custom static files (such as style sheets) here,
+# relative to this directory. They are copied after the builtin static files,
+# so a file named "default.css" will overwrite the builtin "default.css".
+html_static_path = ['_static']
+
+# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
+# using the given strftime format.
+#html_last_updated_fmt = '%b %d, %Y'
+
+# If true, SmartyPants will be used to convert quotes and dashes to
+# typographically correct entities.
+#html_use_smartypants = True
+
+# Custom sidebar templates, maps document names to template names.
+#html_sidebars = {}
+
+# Additional templates that should be rendered to pages, maps page names to
+# template names.
+#html_additional_pages = {}
+
+# If false, no module index is generated.
+#html_use_modindex = True
+
+# If false, no index is generated.
+#html_use_index = True
+
+# If true, the index is split into individual pages for each letter.
+#html_split_index = False
+
+# If true, links to the reST sources are added to the pages.
+#html_show_sourcelink = True
+
+# If true, an OpenSearch description file will be output, and all pages will
+# contain a <link> tag referring to it.  The value of this option must be the
+# base URL from which the finished HTML is served.
+#html_use_opensearch = ''
+
+# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
+#html_file_suffix = ''
+
+# Output file base name for HTML help builder.
+htmlhelp_basename = 'loggerheaddoc'
+
+
+# -- Options for LaTeX output --------------------------------------------------
+
+# The paper size ('letter' or 'a4').
+#latex_paper_size = 'letter'
+
+# The font size ('10pt', '11pt' or '12pt').
+#latex_font_size = '10pt'
+
+# Grouping the document tree into LaTeX files. List of tuples
+# (source start file, target name, title, author, documentclass [howto/manual]).
+latex_documents = [
+  ('index', 'loggerhead.tex', u'Loggerhead Documentation',
+   u'Loggerhead team (https://launchpad.net/\\textasciitilde{}loggerhead-team)', 'manual'),
+]
+
+# The name of an image file (relative to this directory) to place at the top of
+# the title page.
+#latex_logo = None
+
+# For "manual" documents, if this is true, then toplevel headings are parts,
+# not chapters.
+#latex_use_parts = False
+
+# Additional stuff for the LaTeX preamble.
+#latex_preamble = ''
+
+# Documents to append as an appendix to all manuals.
+#latex_appendices = []
+
+# If false, no module index is generated.
+#latex_use_modindex = True

=== added file 'docs/index.rst'
--- docs/index.rst	1970-01-01 00:00:00 +0000
+++ docs/index.rst	2012-02-02 04:58:46 +0000
@@ -0,0 +1,264 @@
+Loggerhead:  A web viewer for ``bzr`` branches
+==============================================
+
+Loggerhead is a web viewer for projects in Bazaar. It can be used to navigate 
+a branch history, annotate files, view patches, perform searches, etc.
+Loggerhead is heavily based on `bazaar-webserve
+<https://launchpad.net/bzr-webserve>`_, which was, in turn, loosely
+based on `hgweb <http://mercurial.selenic.com/wiki/HgWebDirStepByStep>`_.
+
+
+Getting Started
+---------------
+
+Loggerhead depends on the following Python libraries.:
+
+- SimpleTAL for templating.
+
+- simplejson for producing JSON data.
+
+- Paste for the server. (You need version 1.2 or newer of Paste).
+
+- Paste Deploy  (optional, needed when proxying through Apache).
+
+- flup (optional, needed to use FastCGI, SCGI or AJP).
+
+
+Installing Dependencies Using Ubuntu Packages
+#############################################
+
+.. code-block:: sh
+
+   $ sudo apt-get install python-simpletal
+   $ sudo apt-get install python-simplejson
+   $ sudo apt-get install python-paste
+   $ sudo apt-get install python-pastedeploy
+   $ sudo apt-get install python-flup
+
+Installing Dependencies Using :command:`easy_install`
+#####################################################
+
+.. code-block:: sh
+
+   $ easy_install \
+     -f http://www.owlfish.com/software/simpleTAL/py2compatible/download.html \
+     SimpleTAL
+   $ easy_install simplejson
+   $ easy_install Paste
+   $ easy_install PasteDeploy
+   $ easy_install flup
+
+
+Running the Standalone Loggerhead Server
+----------------------------------------
+
+After installing all the dependencies, you should be able to run
+:command:`serve-branches` with the branch you want to serve on the
+command line:
+
+.. code-block:: sh
+
+    ./serve-branches ~/path/to/branch
+
+By default, the script listens on port 8080, so head to
+http://localhost:8080/ in your browser to see the branch.
+
+You can also pass a directory that contains branches to the script,
+and it will serve a very simple directory listing at other pages.
+
+You may update the Bazaar branches being viewed at any time.
+Loggerhead will notice and refresh, and Bazaar uses its own branch
+locking to prevent corruption.
+
+See :doc:`serve-branches` for all command line options.
+
+Running Loggerhead as a Daemon
+------------------------------
+
+To run Loggerhead as a linux daemon:
+
+1) Copy the ``loggerheadd`` scipt to ``/etc/init.d``
+
+.. code-block:: sh
+
+   $ sudo cp ./loggerheadd /etc/init.d
+
+2) Edit the file to configure where your Loggerhead is installed, and which
+   serve-branches options you would like.
+
+.. code-block:: sh
+
+   $ sudo vim /etc/init.d/loggerheadd
+
+3) Register the service
+
+.. code-block:: sh
+
+   # on upstart based systems like Ubuntu run: 
+   $ sudo update-rc.d loggerheadd defaults
+
+   # on Sysvinit based systems like Centos or SuSE run:
+   $ sudo chkconfig --add loggerheadd
+
+
+Using Loggerhead as a Bazaar Plugin
+------------------------------------
+
+This branch contains experimental support for using Loggerhead as a Bazaar
+plugin.  To use it, place the top-level Loggerhead directory (the one
+containing COPYING.txt) at ``~/.bazaar/plugins/loggerhead``.  E.g.:
+
+.. code-block:: sh
+
+   $ bzr branch lp:loggerhead ~/.bazaar/plugins/loggerhead
+   $ cd ~/myproject
+   $ bzr serve --http
+
+
+Using a Config File
+-------------------
+
+To hide branches from being displayed, add to ``~/.bazaar/locations.conf``,
+under the branch's section:
+
+.. code-block:: ini
+
+    [/path/to/branch]
+    http_serve = False
+
+More configuration options to come soon.
+
+
+Serving Loggerhead behind Apache
+--------------------------------
+
+If you want to view Bazaar branches from your existing Apache
+installation, you'll need to configure Apache to proxy certain
+requests to Loggerhead.  Adding lines like this to your Apache
+configuration is one way to do this:
+
+.. code-block:: apache
+
+    <Location "/branches/">
+        ProxyPass http://127.0.0.1:8080/branches/
+        ProxyPassReverse http://127.0.0.1:8080/branches/
+    </Location>
+
+If Paste Deploy is installed, the :command:`serve-branches` script can be
+run behind a proxy at the root of a site, but if you're running it at
+some path into the site, you'll need to specify it using
+``--prefix=/some_path``.
+
+Serving Loggerhead with mod_wsgi
+--------------------------------
+
+A second method for using Loggerhead with apache is to have apache itself
+execute Loggerhead via mod_wsgi.  You need to add configuration for apache and
+for bazaar to make this work.  Example config files are in the Loggerhead doc
+directory as apache-loggerhead.conf and bazaar.conf.  You can copy them into
+place and use them as a starting point following these directions:
+
+1) Install mod_wsgi.  On Ubuntu and other Debian derived distros::
+
+    sudo apt-get install libapache2-mod-wsgi
+
+   On Fedora-derived distros::
+
+    su -c yum install mod_wsgi
+
+2) Copy the bazaar.conf file where apache will find it (May be done for you if
+   you installed Loggerhead from a distribution package)::
+
+    # install -d -o apache -g apache -m 0755 /etc/loggerhead
+    # cp -p /usr/share/doc/loggerhead*/bazaar.conf /etc/loggerhead/
+    # ln -s /etc/loggerhead /var/www/.bazaar
+
+3) Create the cache directory (May be done for you if you installed Loggerhead
+   from a distribution package)::
+
+    # install -d -o apache -g apache -m 0700 /var/cache/loggerhead/
+
+4) Edit /etc/loggerhead/bazaar.conf.  You need to set http_root_dir to the filesystem
+   path that you will find your bzr branches under.  Note that normal
+   directories under that path will also be visible in Loggerhead.
+
+5) Install the apache conf file::
+
+     # cp -p /usr/share/doc/loggerhead*/apache-loggerhead.conf /etc/httpd/conf.d/loggerhead.conf
+
+6) Edit /etc/httpd/conf.d/loggerhead.conf to point to the url you desire to
+   serve Loggerhead on.  This should match with the setting for
+   http_user_prefix in bazaar.conf
+
+7) Restart apache and you should be able to start browsing
+
+.. note:: If you have SELinux enabled on your system you may need to allow
+   apache to execute files in temporary directories.  You will get a
+   MemoryError traceback from python if this is the case.  This is because of
+   the way that python ctypes interacts with libffi.  To rectify this, you may
+   have to do several things, such as mounting tmpdirs so programs can be
+   executed on them and setting this SELinux boolean::
+
+       setsebool httpd_tmp_exec on
+
+   This bug has information about how python and/or Linux distros might solve
+   this issue permanently and links to bugs which diagnose the root cause.
+   https://bugzilla.redhat.com/show_bug.cgi?id=582009
+
+Search
+------
+
+Search is currently supported by using the bzr-search plugin (available
+at: https://launchpad.net/bzr-search ).
+
+You need to have the plugin installed and each branch indexed to allow
+searching on branches.
+
+Command-Line Reference
+----------------------
+
+.. toctree::
+   :maxdepth: 2
+
+   serve-branches
+
+
+Support
+-------
+
+Discussion should take place on the bazaar-dev mailing list at
+mailto:bazaar@lists.canonical.com.  You can join the list at
+<https://lists.ubuntu.com/mailman/listinfo/bazaar>.  You don't need to
+subscribe to post, but your first post will be held briefly for manual
+moderation.
+
+Bugs, support questions and merge proposals are tracked on Launchpad, e.g:
+
+    https://bugs.launchpad.net/loggerhead
+
+
+Hacking
+-------
+
+To run Loggerhead tests, you will need to install the package ``python-nose``,
+and run its :command:`nosetests` script in the Loggerhead directory:
+
+.. code-block:: sh
+
+    nosetests
+
+
+License
+-------
+
+GNU GPLv2 or later.
+
+See Also
+--------
+
+https://launchpad.net/loggerhead
+
+Index
+=====
+
+- :ref:`genindex`

=== added file 'docs/make.bat'
--- docs/make.bat	1970-01-01 00:00:00 +0000
+++ docs/make.bat	2010-03-23 15:44:51 +0000
@@ -0,0 +1,113 @@
+@ECHO OFF
+
+REM Command file for Sphinx documentation
+
+set SPHINXBUILD=sphinx-build
+set BUILDDIR=_build
+set ALLSPHINXOPTS=-d %BUILDDIR%/doctrees %SPHINXOPTS% .
+if NOT "%PAPER%" == "" (
+	set ALLSPHINXOPTS=-D latex_paper_size=%PAPER% %ALLSPHINXOPTS%
+)
+
+if "%1" == "" goto help
+
+if "%1" == "help" (
+	:help
+	echo.Please use `make ^<target^>` where ^<target^> is one of
+	echo.  html      to make standalone HTML files
+	echo.  dirhtml   to make HTML files named index.html in directories
+	echo.  pickle    to make pickle files
+	echo.  json      to make JSON files
+	echo.  htmlhelp  to make HTML files and a HTML help project
+	echo.  qthelp    to make HTML files and a qthelp project
+	echo.  latex     to make LaTeX files, you can set PAPER=a4 or PAPER=letter
+	echo.  changes   to make an overview over all changed/added/deprecated items
+	echo.  linkcheck to check all external links for integrity
+	echo.  doctest   to run all doctests embedded in the documentation if enabled
+	goto end
+)
+
+if "%1" == "clean" (
+	for /d %%i in (%BUILDDIR%\*) do rmdir /q /s %%i
+	del /q /s %BUILDDIR%\*
+	goto end
+)
+
+if "%1" == "html" (
+	%SPHINXBUILD% -b html %ALLSPHINXOPTS% %BUILDDIR%/html
+	echo.
+	echo.Build finished. The HTML pages are in %BUILDDIR%/html.
+	goto end
+)
+
+if "%1" == "dirhtml" (
+	%SPHINXBUILD% -b dirhtml %ALLSPHINXOPTS% %BUILDDIR%/dirhtml
+	echo.
+	echo.Build finished. The HTML pages are in %BUILDDIR%/dirhtml.
+	goto end
+)
+
+if "%1" == "pickle" (
+	%SPHINXBUILD% -b pickle %ALLSPHINXOPTS% %BUILDDIR%/pickle
+	echo.
+	echo.Build finished; now you can process the pickle files.
+	goto end
+)
+
+if "%1" == "json" (
+	%SPHINXBUILD% -b json %ALLSPHINXOPTS% %BUILDDIR%/json
+	echo.
+	echo.Build finished; now you can process the JSON files.
+	goto end
+)
+
+if "%1" == "htmlhelp" (
+	%SPHINXBUILD% -b htmlhelp %ALLSPHINXOPTS% %BUILDDIR%/htmlhelp
+	echo.
+	echo.Build finished; now you can run HTML Help Workshop with the ^
+.hhp project file in %BUILDDIR%/htmlhelp.
+	goto end
+)
+
+if "%1" == "qthelp" (
+	%SPHINXBUILD% -b qthelp %ALLSPHINXOPTS% %BUILDDIR%/qthelp
+	echo.
+	echo.Build finished; now you can run "qcollectiongenerator" with the ^
+.qhcp project file in %BUILDDIR%/qthelp, like this:
+	echo.^> qcollectiongenerator %BUILDDIR%\qthelp\loggerhead.qhcp
+	echo.To view the help file:
+	echo.^> assistant -collectionFile %BUILDDIR%\qthelp\loggerhead.ghc
+	goto end
+)
+
+if "%1" == "latex" (
+	%SPHINXBUILD% -b latex %ALLSPHINXOPTS% %BUILDDIR%/latex
+	echo.
+	echo.Build finished; the LaTeX files are in %BUILDDIR%/latex.
+	goto end
+)
+
+if "%1" == "changes" (
+	%SPHINXBUILD% -b changes %ALLSPHINXOPTS% %BUILDDIR%/changes
+	echo.
+	echo.The overview file is in %BUILDDIR%/changes.
+	goto end
+)
+
+if "%1" == "linkcheck" (
+	%SPHINXBUILD% -b linkcheck %ALLSPHINXOPTS% %BUILDDIR%/linkcheck
+	echo.
+	echo.Link check complete; look for any errors in the above output ^
+or in %BUILDDIR%/linkcheck/output.txt.
+	goto end
+)
+
+if "%1" == "doctest" (
+	%SPHINXBUILD% -b doctest %ALLSPHINXOPTS% %BUILDDIR%/doctest
+	echo.
+	echo.Testing of doctests in the sources finished, look at the ^
+results in %BUILDDIR%/doctest/output.txt.
+	goto end
+)
+
+:end

=== added file 'docs/serve-branches.rst'
--- docs/serve-branches.rst	1970-01-01 00:00:00 +0000
+++ docs/serve-branches.rst	2010-03-25 10:32:47 +0000
@@ -0,0 +1,116 @@
+:command:`serve-branches`
+=========================
+
+The :command:`serve-branches` script runs a standalone Loggerhead server in
+the foreground.
+
+.. program:: serve-branches
+
+Usage
+-----
+
+.. code-block:: sh
+
+   serve-branches [OPTIONS] <target directory>
+
+Options
+-------
+
+.. cmdoption:: --user-dirs
+
+    Serve user directories as ``~user`` (requires ``--trunk-dir``).
+
+    If both options are set, then for requests where the CGI ``PATH_INFO``
+    starts with "/~<name>", serve branches under the <name> directory.
+
+.. cmdoption:: --trunk-dir=DIR
+
+    The directory that contains the trunk branches (requires ``--user-dirs``).
+
+    If both options are set, then for requests where the CGI ``PATH_INFO``
+    does not start with "/~<name>", serve branches under DIR.
+
+.. cmdoption:: --port
+
+    Listen on the given port.
+    
+    Defaults to 8080.
+
+.. cmdoption:: --host
+
+    Listen on the interface corresponding to the given IP. 
+    
+    Defaults to listening on all interfaces, i.e., "0.0.0.0".
+
+.. cmdoption:: --protocol
+
+    Serve the application using the specified protocol.
+    
+    Can be one of: "http", "scgi", "fcgi", "ajp" (defaults to "http").
+
+.. cmdoption:: --prefix
+
+    Set the supplied value as the CGI ``SCRIPT_NAME`` for the application.
+
+    This option is intended for use when serving Loggerhead behind a
+    reverse proxy, with Loggerhead being "mounted" at a directory below
+    the root.  E.g., if the reverse proxy translates requests for
+    ``http://example.com/loggerhead`` onto the standalone Loggerhead process,
+    that process should be run with ``--prefix=/loggerhead``.
+
+.. cmdoption:: --log-folder=LOG_FOLDER
+
+    The directory in which to place Loggerhead's log files.
+    
+    Defaults to the current directory.
+
+.. cmdoption:: --cache-dir=SQL_CACHE_DIR
+
+    The directory in which to place the SQL cache.
+
+    Defaults to the current directory.
+
+.. cmdoption:: --use-cdn
+   
+    Serve YUI javascript libraries from Yahoo!'s CDN.
+
+.. cmdoption:: --allow-writes
+   
+    Allow writing to the Bazaar server.
+    
+    Setting this option keeps Loggerhead from adding a 'readonly+' prefix
+    to the base URL of the branch.  The only effect of suppressing this prefix
+    is to make visible the display of instructions for checking out the
+    'public_branch' URL for the branch being browsed.
+
+.. cmdoption:: -h, --help
+
+    Print the help message and exit
+
+.. cmdoption:: --version
+
+    Print the software version and exit.
+
+Debugging Options
+-----------------
+
+The following options are only useful when developing / debugging Loggerhead
+itself.
+
+.. cmdoption:: --profile
+   
+    Generate per-request callgrind profile data.
+    
+    Data for each request is written to a file ``%d-stats.callgrind``,
+    where ``%d`` is replaced by the sequence number of the request.
+
+.. cmdoption:: --memory-profile
+
+    Profile the memory usage using the `Dozer
+    <http://pypi.python.org/pypi/Dozer>`_ middleware.
+
+.. cmdoption:: --reload
+
+    Restart the application when any of its python file change.
+    
+    This option should only used for development purposes.

=== added file 'info.py'
--- info.py	1970-01-01 00:00:00 +0000
+++ info.py	2011-11-23 08:33:12 +0000
@@ -0,0 +1,15 @@
+#!/usr/bin/env python
+# API Info for loggerhead
+
+bzr_plugin_name = "loggerhead"
+
+bzr_plugin_version = (1, 18, 1)  # Keep in sync with loggerhead/__init__.py
+
+bzr_compatible_versions = [
+    (1, 17, 0), (1, 18, 0), (2, 0, 0), (2, 1, 0), (2, 2, 0), (2, 3, 0),
+    (2, 4, 0), (2, 5, 0),
+    ]
+
+bzr_minimum_version = bzr_compatible_versions[0]
+
+bzr_maximum_version = bzr_compatible_versions[-1]

=== added directory 'load_test_scripts'
=== added file 'load_test_scripts/multiple_instances.script'
--- load_test_scripts/multiple_instances.script	1970-01-01 00:00:00 +0000
+++ load_test_scripts/multiple_instances.script	2011-02-10 01:00:32 +0000
@@ -0,0 +1,19 @@
+{
+    "comment": "Connect to multiple loggerhead instances and make requests on each. One should be on :8080, one should be on :8081. Multiple threads will place requests on each.",
+    "parameters": {"base_url": "http://localhost"},
+    "requests": [
+        {"thread": "1", "relpath": ":8080/changes"},
+        {"thread": "2", "relpath": ":8080/files"},
+        {"thread": "3", "relpath": ":8081/files"},
+        {"thread": "4", "relpath": ":8081/changes"},
+        {"thread": "1", "relpath": ":8080/changes"},
+        {"thread": "2", "relpath": ":8080/files"},
+        {"thread": "3", "relpath": ":8081/files"},
+        {"thread": "4", "relpath": ":8081/changes"},
+        {"thread": "1", "relpath": ":8080/changes"},
+        {"thread": "2", "relpath": ":8080/files"},
+        {"thread": "3", "relpath": ":8081/files"},
+        {"thread": "4", "relpath": ":8081/changes"}
+    ]
+}
+

=== added file 'load_test_scripts/simple.script'
--- load_test_scripts/simple.script	1970-01-01 00:00:00 +0000
+++ load_test_scripts/simple.script	2011-02-10 01:00:32 +0000
@@ -0,0 +1,9 @@
+{
+    "comment": "A fairly trivial load test script. It just loads the main two pages from a loggerhead install running directly on a branch.",
+    "parameters": {"base_url": "http://localhost:8080"},
+    "requests": [
+        {"relpath": "/changes"},
+        {"relpath": "/files"}
+    ]
+}
+

=== removed file 'loggerhead.conf.example'
--- loggerhead.conf.example	2008-07-22 00:40:03 +0000
+++ loggerhead.conf.example	1970-01-01 00:00:00 +0000
@@ -1,76 +0,0 @@
-
-# use this to change the server's port
-#server.socket_port = 8080
-
-# use this if you're mapping loggerhead within apache via proxy
-#server.webpath = 'http://example.com/branches'
-
-# use this if you want loggerhead to listen to another interface than 127.0.0.1
-# server.socket_host = 'your_hostname_or_ip'
-
-# where's bzrlib?
-# (you only need this if you don't install bzr.)
-#bzrpath = '/Users/robey/code/bzr/bzr'
-
-# if you want a special title on the front page, set it here:
-title = 'branches in loggerhead'
-
-# the access and debug logs can be set up to roll 'daily', 'weekly', or 'never':
-log.roll = 'weekly'
-
-# group branches by project:
-[bazaar]
-
-    # name of this project
-    name = 'bazaar'
-    
-    # for the top browsing page, you can include a description, in raw HTML.
-    #description = 'Bazaar is a <i>revision control system</i>.'
-
-    # should we cache files changed? (recommended for large trees)
-    # this will be a folder containing one file.  if the folder doesn't
-    # exist, it'll be created.
-    #
-    # all branches in this project will share the same cache, which is usually
-    # what you want if they share a repository.  if not, you can also specify
-    # this per-branch.
-    cachepath = '/Users/robey/code/bzr/.bzr/loggerhead-files'
-
-    # each branch is in a sub-folder of the website, and the config
-    # key describes that folder name:
-    [[bzr.dev]]
-
-        # where is the bazaar branch located?
-        folder = '/Users/robey/code/bzr/bzr.dev'
-
-        # (optional) is there a friendier name for the branch that i should
-        # use on the web pages?
-        # you can also define this using the branch's nickname.
-        #branch_name = 'bazaar-dev'
-
-        # (optional) short description for the top-level page
-        # you can also define this using the "description" config key in the
-        # branch's 'branch.conf'.
-        description = "bazaar vcs development branch"
-
-        # (optional) public branch url for the top-level page
-        # you can also define this using the "public_url" config key in the
-        # branch's 'branch.conf'.
-        url = 'http://bazaar-ng.org/bzr/bzr.dev'
-
-
-# here's an example of an auto-published folder:
-[paramiko]
-    name = 'paramiko'
-    cachepath = '/Users/robey/code/paramiko/.bzr/loggerhead-files'
-    
-    # if an auto_publish_folder is given, any bazaar branches discovered in
-    # that folder will be published.  using each branch's 'branch.conf' to
-    # provide optional descriptions and public urls is a nice way to avoid
-    # having to do much configuration here.
-    auto_publish_folder = '/Users/robey/code/paramiko'
-    
-    # if all of the branches under an auto_publish_folder are exported to
-    # the same (base) public url, you can specify that url prefix here, and
-    # each discovered branch's public url will be generated.
-    url_prefix = 'http://www.example.com/code/paramiko'

=== added file 'loggerhead.wsgi'
--- loggerhead.wsgi	1970-01-01 00:00:00 +0000
+++ loggerhead.wsgi	2010-04-18 17:58:04 +0000
@@ -0,0 +1,50 @@
+#!/usr/bin/python -tt
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import sys
+import os
+import pwd
+sys.path.insert(0, os.path.dirname(__file__))
+
+from paste.httpexceptions import HTTPExceptionHandler
+from loggerhead.apps.transport import BranchesFromTransportRoot
+from loggerhead.apps.error import ErrorHandlerApp
+from loggerhead.config import LoggerheadConfig
+from bzrlib import config as bzrconfig
+from paste.deploy.config import PrefixMiddleware
+from bzrlib.plugin import load_plugins
+
+class NotConfiguredError(Exception):
+    pass
+
+
+load_plugins()
+config = LoggerheadConfig()
+prefix = config.get_option('user_prefix') or ''
+# Note we could use LoggerheadConfig here if it didn't fail when a
+# config option is not also a commandline option
+root_dir = bzrconfig.GlobalConfig().get_user_option('http_root_dir')
+if not root_dir:
+    raise NotConfiguredError('You must have a ~/.bazaar/bazaar.conf file for'
+            ' %(user)s with http_root_dir set to the base directory you want'
+            ' to serve bazaar repositories from' %
+            {'user': pwd.getpwuid(os.geteuid()).pw_name})
+prefix = prefix.encode('utf-8', 'ignore')
+root_dir = root_dir.encode('utf-8', 'ignore')
+app = BranchesFromTransportRoot(root_dir, config)
+app = PrefixMiddleware(app, prefix=prefix)
+app = HTTPExceptionHandler(app)
+application = ErrorHandlerApp(app)

=== modified file 'loggerhead/__init__.py'
--- loggerhead/__init__.py	2009-08-20 13:30:37 +0000
+++ loggerhead/__init__.py	2011-11-23 08:33:12 +0000
@@ -22,7 +22,7 @@
 
 import pkg_resources
 
-__version__ = '1.17'
+__version__ = '1.18.1'  # Keep in sync with ../info.py.
 required_bzrlib = (1, 17)
 
 pkg_resources.get_distribution('Paste>=1.6')

=== modified file 'loggerhead/apps/__init__.py'
--- loggerhead/apps/__init__.py	2009-02-19 01:24:20 +0000
+++ loggerhead/apps/__init__.py	2010-04-30 21:30:18 +0000
@@ -4,10 +4,17 @@
 
 from paste import urlparser, fileapp
 
+from loggerhead.util import convert_file_errors
+
 static = os.path.join(
     os.path.dirname(os.path.dirname(__file__)), 'static')
 
-static_app = urlparser.make_static(None, static)
-
-favicon_app = fileapp.FileApp(os.path.join(static, 'images', 'favicon.ico'))
-robots_app = fileapp.FileApp(os.path.join(static, 'robots.txt'))
+# Static things can be cached for half a day, we could probably make this
+# longer, except for just before rollout times.
+static_app = urlparser.make_static(None, static, cache_max_age=12*60*60)
+
+favicon_app = convert_file_errors(fileapp.FileApp(
+    os.path.join(static, 'images', 'favicon.ico')))
+
+robots_app = convert_file_errors(fileapp.FileApp(
+    os.path.join(static, 'robots.txt')))

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2009-10-17 06:35:33 +0000
+++ loggerhead/apps/branch.py	2011-11-23 08:18:35 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009 Canonical Ltd.
+# Copyright (C) 2008-2011 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
+
 """The WSGI application for serving a Bazaar branch."""
 
 import logging
@@ -21,6 +21,8 @@
 import sys
 
 import bzrlib.branch
+import bzrlib.errors
+from bzrlib.hooks import Hooks
 import bzrlib.lru_cache
 
 from paste import request
@@ -28,10 +30,11 @@
 
 from loggerhead.apps import static_app
 from loggerhead.controllers.annotate_ui import AnnotateUI
+from loggerhead.controllers.view_ui import ViewUI
 from loggerhead.controllers.atom_ui import AtomUI
 from loggerhead.controllers.changelog_ui import ChangeLogUI
 from loggerhead.controllers.diff_ui import DiffUI
-from loggerhead.controllers.download_ui import DownloadUI
+from loggerhead.controllers.download_ui import DownloadUI, DownloadTarballUI
 from loggerhead.controllers.filediff_ui import FileDiffUI
 from loggerhead.controllers.inventory_ui import InventoryUI
 from loggerhead.controllers.revision_ui import RevisionUI
@@ -47,7 +50,13 @@
 
     def __init__(self, branch, friendly_name=None, config={},
                  graph_cache=None, branch_link=None, is_root=False,
-                 served_url=_DEFAULT, use_cdn=False):
+                 served_url=_DEFAULT, use_cdn=False, private=False,
+                 export_tarballs=True):
+        """Create branch-publishing WSGI app.
+
+        :param export_tarballs: If true, allow downloading snapshots of revisions
+            as tarballs.
+        """
         self.branch = branch
         self._config = config
         self.friendly_name = friendly_name
@@ -59,25 +68,30 @@
         self.is_root = is_root
         self.served_url = served_url
         self.use_cdn = use_cdn
+        self.private = private
+        self.export_tarballs = export_tarballs
+
+    def public_private_css(self):
+        if self.private:
+            return "private"
+        else:
+            return "public"
 
     def get_history(self):
-        file_cache = None
         revinfo_disk_cache = None
         cache_path = self._config.get('cachepath', None)
         if cache_path is not None:
             # Only import the cache if we're going to use it.
             # This makes sqlite optional
             try:
-                from loggerhead.changecache import (
-                    FileChangeCache, RevInfoDiskCache)
+                from loggerhead.changecache import RevInfoDiskCache
             except ImportError:
                 self.log.debug("Couldn't load python-sqlite,"
                                " continuing without using a cache")
             else:
-                file_cache = FileChangeCache(cache_path)
                 revinfo_disk_cache = RevInfoDiskCache(cache_path)
         return History(
-            self.branch, self.graph_cache, file_cache=file_cache,
+            self.branch, self.graph_cache,
             revinfo_disk_cache=revinfo_disk_cache, cache_key=self.friendly_name)
 
     def url(self, *args, **kw):
@@ -88,10 +102,15 @@
             if v is not None:
                 qs.append('%s=%s' % (k, urllib.quote(v)))
         qs = '&'.join(qs)
-        return request.construct_url(
-            self._environ, script_name=self._url_base,
-            path_info=unicode('/'.join(args)).encode('utf-8'),
-            querystring=qs)
+        path_info = urllib.quote(
+            unicode('/'.join(args)).encode('utf-8'), safe='/~:')
+        if qs:
+            path_info += '?' + qs
+        return self._url_base + path_info
+
+    def absolute_url(self, *args, **kw):
+        rel_url = self.url(*args, **kw)
+        return request.resolve_relative_url(rel_url, self._environ)
 
     def context_url(self, *args, **kw):
         kw = util.get_context(**kw)
@@ -118,6 +137,8 @@
         'files': InventoryUI,
         'revision': RevisionUI,
         'search': SearchUI,
+        'view': ViewUI,
+        'tarball': DownloadTarballUI,
         }
 
     def last_updated(self):
@@ -128,7 +149,7 @@
     def public_branch_url(self):
         return self.branch.get_config().get_user_option('public_branch')
 
-    def app(self, environ, start_response):
+    def lookup_app(self, environ):
         # Check again if the branch is blocked from being served, this is
         # mostly for tests. It's already checked in apps/transport.py
         if self.branch.get_config().get_user_option('http_serve') == 'False':
@@ -145,23 +166,34 @@
             else:
                 # Loggerhead only supports serving .bzr/ on local branches, so
                 # we shouldn't suggest something that won't work.
-                if self.branch.base.startswith('file://'):
+                try:
+                    util.local_path_from_url(self.branch.base)
                     self.served_url = self.url([])
-                else:
+                except bzrlib.errors.InvalidURL:
                     self.served_url = None
+        for hook in self.hooks['controller']:
+            controller = hook(self, environ)
+            if controller is not None:
+                return controller
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(
-                self._url_base + '/changes')
+                self.absolute_url('/changes'))
         if path == 'static':
-            return static_app(environ, start_response)
+            return static_app
+        elif path == '+json':
+            environ['loggerhead.as_json'] = True
+            path = request.path_info_pop(environ)
         cls = self.controllers_dict.get(path)
-        if cls is None:
-            raise httpexceptions.HTTPNotFound()
+        if cls is not None:
+            return cls(self, self.get_history)
+        raise httpexceptions.HTTPNotFound()
+
+    def app(self, environ, start_response):
         self.branch.lock_read()
         try:
             try:
-                c = cls(self, self.get_history)
+                c = self.lookup_app(environ)
                 return c(environ, start_response)
             except:
                 environ['exc_info'] = sys.exc_info()
@@ -169,3 +201,23 @@
                 raise
         finally:
             self.branch.unlock()
+
+
+class BranchWSGIAppHooks(Hooks):
+    """A dictionary mapping hook name to a list of callables for WSGI app branch hooks.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+        """
+        Hooks.__init__(self, "bzrlib.plugins.loggerhead.apps.branch",
+            "BranchWSGIApp.hooks")
+        self.add_hook('controller',
+            "Invoked when looking for the controller to use for a "
+            "branch subpage. The api signature is (branch_app, environ)."
+            "If a hook can provide a controller, it should return one, "
+            "as a standard WSGI app. If it can't provide a controller, "
+            "it should return None", (1, 19))
+
+
+BranchWSGIApp.hooks = BranchWSGIAppHooks()

=== removed file 'loggerhead/apps/config.py'
--- loggerhead/apps/config.py	2009-05-13 13:14:04 +0000
+++ loggerhead/apps/config.py	1970-01-01 00:00:00 +0000
@@ -1,227 +0,0 @@
-"""A server that uses a loggerhead.conf file.
-
-We recreate the branch discovery and url scheme of the old branchview
-code.  It's all a bit horrible really.
-"""
-
-import logging
-import os
-import posixpath
-
-import bzrlib.lru_cache
-
-from bzrlib.util.configobj.configobj import ConfigObj
-
-from paste.request import path_info_pop
-from paste import httpexceptions
-from paste.wsgiwrappers import WSGIResponse
-
-from loggerhead.apps.branch import BranchWSGIApp
-from loggerhead.apps import favicon_app, static_app, robots_app
-from loggerhead.templatefunctions import templatefunctions
-from loggerhead.zptsupport import load_template
-from loggerhead import util
-
-log = logging.getLogger("loggerhead.controllers")
-
-from loggerhead.history import is_branch
-
-
-class Project(object):
-    """A project contains the branches.
-
-    There is some complication because we don't want to hold on to the
-    branches when they are not being browsed."""
-
-    def __init__(self, name, config, root_config, graph_cache):
-        self.name = name
-        self.friendly_name = config.get('name', name)
-        self.description = config.get('description', '')
-        self.long_description = config.get('long_description', '')
-        self._config = config
-        self._root_config = root_config
-        self.graph_cache = graph_cache
-
-        self.view_names = []
-        self.view_data_by_name = {}
-        for view_name in config.sections:
-            log.debug('Configuring (project %s) branch %s...', name, view_name)
-            self._add_view(
-                view_name, config[view_name], config[view_name].get('folder'))
-
-        self._auto_folder = config.get('auto_publish_folder', None)
-        self._auto_list = []
-        if self._auto_folder is not None:
-            self._recheck_auto_folders()
-
-    def _recheck_auto_folders(self):
-        if self._auto_folder is None:
-            return
-        auto_list = []
-        # scan a folder for bazaar branches, and add them automatically
-        for path, folders, filenames in os.walk(self._auto_folder):
-            for folder in folders:
-                folder = os.path.join(path, folder)
-                if is_branch(folder):
-                    auto_list.append(folder)
-        auto_list.sort()
-        if auto_list == self._auto_list:
-            # nothing has changed; do nothing.
-            return
-
-        # rebuild views:
-        self.view_names = []
-        log.debug('Rescanning auto-folder for project %s ...', self.name)
-        for folder in auto_list:
-            view_name = os.path.basename(folder)
-            log.debug('Auto-configuring (project %s) branch %s...',
-                      self.name,
-                      view_name)
-            self._add_view(view_name, ConfigObj(), folder)
-        self._auto_list = auto_list
-
-    def _get_branch_url(self, view, view_config, folder):
-        url = view_config.get('url', None)
-        if url is not None:
-            return url
-        url = self._config.get('url_prefix', None)
-        if url is not None:
-            return posixpath.join(url, folder) + '/'
-        return None
-
-    def _get_description(self, view, view_config, history):
-        description = view_config.get('description', None)
-        if description is not None:
-            return description
-        description = history._branch.get_config().get_user_option(
-                          'description')
-        return description
-
-    def _add_view(self, view_name, view_config, folder):
-        b = bzrlib.branch.Branch.open(folder)
-        view = BranchWSGIApp(b, view_name, view_config, self.graph_cache)
-        b.lock_read()
-        try:
-            history = view.get_history()
-            friendly_name = view_config.get('branch_name', None)
-            if friendly_name is None:
-                friendly_name = history.get_config().get_nickname()
-                if friendly_name is None:
-                    friendly_name = view_name
-            branch_url = self._get_branch_url(view, view_config, view_name)
-            description = self._get_description(view, view_config, history)
-            self.view_data_by_name[view_name] = {
-                'branch_path': folder,
-                'config': view_config,
-                'description': description,
-                'friendly_name': friendly_name,
-                'graph_cache': self.graph_cache,
-                'name': view_name,
-                'served_url': branch_url,
-                }
-            self.view_names.append(view_name)
-        finally:
-            b.unlock()
-
-    def view_named(self, name):
-        view_data = self.view_data_by_name.get(name)
-        if view_data is None:
-            return None
-        view_data = view_data.copy()
-        branch_path = view_data.pop('branch_path')
-        description = view_data.pop('description')
-        name = view_data.pop('name')
-        b = bzrlib.branch.Branch.open(branch_path)
-        b.lock_read()
-        view = BranchWSGIApp(b, **view_data)
-        view.description = description
-        view.name = name
-        return view
-
-    def call(self, environ, start_response):
-        segment = path_info_pop(environ)
-        if not segment:
-            raise httpexceptions.HTTPNotFound()
-        else:
-            view = self.view_named(segment)
-            if view is None:
-                raise httpexceptions.HTTPNotFound()
-            try:
-                return view.app(environ, start_response)
-            finally:
-                view.branch.unlock()
-
-
-class Root(object):
-    """The root of the server -- renders as the browse view,
-    dispatches to Project above for each 'project'."""
-
-    def __init__(self, config):
-        self.projects = []
-        self.config = config
-        self.projects_by_name = {}
-        graph_cache = bzrlib.lru_cache.LRUCache()
-        for project_name in self.config.sections:
-            project = Project(project_name, self.config[project_name],
-                              self.config, graph_cache)
-            self.projects.append(project)
-            self.projects_by_name[project_name] = project
-
-    def browse(self, response):
-        # This is insanely complicated because we want to open and
-        # lock all the branches, render the view and then unlock the
-        # branches again.
-        for p in self.projects:
-            p._recheck_auto_folders()
-
-        class branch(object):
-
-            @staticmethod
-            def static_url(path):
-                return self._static_url_base + path
-        views_by_project = {}
-        all_views = []
-        try:
-            for p in self.projects:
-                views_by_project[p] = []
-                for vn in p.view_names:
-                    v = p.view_named(vn)
-                    all_views.append(v)
-                    views_by_project[p].append(v)
-            vals = {
-                'projects': self.projects,
-                'util': util,
-                'title': self.config.get('title', None),
-                'branch': branch,
-                'views_by_project': views_by_project,
-            }
-            vals.update(templatefunctions)
-            response.headers['Content-Type'] = 'text/html'
-            template = load_template('loggerhead.templates.browse')
-            template.expand_into(response, **vals)
-        finally:
-            for v in all_views:
-                v.branch.unlock()
-
-    def __call__(self, environ, start_response):
-        self._static_url_base = environ['loggerhead.static.url'] = \
-                                environ['SCRIPT_NAME']
-        segment = path_info_pop(environ)
-        if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
-        elif segment == '':
-            response = WSGIResponse()
-            self.browse(response)
-            return response(environ, start_response)
-        elif segment == 'robots.txt':
-            return robots_app(environ, start_response)
-        elif segment == 'static':
-            return static_app(environ, start_response)
-        elif segment == 'favicon.ico':
-            return favicon_app(environ, start_response)
-        else:
-            project = self.projects_by_name.get(segment)
-            if project is None:
-                raise httpexceptions.HTTPNotFound()
-            return project.call(environ, start_response)

=== added file 'loggerhead/apps/http_head.py'
--- loggerhead/apps/http_head.py	1970-01-01 00:00:00 +0000
+++ loggerhead/apps/http_head.py	2011-02-10 02:33:15 +0000
@@ -0,0 +1,62 @@
+# Copyright (C) 2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+"""WSGI apps tend to return body content as part of a HEAD request.
+
+We should definitely not do that.
+"""
+
+class HeadMiddleware(object):
+    """When we get a HEAD request, we should not return body content.
+
+    WSGI defaults to just generating everything, and not paying attention to
+    whether it is a GET or a HEAD request. It does that because of potential
+    issues getting the Headers correct.
+
+    This middleware works by just omitting the body if the request method is
+    HEAD.
+    """
+
+    def __init__(self, app):
+        self._wrapped_app = app
+        self._real_environ = None
+        self._real_start_response = None
+        self._real_writer = None
+
+    def noop_write(self, chunk):
+        """We intentionally ignore all body content that is returned."""
+        pass
+
+    def start_response(self, status, response_headers, exc_info=None):
+        if exc_info is None:
+            self._real_writer = self._real_start_response(status,
+                response_headers)
+        else:
+            self._real_writer = self._real_start_response(status,
+                response_headers, exc_info)
+        return self.noop_write
+
+    def __call__(self, environ, start_response):
+        self._real_environ = environ
+        self._real_start_response = start_response
+        if environ.get('REQUEST_METHOD', 'GET') == 'HEAD':
+            result = self._wrapped_app(environ, self.start_response)
+            for chunk in result:
+                pass
+        else:
+            result = self._wrapped_app(environ, start_response)
+            for chunk in result:
+                yield chunk

=== modified file 'loggerhead/apps/transport.py'
--- loggerhead/apps/transport.py	2009-10-17 06:35:33 +0000
+++ loggerhead/apps/transport.py	2011-03-16 14:43:36 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009 Canonical Ltd.
+# Copyright (C) 2008-2011 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 from bzrlib import branch, errors, lru_cache, urlutils
 from bzrlib.config import LocationConfig
+from bzrlib.smart import request
 from bzrlib.transport import get_transport
 from bzrlib.transport.http import wsgi
 
@@ -27,10 +28,12 @@
 from paste import httpexceptions
 from paste import urlparser
 
+from loggerhead import util
 from loggerhead.apps.branch import BranchWSGIApp
-from loggerhead.apps import favicon_app, static_app
+from loggerhead.apps import favicon_app, robots_app, static_app
 from loggerhead.controllers.directory_ui import DirectoryUI
 
+# TODO: Use bzrlib.ui.bool_from_string(), added in bzr 1.18
 _bools = {
     'yes': True, 'no': False,
     'on': True, 'off': False,
@@ -66,8 +69,8 @@
     def app_for_non_branch(self, environ):
         segment = path_info_pop(environ)
         if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
+            raise httpexceptions.HTTPMovedPermanently.relative_redirect(
+                environ['SCRIPT_NAME'] + '/', environ)
         elif segment == '':
             if self.name:
                 name = self.name
@@ -85,17 +88,15 @@
 
     def app_for_bazaar_data(self, relpath):
         if relpath == '/.bzr/smart':
-            wsgi_app = wsgi.SmartWSGIApp(self.transport)
-            return wsgi.RelpathSetter(wsgi_app, '', 'PATH_INFO')
+            root_transport = get_transport_for_thread(self.root.base)
+            wsgi_app = wsgi.SmartWSGIApp(root_transport)
+            return wsgi.RelpathSetter(wsgi_app, '', 'loggerhead.path_info')
         else:
             # TODO: Use something here that uses the transport API
             # rather than relying on the local filesystem API.
             base = self.transport.base
-            readonly_prefix = 'readonly+'
-            if base.startswith(readonly_prefix):
-                base = base[len(readonly_prefix):]
             try:
-                path = urlutils.local_path_from_url(base)
+                path = util.local_path_from_url(base)
             except errors.InvalidURL:
                 raise httpexceptions.HTTPNotFound()
             else:
@@ -130,13 +131,13 @@
 _transport_store = threading.local()
 
 def get_transport_for_thread(base):
-    """ """
     thread_transports = getattr(_transport_store, 'transports', None)
     if thread_transports is None:
         thread_transports = _transport_store.transports = {}
     if base in thread_transports:
         return thread_transports[base]
     transport = get_transport(base)
+    thread_transports[base] = transport
     return transport
 
 
@@ -149,12 +150,15 @@
 
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
+        environ['loggerhead.path_info'] = environ['PATH_INFO']
         if environ['PATH_INFO'].startswith('/static/'):
             segment = path_info_pop(environ)
             assert segment == 'static'
             return static_app(environ, start_response)
         elif environ['PATH_INFO'] == '/favicon.ico':
             return favicon_app(environ, start_response)
+        elif environ['PATH_INFO'] == '/robots.txt':
+            return robots_app(environ, start_response)
         else:
             transport = get_transport_for_thread(self.base)
             return BranchesFromTransportServer(
@@ -171,6 +175,7 @@
 
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
+        environ['loggerhead.path_info'] = environ['PATH_INFO']
         path_info = environ['PATH_INFO']
         if path_info.startswith('/static/'):
             segment = path_info_pop(environ)
@@ -178,6 +183,8 @@
             return static_app(environ, start_response)
         elif path_info == '/favicon.ico':
             return favicon_app(environ, start_response)
+        elif environ['PATH_INFO'] == '/robots.txt':
+            return robots_app(environ, start_response)
         else:
             transport = get_transport_for_thread(self.base)
             # segments starting with ~ are user branches

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2009-05-02 14:01:05 +0000
+++ loggerhead/changecache.py	2011-08-17 11:04:00 +0000
@@ -99,24 +99,6 @@
             pass
 
 
-class FileChangeCache(object):
-
-    def __init__(self, cache_path):
-
-        if not os.path.exists(cache_path):
-            os.mkdir(cache_path)
-
-        self._changes_filename = os.path.join(cache_path, 'filechanges.sql')
-
-    def get_file_changes(self, entry):
-        cache = FakeShelf(self._changes_filename)
-        changes = cache.get(entry.revid)
-        if changes is None:
-            changes = self.history.get_file_changes_uncached(entry)
-            cache.add(entry.revid, changes)
-        return changes
-
-
 class RevInfoDiskCache(object):
     """Like `RevInfoMemoryCache` but backed in a sqlite DB."""
 

=== modified file 'loggerhead/config.py'
--- loggerhead/config.py	2009-10-17 08:47:38 +0000
+++ loggerhead/config.py	2011-07-07 19:09:11 +0000
@@ -36,6 +36,7 @@
         use_cdn=False,
         sql_dir=None,
         allow_writes=False,
+        export_tarballs=True,
         )
     parser.add_option("--user-dirs", action="store_true",
                       help="Serve user directories as ~user.")
@@ -46,6 +47,15 @@
                             "(defaults to 8080)."))
     parser.add_option("--host", dest="user_host",
                       help="Host Loggerhead should listen on.")
+    parser.add_option("--protocol", dest="protocol",
+                      help=("Protocol to use: http, scgi, fcgi, ajp"
+                           "(defaults to http)."))
+    parser.add_option("--log-level", default=None, action='callback',
+                      callback=_optparse_level_to_int_level,
+                      type="string",
+                      help="Set the verbosity of logging. Can either"
+                           " be set to a numeric or string"
+                           " (eg, 10=debug, 30=warning)")
     parser.add_option("--memory-profile", action="store_true",
                       help="Profile the memory usage using Dozer.")
     parser.add_option("--prefix", dest="user_prefix",
@@ -66,9 +76,34 @@
                       help="The directory to place the SQL cache in")
     parser.add_option("--allow-writes", action="store_true",
                       help="Allow writing to the Bazaar server.")
+    parser.add_option("--export-tarballs", action="store_true",
+                      help="Allow exporting revisions to tarballs.")
     return parser
 
 
+_log_levels = {
+    'debug': 10,
+    'info': 20,
+    'warning': 30,
+    'error': 40,
+    'critical': 50,
+}
+
+def _optparse_level_to_int_level(option, opt_str, value, parser):
+    parser.values.log_level = _level_to_int_level(value)
+
+
+def _level_to_int_level(value):
+    """Convert a string level to an integer value."""
+    if value is None:
+        return None
+    try:
+        return int(value)
+    except ValueError:
+        pass
+    return _log_levels[value.lower()]
+
+
 class LoggerheadConfig(object):
     """A configuration object."""
 
@@ -84,7 +119,7 @@
         self.SQL_DIR = sql_dir
 
     def get_option(self, option):
-        """Get the value for the config option, either 
+        """Get the value for the config option, either
         from ~/.bazaar/bazaar.conf or from the command line.
         All loggerhead-specific settings start with 'http_'
         """
@@ -96,6 +131,10 @@
         else:
             return cmd_config
 
+    def get_log_level(self):
+        opt = self.get_option('log_level')
+        return _level_to_int_level(opt)
+
     def get_arg(self, index):
         """Get an arg from the arg list."""
         return self._args[index]

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2009-10-17 08:59:33 +0000
+++ loggerhead/controllers/__init__.py	2011-07-07 19:09:11 +0000
@@ -17,8 +17,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import bzrlib.errors
+import simplejson
 import time
 
+from paste.httpexceptions import HTTPNotFound, HTTPSeeOther
 from paste.request import path_info_pop, parse_querystring
 
 from loggerhead import util
@@ -51,6 +54,7 @@
 class TemplatedBranchView(object):
 
     template_path = None
+    supports_json = False
 
     def __init__(self, branch, history_callable):
         self._branch = branch
@@ -65,8 +69,7 @@
         self.__history = self._history_callable()
         return self.__history
 
-    def __call__(self, environ, start_response):
-        z = time.time()
+    def parse_args(self, environ):
         kwargs = dict(parse_querystring(environ))
         util.set_context(kwargs)
         args = []
@@ -80,30 +83,48 @@
         if len(args) > 1:
             path = unicode('/'.join(args[1:]), 'utf-8')
         self.args = args
+        self.kwargs = kwargs
+        return path
 
-        vals = {
+    def add_template_values(self, values):
+        values.update({
             'static_url': self._branch.static_url,
             'branch': self._branch,
             'util': util,
             'url': self._branch.context_url,
-        }
-        vals.update(templatefunctions)
+        })
+        values.update(templatefunctions)
+
+    def __call__(self, environ, start_response):
+        z = time.time()
+        if environ.get('loggerhead.as_json') and not self.supports_json:
+            raise HTTPNotFound
+        path = self.parse_args(environ)
         headers = {}
-
-        vals.update(self.get_values(path, kwargs, headers))
-
-        self.log.info('Getting information for %s: %r secs' % (
+        values = self.get_values(path, self.kwargs, headers)
+
+        self.log.info('Getting information for %s: %.3f secs' % (
             self.__class__.__name__, time.time() - z))
-        if 'Content-Type' not in headers:
+        if environ.get('loggerhead.as_json'):
+            headers['Content-Type'] = 'application/json'
+        elif 'Content-Type' not in headers:
             headers['Content-Type'] = 'text/html'
         writer = start_response("200 OK", headers.items())
-        template = load_template(self.template_path)
+        if environ.get('REQUEST_METHOD') == 'HEAD':
+            # No content for a HEAD request
+            return []
         z = time.time()
         w = BufferingWriter(writer, 8192)
-        template.expand_into(w, **vals)
+        if environ.get('loggerhead.as_json'):
+            w.write(simplejson.dumps(values,
+                default=util.convert_to_json_ready))
+        else:
+            self.add_template_values(values)
+            template = load_template(self.template_path)
+            template.expand_into(w, **values)
         w.flush()
         self.log.info(
-            'Rendering %s: %r secs, %s bytes' % (
+            'Rendering %s: %.3f secs, %s bytes' % (
                 self.__class__.__name__, time.time() - z, w.bytes))
         return []
 
@@ -112,6 +133,10 @@
         if h is None:
             return None
         if len(self.args) > 0 and self.args != ['']:
-            return h.fix_revid(self.args[0])
+            try:
+                revid = h.fix_revid(self.args[0])
+            except bzrlib.errors.NoSuchRevision:
+                raise HTTPNotFound;
         else:
-            return h.last_revid
+            revid = h.last_revid
+        return revid

=== added file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/controllers/annotate_ui.py	2011-08-09 14:06:49 +0000
@@ -0,0 +1,78 @@
+#
+# Copyright (C) 2010 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+import itertools
+
+from loggerhead.controllers.view_ui import ViewUI
+from loggerhead import util
+
+class AnnotateUI(ViewUI):
+
+    def annotate_file(self, info):
+        file_id = info['file_id']
+        revid = info['change'].revid
+        
+        tree = self.tree_for(file_id, revid)
+        
+        change_cache = {}
+        last_line_revid = None
+        last_lineno = None
+        message = ""
+
+        revisions = {}
+
+        lineno = 0
+        for (line_revid, text), lineno in zip(tree.annotate_iter(file_id), itertools.count(1)):
+            if line_revid != last_line_revid:
+                last_line_revid = line_revid
+
+                change = change_cache.get(line_revid, None)
+                if change is None:
+                    change = self._history.get_changes([line_revid])[0]
+                    change_cache[line_revid] = change
+
+                try:
+                    message = change.comment.splitlines()[0]
+                except IndexError:
+                    # Comment not present for this revision
+                    message = ""
+
+                if last_lineno:
+                    # The revspan is of lines between the last revision and this one.
+                    # We set the one for the previous revision when we're creating the current revision.
+                    revisions[last_lineno].revspan = lineno - last_lineno
+
+                revisions[lineno] = util.Container(change=change, message=message)
+
+                last_lineno = lineno
+                last_line_revid = line_revid
+
+        # Zero-size file. Return empty revisions.
+        if last_lineno is None:
+            return revisions
+
+        # We never set a revspan for the last revision during the loop above, so set it here.
+        revisions[last_lineno].revspan = lineno - last_lineno + 1
+
+        return revisions
+            
+    def get_values(self, path, kwargs, headers):
+        values = super(AnnotateUI, self).get_values(path, kwargs, headers)
+        values['annotated'] = self.annotate_file(values)
+        
+        return values

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2009-06-25 03:35:43 +0000
+++ loggerhead/controllers/changelog_ui.py	2011-03-12 13:25:52 +0000
@@ -20,10 +20,7 @@
 
 import urllib
 
-try:
-    import simplejson
-except ImportError:
-    import json as simplejson
+import simplejson
 
 from paste.httpexceptions import HTTPServerError
 
@@ -50,7 +47,8 @@
 
         try:
             revid, start_revid, revid_list = history.get_view(
-                revid, start_revid, filter_file_id, query)
+                revid, start_revid, filter_file_id, query,
+                extra_rev_count=pagesize+1)
             util.set_context(kwargs)
 
             if (query is not None) and (len(revid_list) == 0):
@@ -106,7 +104,7 @@
             'navigation': navigation,
             'filter_file_id': filter_file_id,
             'start_revid': start_revid,
-            'viewing_from': (orig_start_revid is not None) and 
+            'viewing_from': (orig_start_revid is not None) and
                             (orig_start_revid != history.last_revid),
             'query': query,
             'search_failed': search_failed,

=== modified file 'loggerhead/controllers/diff_ui.py'
--- loggerhead/controllers/diff_ui.py	2009-10-17 08:59:33 +0000
+++ loggerhead/controllers/diff_ui.py	2011-06-28 16:51:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008  Canonical Ltd.
+# Copyright (C) 2008-2011 Canonical Ltd.
 #                     (Authored by Martin Albisetti <argentina@gmail.com>)
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,17 +17,15 @@
 #
 
 from cStringIO import StringIO
-import logging
 import time
 
 from paste.request import path_info_pop
 
 from bzrlib.diff import show_diff_trees
+from bzrlib.revision import NULL_REVISION
 
 from loggerhead.controllers import TemplatedBranchView
 
-log = logging.getLogger("loggerhead.controllers")
-
 
 class DiffUI(TemplatedBranchView):
     """Class to output a diff for a single file or revisions."""
@@ -49,8 +47,10 @@
         revid_from = self._history.fix_revid(revid_from)
         change = self._history.get_changes([revid_from])[0]
 
-        if len(args) is 2:
+        if len(args) == 2:
             revid_to = self._history.fix_revid(args[1])
+        elif len(change.parents) == 0:
+            revid_to = NULL_REVISION
         else:
             revid_to = change.parents[0].revid
 
@@ -72,7 +72,7 @@
         filename = '%s_%s.diff' % (revno1, revno2)
         headers = [
             ('Content-Type', 'application/octet-stream'),
-            ('Content-Length', len(content)),
+            ('Content-Length', str(len(content))),
             ('Content-Disposition', 'attachment; filename=%s' % (filename,)),
             ]
         start_response('200 OK', headers)

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2009-10-17 06:35:33 +0000
+++ loggerhead/controllers/directory_ui.py	2012-02-02 07:42:24 +0000
@@ -35,7 +35,7 @@
         if branch is not None:
             # If a branch is empty, bzr raises an exception when trying this
             try:
-                self.last_change = datetime.datetime.fromtimestamp(
+                self.last_change = datetime.datetime.utcfromtimestamp(
                     branch.repository.get_revision(
                         branch.last_revision()).timestamp)
             except:

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2009-10-17 08:59:33 +0000
+++ loggerhead/controllers/download_ui.py	2011-11-25 01:41:07 +0000
@@ -19,51 +19,89 @@
 
 import logging
 import mimetypes
+import os
 import urllib
 
 from paste import httpexceptions
 from paste.request import path_info_pop
 
 from loggerhead.controllers import TemplatedBranchView
+from loggerhead.exporter import export_archive
 
 log = logging.getLogger("loggerhead.controllers")
 
 
 class DownloadUI (TemplatedBranchView):
 
-    def __call__(self, environ, start_response):
-        # /download/<rev_id>/<file_id>/[filename]
-
-        h = self._history
-
+    def encode_filename(self, filename):
+
+        return urllib.quote(filename.encode('utf-8'))
+
+    def get_args(self, environ):
         args = []
         while True:
             arg = path_info_pop(environ)
             if arg is None:
                 break
             args.append(arg)
+        return args
 
+    def __call__(self, environ, start_response):
+        # /download/<rev_id>/<file_id>/[filename]
+        h = self._history
+        args = self.get_args(environ)
         if len(args) < 2:
-            raise httpexceptions.HTTPMovedPermanently(self._branch.url(
-                      '../changes'))
-
+            raise httpexceptions.HTTPMovedPermanently(
+                self._branch.absolute_url('/changes'))
         revid = h.fix_revid(args[0])
         file_id = args[1]
         path, filename, content = h.get_file(file_id, revid)
         mime_type, encoding = mimetypes.guess_type(filename)
         if mime_type is None:
             mime_type = 'application/octet-stream'
-
         self.log.info('/download %s @ %s (%d bytes)',
                       path,
                       h.get_revno(revid),
                       len(content))
-        encoded_filename = urllib.quote(filename.encode('utf-8'))
+        encoded_filename = self.encode_filename(filename)
         headers = [
             ('Content-Type', mime_type),
-            ('Content-Length', len(content)),
+            ('Content-Length', str(len(content))),
             ('Content-Disposition',
-             'attachment; filename*=utf-8\'\'%s' % (encoded_filename,)),
+             "attachment; filename*=utf-8''%s" % (encoded_filename,)),
             ]
         start_response('200 OK', headers)
         return [content]
+
+
+class DownloadTarballUI(DownloadUI):
+
+    def __call__(self, environ, start_response):
+        """Stream a tarball from a bazaar branch."""
+        # Tried to re-use code from downloadui, not very successful
+        if not self._branch.export_tarballs:
+            raise httpexceptions.HTTPForbidden(
+                "Tarball downloads are not allowed")
+        archive_format = "tgz"
+        history = self._history
+        self.args = self.get_args(environ)
+        if len(self.args):
+            revid = history.fix_revid(self.args[0])
+            version_part = '-r' + self.args[0]
+        else:
+            revid = self.get_revid()
+            version_part = ''
+        # XXX: Perhaps some better suggestion based on the URL or path?
+        #
+        # TODO: Perhaps set the tarball suggested mtime to the revision
+        # mtime.
+        root = self._branch.friendly_name or 'branch'
+        encoded_filename = self.encode_filename(
+            root + version_part + '.' + archive_format)
+        headers = [
+            ('Content-Type', 'application/octet-stream'),
+            ('Content-Disposition',
+                "attachment; filename*=utf-8''%s" % (encoded_filename,)),
+            ]
+        start_response('200 OK', headers)
+        return export_archive(history, root, revid, archive_format)

=== modified file 'loggerhead/controllers/filediff_ui.py'
--- loggerhead/controllers/filediff_ui.py	2009-07-07 23:52:24 +0000
+++ loggerhead/controllers/filediff_ui.py	2011-06-27 15:05:06 +0000
@@ -77,6 +77,7 @@
 class FileDiffUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.filediff'
+    supports_json = True
 
     def get_values(self, path, kwargs, headers):
         revid = urllib.unquote(self.args[0])
@@ -87,6 +88,5 @@
             self._history._branch.repository, file_id, compare_revid, revid)
 
         return {
-            'util': util,
             'chunks': chunks,
         }

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2009-03-19 00:44:05 +0000
+++ loggerhead/controllers/inventory_ui.py	2011-06-27 16:30:57 +0000
@@ -22,7 +22,7 @@
 import posixpath
 import urllib
 
-from paste.httpexceptions import HTTPNotFound
+from paste.httpexceptions import HTTPNotFound, HTTPMovedPermanently
 
 from bzrlib import errors
 from bzrlib.revision import is_null as is_null_rev
@@ -31,8 +31,6 @@
 from loggerhead.controllers import TemplatedBranchView
 
 
-log = logging.getLogger("loggerhead.controllers")
-
 
 def dirname(path):
     if path is not None:
@@ -44,8 +42,9 @@
 class InventoryUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.inventory'
+    supports_json = True
 
-    def get_filelist(self, inv, path, sort_type):
+    def get_filelist(self, inv, path, sort_type, revno_url):
         """
         return the list of all files (and their attributes) within a given
         path subtree.
@@ -58,6 +57,9 @@
         dir_ie = inv[file_id]
         file_list = []
 
+        if dir_ie.kind != 'directory':
+            raise HTTPMovedPermanently(self._branch.context_url(['/view', revno_url, path]))
+
         revid_set = set()
 
         for filename, entry in dir_ie.children.iteritems():
@@ -77,6 +79,11 @@
                 absolutepath = path + '/' + pathname
             revid = entry.revision
 
+            # TODO: For the JSON rendering, this inlines the "change" aka
+            # revision information attached to each file. Consider either
+            # pulling this out as a separate changes dict, or possibly just
+            # including the revision id and having a separate request to get
+            # back the revision info.
             file = util.Container(
                 filename=filename, executable=entry.executable,
                 kind=entry.kind, absolutepath=absolutepath,
@@ -109,9 +116,6 @@
         start_revid = kwargs.get('start_revid', None)
         sort_type = kwargs.get('sort', 'filename')
 
-        # no navbar for revisions
-        navigation = util.Container()
-
         if path is not None:
             path = path.rstrip('/')
             file_id = rev_tree.path2id(path)
@@ -132,14 +136,7 @@
         else:
             updir = dirname(path)
 
-        # Directory Breadcrumbs
-        directory_breadcrumbs = util.directory_breadcrumbs(
-                self._branch.friendly_name,
-                self._branch.is_root,
-                'files')
-
         if not is_null_rev(revid):
-
             change = history.get_changes([ revid ])[0]
             # If we're looking at the tip, use head: in the URL instead
             if revid == branch.last_revision():
@@ -147,32 +144,49 @@
             else:
                 revno_url = history.get_revno(revid)
             history.add_branch_nicks(change)
+            filelist = self.get_filelist(rev_tree.inventory, path, sort_type, revno_url)
 
-            # Create breadcrumb trail for the path within the branch
-            branch_breadcrumbs = util.branch_breadcrumbs(path, rev_tree, 'files')
-            filelist = self.get_filelist(rev_tree.inventory, path, sort_type)
         else:
             start_revid = None
             change = None
             path = "/"
             updir = None
             revno_url = 'head:'
-            branch_breadcrumbs = []
             filelist = []
 
         return {
-            'branch': self._branch,
-            'util': util,
             'revid': revid,
             'revno_url': revno_url,
             'change': change,
             'path': path,
             'updir': updir,
             'filelist': filelist,
-            'navigation': navigation,
-            'url': self._branch.context_url,
             'start_revid': start_revid,
+        }
+
+    def add_template_values(self, values):
+        super(InventoryUI, self).add_template_values(values)
+        # Directory Breadcrumbs
+        directory_breadcrumbs = util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'files')
+
+        path = values['path']
+        revid = values['revid']
+        # no navbar for revisions
+        navigation = util.Container()
+
+        if is_null_rev(revid):
+            branch_breadcrumbs = []
+        else:
+            # Create breadcrumb trail for the path within the branch
+            branch = self._history._branch
+            rev_tree = branch.repository.revision_tree(revid)
+            branch_breadcrumbs = util.branch_breadcrumbs(path, rev_tree, 'files')
+        values.update({
             'fileview_active': True,
             'directory_breadcrumbs': directory_breadcrumbs,
             'branch_breadcrumbs': branch_breadcrumbs,
-        }
+            'navigation': navigation,
+        })

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2009-08-04 04:08:03 +0000
+++ loggerhead/controllers/revision_ui.py	2011-11-23 08:18:35 +0000
@@ -17,10 +17,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-try:
-    import simplejson
-except ImportError:
-    import json as simplejson
+import simplejson
 import urllib
 
 from paste.httpexceptions import HTTPServerError
@@ -39,6 +36,7 @@
 class RevisionUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.revision'
+    supports_json = True
 
     def get_values(self, path, kwargs, headers):
         h = self._history
@@ -47,9 +45,10 @@
         filter_file_id = kwargs.get('filter_file_id', None)
         start_revid = h.fix_revid(kwargs.get('start_revid', None))
         query = kwargs.get('q', None)
-        remember = h.fix_revid(kwargs.get('remember', None))
         compare_revid = h.fix_revid(kwargs.get('compare_revid', None))
 
+        # TODO: This try/except looks to date before real exception handling
+        # and should be removed
         try:
             revid, start_revid, revid_list = h.get_view(revid,
                                                         start_revid,
@@ -58,33 +57,75 @@
         except:
             self.log.exception('Exception fetching changes')
             raise HTTPServerError('Could not fetch changes')
-
+        # XXX: Some concern about namespace collisions. These are only stored
+        # here so they can be expanded into the template later. Should probably
+        # be stored in a specific dict/etc.
+        self.revid_list = revid_list
+        self.compare_revid = compare_revid
+        self.path = path
+        kwargs['start_revid'] = start_revid
+
+        change = h.get_changes([revid])[0]
+
+        if compare_revid is None:
+            file_changes = h.get_file_changes(change)
+        else:
+            file_changes = h.file_changes_for_revision_ids(
+                compare_revid, change.revid)
+
+        h.add_branch_nicks(change)
+
+        if '.' in change.revno:
+            # Walk "up" though the merge-sorted graph until we find a
+            # revision with merge depth 0: this is the revision that merged
+            # this one to mainline.
+            ri = self._history._rev_info
+            i = self._history._rev_indices[change.revid]
+            while ri[i][0][2] > 0:
+                i -= 1
+            merged_in = ri[i][0][3]
+        else:
+            merged_in = None
+
+        return {
+            'revid': revid,
+            'change': change,
+            'file_changes': file_changes,
+            'merged_in': merged_in,
+        }
+
+    def add_template_values(self, values):
+        super(RevisionUI, self).add_template_values(values)
+        remember = self._history.fix_revid(self.kwargs.get('remember', None))
+        query = self.kwargs.get('q', None)
+        filter_file_id = self.kwargs.get('filter_file_id', None)
+        start_revid = self.kwargs['start_revid']
         navigation = util.Container(
-            revid_list=revid_list, revid=revid, start_revid=start_revid,
+            revid_list=self.revid_list, revid=values['revid'],
+            start_revid=start_revid,
             filter_file_id=filter_file_id, pagesize=1,
-            scan_url='/revision', branch=self._branch, feed=True, history=h)
+            scan_url='/revision', branch=self._branch, feed=True,
+            history=self._history)
         if query is not None:
             navigation.query = query
         util.fill_in_navigation(navigation)
-
-        change = h.get_changes([revid])[0]
-
-        if compare_revid is None:
-            file_changes = h.get_file_changes(change)
-        else:
-            file_changes = h.file_changes_for_revision_ids(
-                compare_revid, change.revid)
-
+        path = self.path
         if path in ('', '/'):
             path = None
 
+
+        file_changes = values['file_changes']
         link_data = {}
         path_to_id = {}
         if path:
-            item = [x for x in file_changes.text_changes if x.filename == path][0]
-            diff_chunks = diff_chunks_for_file(
-                self._history._branch.repository, item.file_id,
-                item.old_revision, item.new_revision)
+            items = [x for x in file_changes.text_changes if x.filename == path]
+            if len(items) > 0:
+                item = items[0]
+                diff_chunks = diff_chunks_for_file(
+                    self._history._branch.repository, item.file_id,
+                    item.old_revision, item.new_revision)
+            else:
+                diff_chunks = None
         else:
             diff_chunks = None
             for i, item in enumerate(file_changes.text_changes):
@@ -93,46 +134,27 @@
                     dq(item.new_revision), dq(item.old_revision), dq(item.file_id))
                 path_to_id[item.filename] = 'diff-' + str(i)
 
-        h.add_branch_nicks(change)
-
-        if '.' in change.revno:
-            # Walk "up" though the merge-sorted graph until we find a
-            # revision with merge depth 0: this is the revision that merged
-            # this one to mainline.
-            ri = self._history._rev_info
-            i = self._history._rev_indices[change.revid]
-            while ri[i][0][2] > 0:
-                i -= 1
-            merged_in = ri[i][0][3]
-        else:
-            merged_in = None
-
         # Directory Breadcrumbs
         directory_breadcrumbs = (
             util.directory_breadcrumbs(
                 self._branch.friendly_name,
                 self._branch.is_root,
                 'changes'))
+        can_export = self._branch.export_tarballs
 
-        return {
-            'branch': self._branch,
-            'revid': revid,
-            'change': change,
-            'file_changes': file_changes,
-            'diff_chunks': diff_chunks,
+        values.update({
+            'history': self._history,
             'link_data': simplejson.dumps(link_data),
-            'specific_path': path,
             'json_specific_path': simplejson.dumps(path),
             'path_to_id': simplejson.dumps(path_to_id),
-            'start_revid': start_revid,
+            'directory_breadcrumbs': directory_breadcrumbs,
+            'navigation': navigation,
+            'remember': remember,
+            'compare_revid': self.compare_revid,
             'filter_file_id': filter_file_id,
-            'util': util,
-            'history': h,
-            'merged_in': merged_in,
-            'navigation': navigation,
+            'diff_chunks': diff_chunks,
             'query': query,
-            'remember': remember,
-            'compare_revid': compare_revid,
-            'url': self._branch.context_url,
-            'directory_breadcrumbs': directory_breadcrumbs,
-        }
+            'can_export': can_export,
+            'specific_path': path,
+            'start_revid': start_revid,
+        })

=== modified file 'loggerhead/controllers/revlog_ui.py'
--- loggerhead/controllers/revlog_ui.py	2009-03-19 00:44:05 +0000
+++ loggerhead/controllers/revlog_ui.py	2011-06-27 15:05:06 +0000
@@ -1,12 +1,12 @@
 import urllib
 
-from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
 
 
 class RevLogUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.revlog'
+    supports_json = True
 
     def get_values(self, path, kwargs, headers):
         history = self._history
@@ -18,10 +18,7 @@
         history.add_branch_nicks(change)
 
         return {
-            'branch': self._branch,
             'entry': change,
             'file_changes': file_changes,
-            'util': util,
             'revid': revid,
-            'url': self._branch.context_url,
         }

=== renamed file 'loggerhead/controllers/annotate_ui.py' => 'loggerhead/controllers/view_ui.py'
--- loggerhead/controllers/annotate_ui.py	2009-10-17 06:35:33 +0000
+++ loggerhead/controllers/view_ui.py	2011-09-08 00:33:28 +0000
@@ -17,7 +17,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import cgi
 import os
 import time
 
@@ -25,7 +24,7 @@
 import bzrlib.textfile
 import bzrlib.osutils
 
-from paste.httpexceptions import HTTPBadRequest, HTTPServerError
+from paste.httpexceptions import HTTPBadRequest, HTTPServerError, HTTPMovedPermanently
 
 from loggerhead.controllers import TemplatedBranchView
 try:
@@ -35,20 +34,18 @@
 from loggerhead import util
 
 
-class AnnotateUI(TemplatedBranchView):
-
-    template_path = 'loggerhead.templates.annotate'
-
-    def annotate_file(self, file_id, revid):
-        z = time.time()
-        lineno = 1
-        parity = 0
-
+class ViewUI(TemplatedBranchView):
+
+    template_path = 'loggerhead.templates.view'
+    
+    def tree_for(self, file_id, revid):
         file_revid = self._history.get_inventory(revid)[file_id].revision
-        tree = self._history._branch.repository.revision_tree(file_revid)
+        return self._history._branch.repository.revision_tree(file_revid)
 
+    def text_lines(self, file_id, revid):
         file_name = os.path.basename(self._history.get_path(revid, file_id))
-
+        
+        tree = self.tree_for(file_id, revid)
         file_text = tree.get_file_text(file_id)
         encoding = 'utf-8'
         try:
@@ -58,44 +55,27 @@
             file_text = file_text.decode(encoding)
 
         file_lines = bzrlib.osutils.split_lines(file_text)
+        # This can throw bzrlib.errors.BinaryFile (which our caller catches).
+        bzrlib.textfile.check_text_lines(file_lines)
+        
+        if highlight is not None:
+            hl_lines = highlight(file_name, file_text, encoding)
+            # highlight strips off extra newlines at the end of the file.
+            extra_lines = len(file_lines) - len(hl_lines)
+            hl_lines.extend([u''] * extra_lines)
+        else:
+            hl_lines = map(util.html_escape, file_lines)
+        
+        return hl_lines;
 
+    def file_contents(self, file_id, revid):
         try:
-            bzrlib.textfile.check_text_lines(file_lines)
+            file_lines = self.text_lines(file_id, revid)
         except bzrlib.errors.BinaryFile:
             # bail out; this isn't displayable text
-            yield util.Container(parity=0, lineno=1, status='same',
-                                 text='(This is a binary file.)',
-                                 change=util.Container())
-        else:
-            if highlight is not None:
-                hl_lines = highlight(file_name, file_text, encoding)
-                hl_lines.extend([u''] * (len(file_lines) - len(hl_lines)))
-            else:
-                hl_lines = map(cgi.escape, file_lines)
-
-            change_cache = {}
-
-            last_line_revid = None
-            for line_revid, text in tree.annotate_iter(file_id):
-                if line_revid == last_line_revid:
-                    # remember which lines have a new revno and which don't
-                    status = 'same'
-                else:
-                    status = 'changed'
-                    parity ^= 1
-                    last_line_revid = line_revid
-                    if line_revid in change_cache:
-                        change = change_cache[line_revid]
-                    else:
-                        change = self._history.get_changes([line_revid])[0]
-                        change_cache[line_revid] = change
-
-                yield util.Container(
-                    parity=parity, lineno=lineno, status=status,
-                    change=change, text=hl_lines[lineno - 1])
-                lineno += 1
-
-        self.log.debug('annotate: %r secs' % (time.time() - z,))
+            return ['(This is a binary file.)']
+
+        return file_lines
 
     def get_values(self, path, kwargs, headers):
         history = self._history
@@ -105,7 +85,7 @@
         file_id = kwargs.get('file_id', None)
         if (file_id is None) and (path is None):
             raise HTTPBadRequest('No file_id or filename '
-                                 'provided to annotate')
+                                 'provided to view')
 
         if file_id is None:
             file_id = history.get_file_id(revid, path)
@@ -139,14 +119,21 @@
             raise HTTPServerError('Could not fetch changes')
         branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
 
+        if inv[file_id].kind == "directory":
+            raise HTTPMovedPermanently(self._branch.context_url(['/files', revno_url, path]))
+
         return {
+            # In AnnotateUI, "annotated" is a dictionary mapping lines to changes.
+            # We exploit the fact that bool({}) is False when checking whether
+            # we're in "annotated" mode.
+            'annotated': {},
             'revno_url': revno_url,
             'file_id': file_id,
-            'path': path,
+            'file_path': path,
             'filename': filename,
             'navigation': navigation,
             'change': change,
-            'contents': list(self.annotate_file(file_id, revid)),
+            'contents':  self.file_contents(file_id, revid),
             'fileview_active': True,
             'directory_breadcrumbs': directory_breadcrumbs,
             'branch_breadcrumbs': branch_breadcrumbs,

=== added file 'loggerhead/exporter.py'
--- loggerhead/exporter.py	1970-01-01 00:00:00 +0000
+++ loggerhead/exporter.py	2011-11-24 07:46:41 +0000
@@ -0,0 +1,60 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+"""Exports an archive from a bazaar branch"""
+
+from bzrlib.export import get_export_generator
+
+
+class ExporterFileObject(object):
+    """Shim that accumulates temporarily written out data.
+
+    There are python tarfile classes that want to write to a file like object.
+    We want to stream data.  But wsgi assumes it can pull data from the
+    handler, rather than having bytes pushed.
+
+    So this class holds the data temporarily, until it is pulled.  It 
+    should never buffer everything because as soon as a chunk is produced, 
+    wsgi will be given the chance to take it.
+    """
+
+    def __init__(self):
+        self._buffer = []
+
+    def write(self, s):
+        self._buffer.append(s)
+
+    def get_buffer(self):
+        try:
+            return ''.join(self._buffer)
+        finally:
+            self._buffer = []
+
+    def close(self):
+        pass
+
+
+def export_archive(history, root, revid, archive_format):
+    """Export tree contents to an archive
+
+    :param history: Instance of history to export
+    :param root: Root location inside the archive.
+    :param revid: Revision to export
+    :param archive_format: Format of the archive, eg 'tar.gz'.
+    """
+    fileobj = ExporterFileObject()
+    tree = history._branch.repository.revision_tree(revid)
+    for _ in get_export_generator(tree=tree, root=root, fileobj=fileobj,
+        format=archive_format):
+        yield fileobj.get_buffer()
+    # Might have additonal contents written
+    yield fileobj.get_buffer()

=== modified file 'loggerhead/highlight.py'
--- loggerhead/highlight.py	2009-05-16 06:03:47 +0000
+++ loggerhead/highlight.py	2010-12-01 08:30:02 +0000
@@ -16,6 +16,9 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
+import bzrlib.osutils
+import cgi
+
 from pygments import highlight as _highlight_func
 from pygments.lexers import guess_lexer, guess_lexer_for_filename, TextLexer
 from pygments.formatters import HtmlFormatter
@@ -23,12 +26,19 @@
 
 DEFAULT_PYGMENT_STYLE = 'colorful'
 
+# Trying to highlight very large files using pygments was killing
+# loggerhead on launchpad.net, because pygments isn't very fast.
+# So we only highlight files if they're 512K or smaller.
+MAX_HIGHLIGHT_SIZE = 512000;
 
 def highlight(path, text, encoding, style=DEFAULT_PYGMENT_STYLE):
     """
     Returns a list of highlighted (i.e. HTML formatted) strings.
     """
 
+    if len(text) > MAX_HIGHLIGHT_SIZE:
+        return map(cgi.escape,  bzrlib.osutils.split_lines(text))
+
     formatter = HtmlFormatter(style=style, nowrap=True, classprefix='pyg-')
 
     try:
@@ -39,6 +49,7 @@
         except (ClassNotFound, ValueError):
             lexer = TextLexer(encoding=encoding)
 
-    hl_lines = _highlight_func(text, lexer, formatter).split('\n')
+    hl_lines = _highlight_func(text, lexer, formatter)
+    hl_lines = bzrlib.osutils.split_lines(hl_lines)
 
     return hl_lines

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2009-10-17 08:59:33 +0000
+++ loggerhead/history.py	2012-01-23 21:10:46 +0000
@@ -1,5 +1,4 @@
-#
-# Copyright (C) 2008, 2009 Canonical Ltd.
+# Copyright (C) 2006-2011 Canonical Ltd.
 #                     (Authored by Martin Albisetti <argentina@gmail.com>)
 # Copyright (C) 2006  Robey Pointer <robey@lag.net>
 # Copyright (C) 2006  Goffredo Baroncelli <kreijack@inwind.it>
@@ -33,10 +32,14 @@
 import logging
 import re
 import textwrap
+import threading
+import tarfile
 
+from bzrlib import tag
 import bzrlib.branch
 import bzrlib.delta
 import bzrlib.errors
+import bzrlib.foreign
 import bzrlib.revision
 
 from loggerhead import search
@@ -133,23 +136,23 @@
                 new_revision=self.revid(self.new_inv, file_id)))
         if versioned == 'added':
             self.added.append(util.Container(
-                filename=rich_filename(paths[1], kind),
-                file_id=file_id, kind=kind[1]))
+                filename=rich_filename(paths[1], kind), kind=kind[1]))
         elif versioned == 'removed':
             self.removed.append(util.Container(
-                filename=rich_filename(paths[0], kind),
-                file_id=file_id, kind=kind[0]))
+                filename=rich_filename(paths[0], kind), kind=kind[0]))
         elif renamed:
             self.renamed.append(util.Container(
                 old_filename=rich_filename(paths[0], kind[0]),
                 new_filename=rich_filename(paths[1], kind[1]),
-                file_id=file_id,
-                text_modified=modified == 'modified'))
+                text_modified=modified == 'modified', exe_change=exe_change))
         else:
             self.modified.append(util.Container(
                 filename=rich_filename(paths[1], kind),
-                file_id=file_id))
+                text_modified=modified == 'modified', exe_change=exe_change))
 
+# The lru_cache is not thread-safe, so we need a lock around it for
+# all threads.
+rev_info_memory_cache_lock = threading.RLock()
 
 class RevInfoMemoryCache(object):
     """A store that validates values against the revids they were stored with.
@@ -173,7 +176,11 @@
         If a value was stored under `key`, with the same revid, return it.
         Otherwise return None.
         """
-        cached = self._cache.get(key)
+        rev_info_memory_cache_lock.acquire()
+        try:
+            cached = self._cache.get(key)
+        finally:
+            rev_info_memory_cache_lock.release()
         if cached is None:
             return None
         stored_revid, data = cached
@@ -185,8 +192,18 @@
     def set(self, key, revid, data):
         """Store `data` under `key`, to be checked against `revid` on get().
         """
-        self._cache[key] = (revid, data)
+        rev_info_memory_cache_lock.acquire()
+        try:
+            self._cache[key] = (revid, data)
+        finally:
+            rev_info_memory_cache_lock.release()
 
+# Used to store locks that prevent multiple threads from building a 
+# revision graph for the same branch at the same time, because that can
+# cause severe performance issues that are so bad that the system seems
+# to hang.
+revision_graph_locks = {}
+revision_graph_check_lock = threading.Lock()
 
 class History(object):
     """Decorate a branch to provide information for rendering.
@@ -196,8 +213,6 @@
     around it, serve the request, throw the History object away, unlock the
     branch and throw it away.
 
-    :ivar _file_change_cache: An object that caches information about the
-        files that changed between two revisions.
     :ivar _rev_info: A list of information about revisions.  This is by far
         the most cryptic data structure in loggerhead.  At the top level, it
         is a list of 3-tuples [(merge-info, where-merged, parents)].
@@ -226,18 +241,33 @@
         def update_missed_caches():
             for cache in missed_caches:
                 cache.set(cache_key, self.last_revid, self._rev_info)
-        for cache in caches:
-            data = cache.get(cache_key, self.last_revid)
-            if data is not None:
-                self._rev_info = data
+
+        # Theoretically, it's possible for two threads to race in creating
+        # the Lock() object for their branch, so we put a lock around
+        # creating the per-branch Lock().
+        revision_graph_check_lock.acquire()
+        try:
+            if cache_key not in revision_graph_locks:
+                revision_graph_locks[cache_key] = threading.Lock()
+        finally:
+            revision_graph_check_lock.release()
+
+        revision_graph_locks[cache_key].acquire()
+        try:
+            for cache in caches:
+                data = cache.get(cache_key, self.last_revid)
+                if data is not None:
+                    self._rev_info = data
+                    update_missed_caches()
+                    break
+                else:
+                    missed_caches.append(cache)
+            else:
+                whole_history_data = compute_whole_history_data(self._branch)
+                self._rev_info, self._rev_indices = whole_history_data
                 update_missed_caches()
-                break
-            else:
-                missed_caches.append(cache)
-        else:
-            whole_history_data = compute_whole_history_data(self._branch)
-            self._rev_info, self._rev_indices = whole_history_data
-            update_missed_caches()
+        finally:
+            revision_graph_locks[cache_key].release()
 
         if self._rev_indices is not None:
             self._revno_revid = {}
@@ -250,16 +280,12 @@
                 self._rev_indices[revid] = seq
                 self._revno_revid[revno_str] = revid
 
-    def __init__(self, branch, whole_history_data_cache, file_cache=None,
+    def __init__(self, branch, whole_history_data_cache,
                  revinfo_disk_cache=None, cache_key=None):
         assert branch.is_locked(), (
             "Can only construct a History object with a read-locked branch.")
-        if file_cache is not None:
-            self._file_change_cache = file_cache
-            file_cache.history = self
-        else:
-            self._file_change_cache = None
         self._branch = branch
+        self._branch_tags = None
         self._inventory_cache = {}
         self._branch_nick = self._branch.get_config().get_nickname()
         self.log = logging.getLogger('loggerhead.%s' % (self._branch_nick,))
@@ -292,7 +318,16 @@
         revid in revid_list.
         """
         if revid_list is None:
-            revid_list = [r[0][1] for r in self._rev_info]
+            # Just yield the mainline, starting at start_revid
+            revid = start_revid
+            is_null = bzrlib.revision.is_null
+            while not is_null(revid):
+                yield revid
+                parents = self._rev_info[self._rev_indices[revid]][2]
+                if not parents:
+                    return
+                revid = parents[0]
+            return
         revid_set = set(revid_list)
         revid = start_revid
 
@@ -305,10 +340,14 @@
                 r.add(self._rev_info[i][0][1])
                 i += 1
             return r
-        while True:
+        while revid_set:
             if bzrlib.revision.is_null(revid):
                 return
-            if introduced_revisions(revid) & revid_set:
+            rev_introduced = introduced_revisions(revid)
+            matching = rev_introduced.intersection(revid_set)
+            if matching:
+                # We don't need to look for these anymore.
+                revid_set.difference_update(matching)
                 yield revid
             parents = self._rev_info[self._rev_indices[revid]][2]
             if len(parents) == 0:
@@ -429,15 +468,41 @@
         if revid is None:
             revid = self.last_revid
         if file_id is not None:
-            # since revid is 'start_revid', possibly should start the path
-            # tracing from revid... FIXME
-            revlist = list(self.get_short_revision_history_by_fileid(file_id))
-            revlist = list(self.get_revids_from(revlist, revid))
+            revlist = list(
+                self.get_short_revision_history_by_fileid(file_id))
+            revlist = self.get_revids_from(revlist, revid)
         else:
-            revlist = list(self.get_revids_from(None, revid))
+            revlist = self.get_revids_from(None, revid)
         return revlist
 
-    def get_view(self, revid, start_revid, file_id, query=None):
+    @staticmethod
+    def _iterate_sufficiently(iterable, stop_at, extra_rev_count):
+        """Return a list of iterable.
+
+        If extra_rev_count is None, fully consume iterable.
+        Otherwise, stop at 'stop_at' + extra_rev_count.
+
+        Example:
+          iterate until you find stop_at, then iterate 10 more times.
+        """
+        if extra_rev_count is None:
+            return list(iterable)
+        result = []
+        found = False
+        for n in iterable:
+            result.append(n)
+            if n == stop_at:
+                found = True
+                break
+        if found:
+            for count, n in enumerate(iterable):
+                if count >= extra_rev_count:
+                    break
+                result.append(n)
+        return result
+
+    def get_view(self, revid, start_revid, file_id, query=None,
+                 extra_rev_count=None):
         """
         use the URL parameters (revid, start_revid, file_id, and query) to
         determine the revision list we're viewing (start_revid, file_id, query)
@@ -448,6 +513,10 @@
               file.
             - if a start_revid is given, we're viewing the branch from a
               specific revision up the tree.
+            - if extra_rev_count is given, find the view from start_revid =>
+              revid, and continue an additional 'extra_rev_count'. If not
+              given, then revid_list will contain the full history of
+              start_revid
 
         these may be combined to view revisions for a specific file, from
         a specific revision, with a specific search query.
@@ -466,12 +535,16 @@
 
         if query is None:
             revid_list = self.get_file_view(start_revid, file_id)
+            revid_list = self._iterate_sufficiently(revid_list, revid,
+                                                    extra_rev_count)
             if revid is None:
                 revid = start_revid
             if revid not in revid_list:
                 # if the given revid is not in the revlist, use a revlist that
                 # starts at the given revid.
                 revid_list = self.get_file_view(revid, file_id)
+                revid_list = self._iterate_sufficiently(revid_list, revid,
+                                                        extra_rev_count)
                 start_revid = revid
             return revid, start_revid, revid_list
 
@@ -494,7 +567,7 @@
     def get_inventory(self, revid):
         if revid not in self._inventory_cache:
             self._inventory_cache[revid] = (
-                self._branch.repository.get_revision_inventory(revid))
+                self._branch.repository.get_inventory(revid))
         return self._inventory_cache[revid]
 
     def get_path(self, revid, file_id):
@@ -630,16 +703,26 @@
         """
         message, short_message = clean_message(revision.message)
 
-        tags = self._branch.tags.get_reverse_tag_dict()
+        if self._branch_tags is None:
+            self._branch_tags = self._branch.tags.get_reverse_tag_dict()
 
         revtags = None
-        if tags.has_key(revision.revision_id):
-          revtags = ', '.join(tags[revision.revision_id])
+        if revision.revision_id in self._branch_tags:
+          # tag.sort_* functions expect (tag, data) pairs, so we generate them,
+          # and then strip them
+          tags = [(t, None) for t in self._branch_tags[revision.revision_id]]
+          sort_func = getattr(tag, 'sort_natural', None)
+          if sort_func is None:
+              tags.sort()
+          else:
+              sort_func(self._branch, tags)
+          revtags = u', '.join([t[0] for t in tags])
 
         entry = {
             'revid': revision.revision_id,
             'date': datetime.datetime.fromtimestamp(revision.timestamp),
             'utc_date': datetime.datetime.utcfromtimestamp(revision.timestamp),
+            'committer': revision.committer,
             'authors': revision.get_apparent_authors(),
             'branch_nick': revision.properties.get('branch-nick', None),
             'short_comment': short_message,
@@ -649,21 +732,31 @@
             'bugs': [bug.split()[0] for bug in revision.properties.get('bugs', '').splitlines()],
             'tags': revtags,
         }
+        if isinstance(revision, bzrlib.foreign.ForeignRevision):
+            foreign_revid, mapping = (
+                revision.foreign_revid, revision.mapping)
+        elif ":" in revision.revision_id:
+            try:
+                foreign_revid, mapping = \
+                    bzrlib.foreign.foreign_vcs_registry.parse_revision_id(
+                        revision.revision_id)
+            except bzrlib.errors.InvalidRevisionId:
+                foreign_revid = None
+                mapping = None
+        else:
+            foreign_revid = None
+        if foreign_revid is not None:
+            entry["foreign_vcs"] = mapping.vcs.abbreviation
+            entry["foreign_revid"] = mapping.vcs.show_foreign_revid(foreign_revid)
         return util.Container(entry)
 
-    def get_file_changes_uncached(self, entry):
+    def get_file_changes(self, entry):
         if entry.parents:
             old_revid = entry.parents[0].revid
         else:
             old_revid = bzrlib.revision.NULL_REVISION
         return self.file_changes_for_revision_ids(old_revid, entry.revid)
 
-    def get_file_changes(self, entry):
-        if self._file_change_cache is None:
-            return self.get_file_changes_uncached(entry)
-        else:
-            return self._file_change_cache.get_file_changes(entry)
-
     def add_changes(self, entry):
         changes = self.get_file_changes(entry)
         entry.changes = changes
@@ -693,7 +786,8 @@
         """
         repo = self._branch.repository
         if (bzrlib.revision.is_null(old_revid) or
-            bzrlib.revision.is_null(new_revid)):
+            bzrlib.revision.is_null(new_revid) or
+            old_revid == new_revid):
             old_tree, new_tree = map(
                 repo.revision_tree, [old_revid, new_revid])
         else:
@@ -708,4 +802,6 @@
             renamed=sorted(reporter.renamed, key=lambda x: x.new_filename),
             removed=sorted(reporter.removed, key=lambda x: x.filename),
             modified=sorted(reporter.modified, key=lambda x: x.filename),
-            text_changes=sorted(reporter.text_changes, key=lambda x: x.filename))
+            text_changes=sorted(reporter.text_changes,
+                                key=lambda x: x.filename))
+

=== added file 'loggerhead/load_test.py'
--- loggerhead/load_test.py	1970-01-01 00:00:00 +0000
+++ loggerhead/load_test.py	2011-03-02 14:07:21 +0000
@@ -0,0 +1,230 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Code to do some load testing against a loggerhead instance.
+
+This is basically meant to take a list of actions to take, and run it against a
+real host, and see how the results respond.::
+
+    {"parameters": {
+         "base_url": "http://localhost:8080",
+     },
+     "requests": [
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"}
+     ],
+    }
+
+All threads have a Queue length of 1. When a third request for a given thread
+is seen, no more requests are queued until that thread finishes its current
+job. So this results in all requests being issued sequentially::
+
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"}
+
+While this would cause all requests to be sent in parallel:
+
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"},
+        {"thread": "3", "relpath": "/changes"},
+        {"thread": "4", "relpath": "/changes"}
+
+This should keep 2 threads pipelined with activity, as long as they finish in
+approximately the same speed. We'll start the first thread running, and the
+second thread, and queue up both with a second request once the first finishes.
+When we get to the third request for thread "1", we block on queuing up more
+work until the first thread 1 request has finished.
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"}
+
+There is not currently a way to say "run all these requests keeping exactly 2
+threads active". Though if you know the load pattern, you could approximate
+this.
+"""
+
+import threading
+import time
+import Queue
+
+import simplejson
+
+from bzrlib import (
+    errors,
+    transport,
+    urlutils,
+    )
+
+# This code will be doing multi-threaded requests against bzrlib.transport
+# code. We want to make sure to load everything ahead of time, so we don't get
+# lazy-import failures
+_ = transport.get_transport('http://example.com')
+
+
+class RequestDescription(object):
+    """Describes info about a request."""
+
+    def __init__(self, descrip_dict):
+        self.thread = descrip_dict.get('thread', '1')
+        self.relpath = descrip_dict['relpath']
+
+
+class RequestWorker(object):
+    """Process requests in a worker thread."""
+
+    _timer = time.time
+
+    def __init__(self, identifier, blocking_time=1.0, _queue_size=1):
+        self.identifier = identifier
+        self.queue = Queue.Queue(_queue_size)
+        self.start_time = self.end_time = None
+        self.stats = []
+        self.blocking_time = blocking_time
+
+    def step_next(self):
+        url = self.queue.get(True, self.blocking_time)
+        if url == '<noop>':
+            # This is usually an indicator that we want to stop, so just skip
+            # this one.
+            self.queue.task_done()
+            return
+        self.start_time = self._timer()
+        success = self.process(url)
+        self.end_time = self._timer()
+        self.update_stats(url, success)
+        self.queue.task_done()
+
+    def run(self, stop_event):
+        while not stop_event.isSet():
+            try:
+                self.step_next()
+            except Queue.Empty:
+                pass
+
+    def process(self, url):
+        base, path = urlutils.split(url)
+        t = transport.get_transport(base)
+        try:
+            # TODO: We should probably look into using some part of
+            #       blocking_timeout to decide when to stop trying to read
+            #       content
+            content = t.get_bytes(path)
+        except (errors.TransportError, errors.NoSuchFile):
+            return False
+        return True
+
+    def update_stats(self, url, success):
+        self.stats.append((url, success, self.end_time - self.start_time))
+
+
+class ActionScript(object):
+    """This tracks the actions that we want to perform."""
+
+    _worker_class = RequestWorker
+    _default_base_url = 'http://localhost:8080'
+    _default_blocking_timeout = 60.0
+
+    def __init__(self):
+        self.base_url = self._default_base_url
+        self.blocking_timeout = self._default_blocking_timeout
+        self._requests = []
+        self._threads = {}
+        self.stop_event = threading.Event()
+
+    @classmethod
+    def parse(cls, content):
+        script = cls()
+        json_dict = simplejson.loads(content)
+        if 'parameters' not in json_dict:
+            raise ValueError('Missing "parameters" section')
+        if 'requests' not in json_dict:
+            raise ValueError('Missing "requests" section')
+        param_dict = json_dict['parameters']
+        request_list = json_dict['requests']
+        base_url = param_dict.get('base_url', None)
+        if base_url is not None:
+            script.base_url = base_url
+        blocking_timeout = param_dict.get('blocking_timeout', None)
+        if blocking_timeout is not None:
+            script.blocking_timeout = blocking_timeout
+        for request_dict in request_list:
+            script.add_request(request_dict)
+        return script
+
+    def add_request(self, request_dict):
+        request = RequestDescription(request_dict)
+        self._requests.append(request)
+
+    def _get_worker(self, thread_id):
+        if thread_id in self._threads:
+            return self._threads[thread_id][0]
+        handler = self._worker_class(thread_id,
+                                     blocking_time=self.blocking_timeout/10.0)
+
+        t = threading.Thread(target=handler.run, args=(self.stop_event,),
+                             name='Thread-%s' % (thread_id,))
+        self._threads[thread_id] = (handler, t)
+        t.start()
+        return handler
+
+    def finish_queues(self):
+        """Wait for all queues of all children to finish."""
+        for h, t in self._threads.itervalues():
+            h.queue.join()
+
+    def stop_and_join(self):
+        """Stop all running workers, and return.
+
+        This will stop even if workers still have work items.
+        """
+        self.stop_event.set()
+        for h, t in self._threads.itervalues():
+            # Signal the queue that it should stop blocking, we don't have to
+            # wait for the queue to empty, because we may see stop_event before
+            # we see the <noop>
+            h.queue.put('<noop>')
+            # And join the controlling thread
+            for i in range(10):
+                t.join(self.blocking_timeout / 10.0)
+                if not t.isAlive():
+                    break
+
+    def _full_url(self, relpath):
+        return self.base_url + relpath
+
+    def run(self):
+        self.stop_event.clear()
+        for request in self._requests:
+            full_url = self._full_url(request.relpath)
+            worker = self._get_worker(request.thread)
+            worker.queue.put(full_url, True, self.blocking_timeout)
+        self.finish_queues()
+        self.stop_and_join()
+
+
+def run_script(filename):
+    with open(filename, 'rb') as f:
+        content = f.read()
+    script = ActionScript.parse(content)
+    script.run()
+    return script

=== modified file 'loggerhead/main.py'
--- loggerhead/main.py	2009-10-17 06:35:33 +0000
+++ loggerhead/main.py	2011-03-16 11:39:27 +0000
@@ -56,21 +56,39 @@
     return config, base
 
 
-def setup_logging(config):
-    logging.basicConfig()
-    logging.getLogger('').setLevel(logging.DEBUG)
+def setup_logging(config, init_logging=True, log_file=None):
+    log_level = config.get_log_level()
+    if init_logging:
+        logging.basicConfig()
+        if log_level is not None:
+            logging.getLogger('').setLevel(log_level)
     logger = logging.getLogger('loggerhead')
-    if config.get_option('log_folder'):
-        logfile_path = os.path.join(
-            config.get_option('log_folder'), 'serve-branches.log')
+    if log_level is not None:
+        logger.setLevel(log_level)
+    if log_file is not None:
+        handler = logging.StreamHandler(log_file)
     else:
-        logfile_path = 'serve-branches.log'
-    logfile = logging.FileHandler(logfile_path, 'a')
-    formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(name)s:'
-                                  ' %(message)s')
-    logfile.setFormatter(formatter)
-    logfile.setLevel(logging.DEBUG)
-    logger.addHandler(logfile)
+        if config.get_option('log_folder'):
+            logfile_path = os.path.join(
+                config.get_option('log_folder'), 'serve-branches.log')
+        else:
+            logfile_path = 'serve-branches.log'
+        handler = logging.FileHandler(logfile_path, 'a')
+        formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(name)s:'
+                                      ' %(message)s')
+        handler.setFormatter(formatter)
+    # We set the handler to accept all messages, the *logger* won't emit them
+    # if it is configured to suppress it
+    handler.setLevel(logging.DEBUG)
+    logger.addHandler(handler)
+    def _restrict_logging(logger_name):
+        logger = logging.getLogger(logger_name)
+        if logger.getEffectiveLevel() < logging.INFO:
+            logger.setLevel(logging.INFO)
+    # simpleTAL is *very* verbose in DEBUG mode, which is otherwise the
+    # default. So quiet it up a bit.
+    _restrict_logging('simpleTAL')
+    _restrict_logging('simpleTALES')
     return logger
 
 
@@ -151,4 +169,21 @@
     else:
         host = config.get_option('user_host')
 
-    httpserver.serve(app, host=host, port=port)
+    if not config.get_option('protocol'):
+        protocol = 'http'
+    else:
+        protocol = config.get_option('protocol')
+
+    if protocol == 'http':
+        httpserver.serve(app, host=host, port=port)
+    else:
+        if protocol == 'fcgi':
+            from flup.server.fcgi import WSGIServer
+        elif protocol == 'scgi':
+            from flup.server.scgi import WSGIServer
+        elif protocol == 'ajp':
+            from flup.server.ajp import WSGIServer
+        else:
+            print 'Unknown protocol: %s.' % (protocol)
+            sys.exit(1)
+        WSGIServer(app, bindAddress=(host, int(port))).run()

=== modified file 'loggerhead/static/css/diff.css'
--- loggerhead/static/css/diff.css	2009-06-17 20:28:20 +0000
+++ loggerhead/static/css/diff.css	2011-02-18 05:41:17 +0000
@@ -1,164 +1,113 @@
 /*Info Box*/
 .infoContainer {
-    width:970px;
-    padding:10px 10px 0px 10px;
-    margin:0px 0px 10px 0px;
-    color:#666;
-    background:#fcfcfc url(../images/bg_infobox.gif) repeat-y top right;
-    border:1px solid #b7b7b7;
-}
+    width: 970px;
+    padding: 10px 10px 0 10px;
+    margin: 0 0 10px 0;
+    color: #666;
+    background: #fcfcfc url(../images/bg_infobox.gif) repeat-y top right;
+    border: 1px solid #b7b7b7;
+    }
 ul#list-files {
-    float:right;
-    width:460px;
-    list-style:none;
-    margin:0px;
-    padding:0px;
-    font:normal 11px/normal sans-serif;
-    color:#000;
-}
+    float: right;
+    width: 460px;
+    font-size: 77%;
+    color: #000;
+    }
 ul#list-files li {
-    width:440px;
-    padding:2px 0px 2px 20px;
-    margin:0px 0px 2px 8px;
-    background:url(../images/ico_file.gif) 2px 0px no-repeat;
-}
+    width: 440px;
+    padding: 2px 0 2px 20px;
+    margin: 0 0 2px 8px;
+    background: url(../images/ico_file.gif) 2px 0 no-repeat;
+    }
 ul#list-files li.desc {
     margin-left: 0;
-    background:url(../images/ico_description.gif) 0px 0px no-repeat;
-}
+    background: url(../images/ico_description.gif) 0 0 no-repeat;
+    }
 #infTxt {
-    float:left;
-    width:493px;
-    margin:0px 0px 5px 0px;
-}
-.information, .committer, .timer, .mfrom, .mto, .revid, .bug, .tags {
-    clear:both;
-    margin:0px;
-    padding:0px 0px 2px 18px;
-    font:normal 11px/1.3 sans-serif;
-    color:#666;
-    background:url(../images/ico_description.gif) no-repeat top left;
-}
-.revid {
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-}
-.tags {
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-    background:url(../images/ico_tag.gif) no-repeat top left;
-}
-.committer, .mfrom, .mto {
-    color:#000;
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-    float:left;
-    display:inline;
-    background:url(../images/ico_committer.gif) no-repeat 2px 0px;
-}
-.timer {
-    margin:0px 10px 5px 0px;
-    padding:1px 0px 0px 18px;
-    float:left;
-    clear:none;
-    display:inline;
-    background:url(../images/ico_time.gif) no-repeat top left;
-}
-.mfrom {
-    background:url(../images/ico_mergefrom.gif) no-repeat top left;
-}
-.mto {
-    background:url(../images/ico_mergeto.gif) no-repeat top left;
-}
-.bug {
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-    float:left;
-    clear:none;
-    display:inline;
-    background:url(../images/ico_bug.png) no-repeat top left;
-}
-/**/
-#loggerheadCont p.expand, #loggerheadCont p.expand a, #loggerheadCont p.expand a:link, #loggerheadCont p.expand a:visited, #loggerheadCont p.expand a:hover {
-    color:#0f6b97;
-    text-decoration:none;
-    margin-right:10px;
-    float:left;
-}
-#loggerheadCont p.codin, #loggerheadCont p.codin a, #loggerheadCont p.codin a:link, #loggerheadCont p.codin a:visited, #loggerheadCont p.codin a:hover {
-    color:#000;
-    font-size:12px;
-    text-decoration:none;
-    margin-right:10px;
-    float:left;
-}
+    float: left;
+    width: 493px;
+    margin: 0 0 5px 0;
+    font-size:  77%;
+    color: #666;
+    }
 /*Diff Boxes*/
 .diffBox {
-    width:100%;
-    padding:6px 0%;
-    background:#eef6fa;
-    border:1px solid #d2d2d2;
-    margin:10px 0px 0px 0px;
-}
+    width: 100%;
+    padding: 6px 0;
+    background: #eef6fa;
+    border: 1px solid #d2d2d2;
+    margin: 10px 0 0 0;
+    }
 .diffBox img {
-    vertical-align:middle;
-}
-.diffBox a, .diffBox a:link, .diffBox a:visited, .diffBox a:hover {
-    text-decoration:none;
-}
+    vertical-align: middle;
+    }
+.diffBox a {
+    text-decoration: none;
+    }
 .diff {
-    margin: 2em 0 2em 0;
-}
+    margin: 2em 0;
+    }
 .diffinfo {
     overflow: hidden;
-}
+    }
 .lineNumber {
-    width:2%;
-    border-right:1px solid #d5d5d5;
-    border-left:1px solid #d5d5d5;
-    background:#f8fbfd;
-    text-align:right;
-}
+    width: 2%;
+    border-width: 0 1px;
+    border-style: solid;
+    border-color: #d5d5d5;
+    background: #f8fbfd;
+    text-align: right;
+    }
 .unified .lineNumber.second {
-    border-left:0px;
-}
-.code, .lineNumber {
-    float:left;
-    padding:0px 1%;
-    color:#000;
-    font:normal 12px/1.8 monospace;
-}
-.blank {
-    float: left;
-    padding:0px 1%;
-}
-.unified .code { width:89%; }
-.sbs .code, .blank {
-    width:43%;
+    border-left: 0;
+    }
+.code,
+.lineNumber {
+    float: left;
+    padding: 0 1%;
+    color: #000;
+    font-size: 85%;
+    font-family: 'UbuntuBeta Mono', 'Ubuntu Mono', monospace;
+    line-height: 180%;
+    }
+.blank {
+    float: left;
+    padding: 0 1%;
+    }
+.unified .code {
+    width: 89%;
+    }
+.sbs .code,
+.blank {
+    width: 43%;
     overflow: hidden;
-}
+    }
 .unified .insert-row .lineNumber.first {
-    background:#c9ffc9;
-}
+    background: #c9ffc9;
+    }
 .unified .delete-row .lineNumber.second {
-    background:#ffbfbf;
-}
-.code.insert, .insert-row .lineNumber.second, .both-row .lineNumber.second {
-    background:#92ed92;
-}
-.code.delete, .delete-row .lineNumber.first, .both-row .lineNumber.first {
-    background:#ff7f7f;
-}
+    background: #ffbfbf;
+    }
+.code.insert,
+.insert-row .lineNumber.second,
+.both-row .lineNumber.second {
+    background: #92ed92;
+    }
+.code.delete,
+.delete-row .lineNumber.first,
+.both-row .lineNumber.first {
+    background: #ff7f7f;
+    }
 .separate {
     height: 2em;
     background: #e3e3e3;
-}
+    }
 ul#pages {
-    margin-top:10px;
-}
+    margin-top: 10px;
+    }
 /*Foot Links*/
 #loggerheadCont p.lpandbzr {
-    float:left;
-    padding:3px 0px 0px 0px;
-    margin-top:10px;
-}
+    float: left;
+    padding: 3px 0 0 0;
+    margin-top: 10px;
+    }

=== modified file 'loggerhead/static/css/files.css'
--- loggerhead/static/css/files.css	2009-06-17 20:28:20 +0000
+++ loggerhead/static/css/files.css	2011-02-18 05:41:17 +0000
@@ -1,50 +1,5 @@
-/*Info Box*/
-.information, .committer, .timer, .mfrom, .mto, .revid, .bug, .tags {
-	float:left;
-	margin:0px 10px 5px 0px;
-	padding:0px 0px 2px 18px;
-	font:normal 11px/1.3 sans-serif;
-	color:#666;
-	background:url(../images/ico_description.gif) no-repeat top left;
-}
-.revid {
-    float:left;
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-}
-.tags {
-    float:left;
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-    background:url(../images/ico_tag.gif) no-repeat top left;
-}
-.comiter, .mfrom, .mto {
-	color:#000;
-	padding:1px 0px 0px 18px;
-	margin:0px 10px 5px 0px;
-	float:left;
-	display:inline;
-	background:url(../images/ico_committer.gif) no-repeat 2px 0px;
-}
-.timer {
-	margin:0px 10px 5px 0px;
-	padding:1px 0px 0px 18px;
-	float:left;
-	clear:none;
-	display:inline;
-	background:url(../images/ico_time.gif) no-repeat top left;
-}
-.mfrom {
-	background:url(../images/ico_mergefrom.gif) no-repeat top left;
-}
-.mto {
-	background:url(../images/ico_mergetop.gif) no-repeat top left;
-}
-.information {
-    clear:both;
-    width:100%;
-    position:relative;
-}
-.bug {
-	background:url(../images/ico_bug.png) no-repeat top left;
-}
+#infTxt {
+    font-size: 77%;
+    color: #666;
+    margin-bottom: 10px;
+    }

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2009-06-11 21:16:27 +0000
+++ loggerhead/static/css/global.css	2011-11-23 17:21:39 +0000
@@ -1,354 +1,410 @@
+/*
+    CSS Structure
+    - Reset and Generic Styles
+    - Header & Navigation
+        - Loggerhead Menu Tabs
+        - Sub-menu Tabs
+        - Breadcrumbs
+    - Search & RSS 
+    - General Content
+    - Log Table
+    - Revision List
+    - Pagination
+    - Footer
+*/
+
+
+/* =========================
+   Reset and Generic Styles
+*/
+body, img, form, tr, ul, li, h1, h2, h3, h4, p, table {
+    padding: 0;
+    margin: 0;
+    }
+body, html, table {
+    font-size: 93%;
+    }
 body {
-	padding:0px;
-	margin:0px;
-	font:normal 12px/normal sans-serif;
-	color:#666;
-	background:#fff;
-}
-a, a:link, a:visited, a:hover {
-	color:#0f597f;
-	text-decoration:none;
-	font-weight:normal;
-	outline:none;
-}
-b a:link, b a:visited {
-    font-weight: bold;
-}
+    font-weight: normal;
+    font-family: UbuntuBeta, Ubuntu, 'Bitstream Vera Sans', 'DejaVu Sans', Tahoma, sans-serif;
+    color: #000;
+    background: #fff;
+    }
+a {
+    color: #03A;
+    text-decoration: none;
+    }
+a:hover {
+    text-decoration: underline;
+    }
 img {
-	padding:0px;
-	margin:0px;
-	border:0px;
-	outline:none;
-}
-form {
-  margin: 0;
-  padding: 0;
-}
-tr td {
-  vertical-align: top;
-}
+    border: 0;
+    outline: none;
+    }
 .clear {
-	clear:both;
-	font-size:0px;
-	height:0px;
-}
-.fr {float:right;}
-.fl {float:left;}
-
+    clear: both;
+    font-size: 0;
+    height: 0;
+    }
+.fr {
+    float: right;
+    }
+.fl {
+    float: left;
+    }
 ul {
-    list-style:none;
+    list-style: none;
+    }
+table {
+    caption-side: top;
+    width: 100%;
+    border: none;
+    border-spacing: 0;
+    border-collapse: collapse;
+    }
+tr td {
+    vertical-align: top;
+    }
+th, td {
+    border-color: #D2D2D2;
+    }
+th {
+    text-align: left;
+    color: #5A5A5A;
+    padding: 4px 8px;
+    border-width: 1px;
+    border-style: solid;
+    }
+td {
+    padding: 4px 4px;
+    border-width: 0 0 1px 0;
+    border-style: dotted;
+    }
+h1 {
+    font-size:138.5%;
+    padding: 10px 0 5px 0;
+    }
+h2 {
+    font-size: 123.1%;
+    padding: 10px 0 5px 0;
+    }
+p {
+    padding: 5px 0;
+    }
+p img {
+    vertical-align: middle;
+    }
+
+
+/* =========================
+   Header & Navigation
+*/
+
+.black-link {
+    border-bottom: 1px solid #D2D2D2;
+    padding: 6px 1% 8px 1%;
+    font-size: 85%;
 }
 
-/* Loggerhead Menu tabs */
+/* Branch Title */
+h1.branch-name {
+    font-size: 131%;
+    padding: 0;
+    margin: 20px 0 0 1%;
+    }
+
+/* Loggerhead Menu Tabs */
 ul#menuTabs {
-	width:98%;
-	height:33px;
-	list-style:none;
-	color:#fff;
-	background:#fff url(../images/bg_menuTabs.gif) bottom repeat-x;
-	margin:0px;
-	padding:12px 0px 0px 2%;
-}
-ul#menuTabs li a:link, ul#menuTabs li a:visited {
-	float:left;
-	width:93px;
-	padding:10px 0px 0px 0px;
-	height:23px;
-	font:normal 14px/1.5 sans-serif;
-	text-align:center;
-	color:#fff;
-	text-decoration:none;
-	background:#ad2001 url(../images/bg_Tabs.gif) bottom center;
-}
+    height: 33px;
+    color: #fff;
+    margin: 0;
+    padding: 15px 20px 0 1%;
+    }
 ul#menuTabs li {
-	float:left;
-	color:#fff;
-	background:#f00 url(../images/bg_Tabs.gif) bottom center;
-}
+    float: left;
+    padding: 0 7px 0 0;
+    }
+ul#menuTabs li a {
+    padding: 3px 6px;
+    color: #000;
+    }
 ul#menuTabs li a:hover {
-	float:left;
-	color:#000;
-	background:#666 url(../images/bg_Tabs.gif) top center;
-	text-decoration:none;
-}
-ul#menuTabs li a#on:link, ul#menuTabs li a#on:visited, ul#menuTabs a#on:hover {
-	float:left;
-	color:#000;
-	background:#666 url(../images/bg_Tabs.gif) top center;
-	text-decoration:none;
-}
-/*Sub menu tabs*/
+    background-color: #f3f3f3;
+    text-decoration: none;
+    }
+ul#menuTabs li a#on {
+    font-weight: bold;
+    color: #fff;
+    background-color: #747474;
+    }
+
+/* Sub-menu Tabs */
 ul#submenuTabs {
-	width:100%;
-	height:24px;
-	list-style:none;
-	color:#fff;
-	background:#fff;
-	margin:0px;
-	padding:10px 0%;
-}
-ul#submenuTabs li a:link, ul#submenuTabs li a:visited {
-	float:left;
-	display:inline;
-	height:22px;
-	padding:2px 15px 0px 15px;
-	font:normal 14px/1.5 sans-serif;
-	text-align:center;
-	color:#fff;
-	text-decoration:none;
-	background:#2f40a5;
-}
+    height: 24px;
+    margin-bottom: 10px;
+    }
 ul#submenuTabs li {
-	float:left;
-	display:inline;
-	margin-left:1px;
-	color:#fff;
-	background:#2f40a5;
-}
-ul#submenuTabs li a:hover {
-	float:left;
-	display:inline;
-	color:#fff;
-	background:#2f40a5;
-	text-decoration:underline;
-}
-ul#submenuTabs li#first a, ul#submenuTabs li#first a:link, ul#submenuTabs li#first a:visited, ul#submenuTabs li#first a:hover {
-	background:#2f40a5 url(../images/bg_submenuTabs.gif) top left no-repeat;
-}
-ul#submenuTabs li#last a, ul#submenuTabs li#last a:link, ul#submenuTabs li#last a:visited, ul#submenuTabs li#last a:hover {
-	background:#2f40a5 url(../images/bg_submenuTabs.gif) bottom right no-repeat;
-}
-
-/* Search & RSS */
-#finderBox {
-	width:300px;
-	float:right;
-	padding: 10px 0px 0px 0px;
-}
-#finderBox label {
-	float:left;
-	display:inline;
-	width:100px;
-	font:normal 12px/normal sans-serif;
-	text-align:right;
-	color:#000;
-	padding:5px 5px 0px 0px;
-}
-#finderBox input {
-	float:left;
-	font:normal 12px/normal sans-serif;
-	color:#666;
-	display:inline;
-	width:130px;
-	padding:3px;
-	background:#fff url(../images/bg_search_input.gif) 110px 3px no-repeat;
-	border:1px solid #e9e9e9;
-}
-#search_terms {
-    display:none;
-    border:solid 1px black;
-    width:180px;
-    overflow:auto;
-    background: #fff;
-}
-.rssfeed {
-	float:left;
-	display:inline;
-	padding:0px 0px 0px 10px;
-}
-
-/* General Content */
+    float: left;
+    display: inline;
+    margin-right: 20px;
+    }
+ul#submenuTabs li a {
+    float: left;
+    display: inline;
+    height: 22px;
+    line-height: 22px;
+    padding: 0 0 0 18px;
+    background: transparent url(../images/ico_description.gif) no-repeat center left;
+    }
+
+/* Breadcrumbs */
+#breadcrumbs {
+    font-size: 77%;
+    margin-bottom: 10px;
+    }
+
+#branch-info {
+    padding: 10px;
+    margin: 0 0 10px 0;
+    color: #666;
+    background: #fcfcfc;
+    border: 1px solid #b7b7b7;
+    }
+
+
+/* =========================
+   General Content
+*/
 #loggerheadCont {
-	width:98%;
-	padding:10px 1%;
-	color:#000;
-	background:#fff;
-}
-#loggerheadCont h1 {
-	font-size:18px;
-	color:#666;
-	margin:0px;
-	padding:10px 0px 5px 0px;
-}
-#loggerheadCont h1 span {
-    color:#889ea9;
-}
-#loggerheadCont p {
-	color:#000;
-	margin:0px;
-	padding:5px 0px;
-}
-#loggerheadCont p.expand, #loggerheadCont p.expand a, #loggerheadCont p.expand a:link, #loggerheadCont p.expand a:visited, #loggerheadCont p.expand a:hover {
-	color:#000;
-	text-decoration:none;	
-}
-#loggerheadCont p img {
-	vertical-align:middle;
-}
-.link {
-	padding:0px 11px 0px 0px;
-	margin:0px 2px 0px 0px;
-	font-weight:bold;
-}
-.revnolink a, .revnolink a:visited, .revnolink a:link, .revnolink a:hover {
-	color:#CC6600;
-	text-decoration:none;		
-}
-
-/*Log Table*/
+    width: 98%;
+    padding: 0 1% 10px 1%;
+    }
+
+
+/* =========================
+   Log Table
+*/
 #logentries {
-	caption-side:top;
-	width:99%;
-	margin:0px auto 10px auto;
-	padding:0px;
-	background:#fff;
-	border:none;
-	border-spacing:0px;
-	border-collapse:collapse;
-}
-.logheader {
-	background:#fff;
-	font:bold 14px/normal sans-serif;
-	padding:0px;
-	margin:0px;
-}
-.blueRow0, .blueRow1 {
-	background:#eef6fa;
-	font:normal 14px/normal sans-serif;	
-	padding:0px;
-	margin:0px;
-}
-.blueRow1 {
-	background:#f8fbfd;
-}
-.revisionnumber, .expandcell, .diffcell, .downloadcell, .revnro, .expcell, .diffr, .downr, .allicones {
-	padding:4px 0px;
-	width:42px;
-	text-align:center;
-	border:1px solid #d2d2d2;
-}
-.expandcell, .expcell {
-	width:28px;	
-}
-.summarycell, .authorcell, .tagscell, .datecell, .timedate{
-  padding: 4px 8px;
-  border: 1px solid #d2d2d2;
-}
-.icocell {
-  width: 18px;
-}
-.authorcell, .autcell {
-  width: 240px;
-}
-.tagscell, .tagcell {
-  width: 240px;
-}
-.datecell, .date {
-  width: 180px;
-}
-.timedate, .timedate2 {
-	width:80px;
-}
-.downloadcell, .downr {
-	width:47px;
-}
-.revnro, .expcell, .diffr, .summcell, .autcell, .tagcell, .icocell, .date, .downr, .allicones, .timedate2 {
-  border: 0;
-  padding: 4px 4px;
-  border-bottom:1px solid #d2d2d2;
-}
+    margin-bottom: 10px;
+    clear: left;
+    }
+p.expand, p.codin {
+    margin-right: 10px;
+    float: left;
+    }
+p.expand a, p.codin a {
+    color: #000;
+    text-decoration: none;
+    }
+.show_if_js {
+    display: none;
+    }
+
+/* th and td styles */
+th.revisionnumber,
+th.expandcell,
+th.diffcell,
+th.downloadcell {
+    padding-left: 0;
+    padding-right: 0;
+    text-align: center;
+    }
+th.diffcell,
+th.revisionnumber {
+    width: 42px;
+    }
+th.expandcell,
+td.expcell {
+    width: 10px;
+    }
+td.icocell {
+    width: 18px;
+    }
+th.authorcell,
+td.autcell,
+th.tagscell,
+td.tagcell {
+    width: 240px;
+    }
+th.datecell,
+td.date {
+    width: 180px;
+    }
+th.timedate,
+td.timedate2 {
+    width: 80px;
+    }
+th.downloadcell,
+td.downr {
+    width: 47px;
+    }
+
+
+/* =========================
+   Revision List
+*/
+.revision_log {
+    overflow: hidden;
+    }
 .revisioninfo {
-    margin-top:4px;
-}
+    margin-top: 4px;
+    }
 ul.expandrev {
-	list-style:none;
-	font:normal 11px/normal sans-serif;
-	color:#000;
-}
+    font-size: 85%;
+    }
 ul.expandrev li {
-	padding:2px 0px 2px 20px;
-	margin:0px 0px 2px 0px;
-}
+    padding: 2px 0 2px 20px;
+    margin: 0 0 2px 0;
+    }
 ul.expandrev li.mfrom {
-	background:url(../images/ico_mergefrom.gif) 0px 0px no-repeat;
-}
+    background: url(../images/ico_mergefrom.gif) 0 0 no-repeat;
+    }
 ul.expandrev li.mto {
-	background:url(../images/ico_mergeto.gif) 0px 0px no-repeat;
-}
+    background: url(../images/ico_mergeto.gif) 0 0 no-repeat;
+    }
 ul.expandrev li.committerli {
-	background:url(../images/ico_committer.gif) 2px 0px no-repeat;
-}
+    background: url(../images/ico_committer.gif) 2px 0 no-repeat;
+    }
 ul.expandrev li.desc {
-	background:url(../images/ico_description.gif) 0px 0px no-repeat;
-	color:#666;
-}
+    background: url(../images/ico_description.gif) 0 0 no-repeat;
+    color: #666;
+    }
 ul.expandrev li.files {
-	background:url(../images/ico_file.gif) 2px 0px no-repeat;
+    background: url(../images/ico_file.gif) 2px 0 no-repeat;
     margin-left: 8px;
-}
-
-/* Navigation */
-ul {border:0; margin:0; padding:0;}
-
+    }
+.revnolink a {
+    color: #000;
+    }
+
+
+/* =========================
+   Pagination
+*/
 ul#pages {
-	float:right;
-}
+    float: right;
+    }
 #pages li {
-	border:0; margin:0; padding:0;
-	font-size:12px;
-	list-style:none;
-	float:left;
-	display:inline;
-}
-#pages a {
-	border:solid 1px #DDDDDD;
-	margin-right:2px;
-}
-#pages .next a, #pages .previous a {
-	font-weight:bold;
-	border:solid 1px #fff;
-}
-#pages .active {
-	color:#f37b06;
-	font-weight:bold;
-	display:block;
-	float:left;
-	padding:4px 6px;
-}
-#pages a:link, #pages a:visited {
-	color:#f37b06;
-	display:block;
-	float:left;
-	padding:3px 6px;
-	text-decoration:none;
-}
-#pages .next a {
-	color:#0f6b97;
-}
-#pages a:hover {
-	border:solid 1px #666666;
-}
-.breadcrumb a {
-	color:#889ea9;
-	font-weight:bold;
-	font-size:18px;
-}
-.breadcrumb a:hover{
-	color:#666;
-	font-weight:bold;
-	font-size:18px;
-}
-#branch-info {
-	width:900px;
-	padding:10px 10px 10px 10px;
-	margin:0px 0px 10px 0px;
-	color:#666;
-	background:#fcfcfc;
-	border:1px solid #b7b7b7;
-}
-.show_if_js {
+    font-size: 77%;
+    font-weight: bold;
+    float: left;
+    display: inline;
+    padding: 3px 6px;
+    margin-right: 2px;
+    }
+
+
+/* =========================
+   Search & RSS
+*/
+#finderBox {
+    width: 300px;
+    float: right;
+    padding: 10px 0 0 0;
+    }
+#finderBox label,
+#finderBox input {
+    float: left;
+    font-weight: normal;
+    font-size: 12px;
+    }
+#finderBox label {
+    display: inline;
+    width: 100px;
+    text-align: right;
+    color: #000;
+    padding: 5px 5px 0 0;
+    }
+#finderBox input {
+    color: #666;
+    display: inline;
+    width: 130px;
+    padding: 3px;
+    background: #fff url(../images/bg_search_input.gif) 110px 3px no-repeat;
+    border: 1px solid #e9e9e9;
+    }
+#search_terms {
     display: none;
-}
-.revision_log {
-    overflow: hidden;
-}
+    border: solid 1px black;
+    width: 180px;
+    overflow: auto;
+    background: #fff;
+    }
+.rssfeed {
+    float: left;
+    display: inline;
+    padding: 0 0 0 10px;
+    }
+
+
+/* =========================
+   Footer
+*/
+#footer {
+    font-size: 77%;
+    }
+
+    
+/* ====================
+   Global notifications
+*/
+/* Move the content down so the notification banner doesn't hide any content. */
+body.global-notification-visible {
+    padding-top: 40px;
+    }
+body.global-notification-visible .login-logout {
+    top: 45px;
+    }
+.notification-private {
+    background-image: url(/static/images/notification-private.png); /* sprite-ref: icon-sprites */
+    background-repeat: no-repeat;
+    }
+.global-notification {
+    position: fixed;
+    z-index: 10;
+    top: 0;
+    left: 0;
+    right: 0;
+    padding: 8px 20px;
+    /* Define colour for browsers that don't support transparency */
+    background-color: #8d1f1f;
+    /* Set transparent background for browsers that support it */
+    background-color: rgba(125,0,0,0.9);
+    color: #fff;
+    text-shadow: 0 -1px 0 #631616;
+    font-size: 14px;
+    line-height: 21px;
+    font-weight: bold;
+    -moz-box-shadow: 0 0 5px #333;
+    -webkit-box-shadow: 0 0 5px #333;
+    box-shadow: 0 0 5px #333;
+    }
+.global-notification .sprite.notification-private {
+    float: left;
+    display: inline-block;
+    height: 21px;
+    width: 20px;
+    margin-right: 10px;
+    padding: 0;
+    }
+.global-notification-close, .global-notification-close:active,
+.global-notification-close:visited {
+    color: #e47a7a;
+    }
+.global-notification-close {
+    display: block;
+    position: absolute;
+    top: 11px;
+    right: 20px;
+    font-size: 12px;
+    font-weight: normal;
+    line-height: 14px;
+    }
+.global-notification-close .sprite.notification-close {
+    float: right;
+    display: block;
+    height: 9px;
+    width: 8px;
+    margin: 3px 0 0 7px;
+    padding: 0;
+    }

=== renamed file 'loggerhead/static/css/annotate.css' => 'loggerhead/static/css/view.css'
--- loggerhead/static/css/annotate.css	2009-03-01 20:57:52 +0000
+++ loggerhead/static/css/view.css	2011-04-22 17:22:26 +0000
@@ -1,29 +1,47 @@
 /*table*/
-.annoLineTit, .annoLine, .annoRevTit, .annoRev, .annoComm, .annoCommTit, .annoContTit, .annoCont {
-	width:45px;
-	border:1px solid #d2d2d2;
-}
-.annoLine, .annoRev, .annoComm, .annoCont {
-	border:none;	
-}
-.annoLine {
-	width:37px;
-}
-.annoContTit, .annoCont {
-	width:auto;
-}
-.annoRevTit, .annoRev {
-	width:70px;
-	text-align:center;
-}
-.annoComm, .annoCommTit {
-	width:200px;
-}
-.annoLine, .annoCont {
-	font:normal 12px/normal monospace;
-        whitespace: pre;
-}
-.annoCont pre { margin: 0; }
-.annoLine {
-	text-align:right;
+#logentries,
+.viewRev,
+.viewLine {
+    border: 1px solid #d2d2d2;
+}
+.viewRev {
+    border-right: none;
+    white-space: nowrap;
+    width: 1px;
+    padding: .3em .6em;
+}
+.viewComm,
+.viewCont {
+    border: none;
+    }
+.viewLine {
+    border-style: none solid none none;
+    width: 20px;
+    padding-left: .6em;
+    padding-right: .2em;
+    }
+.viewLine.first {
+    border-style: solid solid none none;
+}
+.viewCont {
+    padding-left: .4em;
+    }
+.viewLine,
+.viewCont {
+    font-size: 116%;
+    font-family: 'UbuntuBeta Mono', 'Ubuntu Mono', monospace;
+    line-height: 108%;
+    whitespace: pre;
+    }
+.viewCont pre {
+    margin: 0;
+    }
+.viewLine {
+    text-align: center;
+    }
+.viewLine {
+    font-size: 93%;
+    }
+.viewAuthors {
+    color: #999999;
 }

=== added directory 'loggerhead/static/downloads'
=== modified file 'loggerhead/static/images/bg_Tabs.gif'
Binary files loggerhead/static/images/bg_Tabs.gif and loggerhead/static/images/bg_Tabs.gif differ
=== modified file 'loggerhead/static/images/bg_menuTabs.gif'
Binary files loggerhead/static/images/bg_menuTabs.gif and loggerhead/static/images/bg_menuTabs.gif differ
=== modified file 'loggerhead/static/images/bg_submenuTabs.gif'
Binary files loggerhead/static/images/bg_submenuTabs.gif and loggerhead/static/images/bg_submenuTabs.gif differ
=== added file 'loggerhead/static/images/favicon.png'
Binary files loggerhead/static/images/favicon.png and loggerhead/static/images/favicon.png differ
=== added file 'loggerhead/static/images/notification-private.png'
Binary files loggerhead/static/images/notification-private.png and loggerhead/static/images/notification-private.png differ
=== modified file 'loggerhead/static/javascript/custom.js'
--- loggerhead/static/javascript/custom.js	2009-04-01 15:28:21 +0000
+++ loggerhead/static/javascript/custom.js	2011-10-26 16:06:54 +0000
@@ -1,4 +1,4 @@
-Y = YUI().use("node", "io-base", "anim");
+Y = YUI().use("base", "node", "io-base", "anim");
 
 var global_timeout_id = null;
 var global_search_request = null;
@@ -280,3 +280,89 @@
   }
 };
 
+var notification_node = null;
+/*
+ * Display privacy notifications
+ *
+ * This should be called after the page has loaded e.g. on 'domready'.
+ */
+function setup_privacy_notification(config) {
+    if (notification_node !== null) {
+        return;
+    }
+    var notification_text = 'The information on this page is private';
+    var hidden = true;
+    var target_id = "loggerheadCont";
+    if (config !== undefined) {
+        if (config.notification_text !== undefined) {
+            notification_text = config.notification_text;
+        }
+        if (config.hidden !== undefined) {
+            hidden = config.hidden;
+        }
+        if (config.target_id !== undefined) {
+            target_id = config.target_id;
+        }
+    }
+    var id_selector = "#" + target_id;
+    var main = Y.get(id_selector);
+    notification_node = Y.Node.create('<div></div>')
+        .addClass('global-notification');
+    if (hidden) {
+        notification_node.addClass('hidden');
+    }
+    var notification_span = Y.Node.create('<span></span>')
+        .addClass('sprite')
+        .addClass('notification-private');
+    notification_node.set('innerHTML', notification_text);
+    main.appendChild(notification_node);
+    notification_node.appendChild(notification_span);
+};
+
+function display_privacy_notification() {
+    /* Set a temporary class on the body for the feature flag,
+     this is because we have no way to use feature flags in
+     css directly. This should be removed if the feature
+     is accepted. */
+    var body = Y.get('body');
+    body.addClass('feature-flag-bugs-private-notification-enabled');
+    // Set the visible flag so that the content moves down.
+    body.addClass('global-notification-visible');
+
+    setup_privacy_notification();
+    var global_notification = Y.get('.global-notification');
+    if (global_notification.hasClass('hidden')) {
+        global_notification.addClass('transparent');
+        global_notification.removeClass('hidden');
+
+        var fade_in = new Y.Anim({
+            node: global_notification,
+            to: {opacity: 1},
+            duration: 0.3
+        });
+        var body_space = new Y.Anim({
+            node: 'body',
+            to: {'paddingTop': '40px'},
+            duration: 0.2,
+            easing: Y.Easing.easeOut
+        });
+        var black_link_space = new Y.Anim({
+            node: '.black-link',
+            to: {'top': '45px'},
+            duration: 0.2,
+            easing: Y.Easing.easeOut
+        });
+
+        fade_in.run();
+        body_space.run();
+        black_link_space.run();
+    }
+};
+
+Y.on('domready', function() {
+    var body = Y.get('body');
+    if (body.hasClass('private')) {
+        setup_privacy_notification();
+        display_privacy_notification();
+    }
+});

=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2009-10-17 08:47:38 +0000
+++ loggerhead/templatefunctions.py	2011-09-08 00:33:28 +0000
@@ -14,8 +14,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import cgi
 import os
+import urllib
 
 import pkg_resources
 
@@ -23,6 +23,7 @@
 
 import loggerhead
 from loggerhead.zptsupport import zpt
+from loggerhead.util import html_format
 
 
 templatefunctions = {}
@@ -49,16 +50,21 @@
     if style == 'fragment':
         def file_link(filename):
             if currently_showing and filename == currently_showing:
-                return '<b><a href="#%s">%s</a></b>' % (
-                    cgi.escape(filename), cgi.escape(filename))
+                return html_format(
+                    '<b><a href="#%s">%s</a></b>',
+                    urllib.quote(filename.encode('utf-8')), filename)
             else:
                 return revision_link(
-                    url, entry.revno, filename, '#' + filename)
+                    url, entry.revno, filename,
+                    '#' + urllib.quote(filename.encode('utf-8')))
     else:
         def file_link(filename):
-            return '<a href="%s%s" title="View changes to %s in revision %s">%s</a>' % (
-                url(['/revision', entry.revno]), '#' + filename, cgi.escape(filename),
-                cgi.escape(entry.revno), cgi.escape(filename))
+            return html_format(
+                '<a href="%s%s" title="View changes to %s in revision %s">'
+                '%s</a>',
+                url(['/revision', entry.revno]),
+                '#' + urllib.quote(filename.encode('utf-8')),
+                filename, entry.revno, filename)
     return _pt('revisionfilechanges').expand(
         entry=entry, file_changes=file_changes, file_link=file_link, **templatefunctions)
 
@@ -121,15 +127,17 @@
 
 
 @templatefunc
-def annotate_link(url, revno, path):
-    return '<a href="%s" title="Annotate %s">%s</a>' % (
-        url(['/annotate', revno, path]), cgi.escape(path), cgi.escape(path))
+def view_link(url, revno, path):
+    return html_format(
+        '<a href="%s" title="Annotate %s">%s</a>',
+        url(['/view', revno, path]), path, path)
+
 
 @templatefunc
 def revision_link(url, revno, path, frag=''):
-    return '<a href="%s%s" title="View changes to %s in revision %s">%s</a>' % (
-        url(['/revision', revno, path]), frag, cgi.escape(path),
-        cgi.escape(revno), cgi.escape(path))
+    return html_format(
+        '<a href="%s%s" title="View changes to %s in revision %s">%s</a>',
+        url(['/revision', revno, path]), frag, path, revno, path)
 
 
 @templatefunc
@@ -186,12 +194,8 @@
 
         # TODO: On old Python versions, elementtree may be used.
 
-        try:
-            import simplejson
-        except ImportError:
-            pass
-        else:
-            versions.append(('simplejson', simplejson.__version__))
+        import simplejson
+        versions.append(('simplejson', simplejson.__version__))
 
         try:
             Dozer = pkg_resources.get_distribution('Dozer')

=== modified file 'loggerhead/templates/atom.pt'
--- loggerhead/templates/atom.pt	2009-06-06 12:23:11 +0000
+++ loggerhead/templates/atom.pt	2010-05-05 18:42:58 +0000
@@ -4,8 +4,8 @@
     bazaar changes for <tal:branch-name content="branch/friendly_name">branch name</tal:branch-name>
   </title>
   <updated tal:content="string:${updated}Z">${updated}</updated>
-  <id tal:content="python:branch.url(['/atom'])">url</id>
-  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.url(['/atom'])" />
+  <id tal:content="python:branch.absolute_url(['/atom'])">url</id>
+  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.absolute_url(['/atom'])" />
   <link rel="alternate" type="text/html" tal:attributes="href python:branch.url(['/changes'])" />
 
   <entry tal:repeat="entry changes">
@@ -16,7 +16,7 @@
       updated
     </updated>
     <!-- TODO: The feed validator will generate warnings because this is URL-encoded -->
-    <id tal:content="python:branch.url(['/revision', entry.revno])">
+    <id tal:content="python:branch.absolute_url(['/revision', entry.revno])">
       ID
     </id>
     <author tal:repeat="author entry/authors">

=== modified file 'loggerhead/templates/branchinfo.pt'
--- loggerhead/templates/branchinfo.pt	2009-01-23 20:53:40 +0000
+++ loggerhead/templates/branchinfo.pt	2011-11-25 01:23:32 +0000
@@ -1,4 +1,5 @@
-<div id="branch-info">
+<div id="branch-info"
+  tal:condition="python:branch.served_url">
   To get this branch, use: <br/>
   <code>bzr branch
   <tal:served-url tal:replace="python:branch.served_url" /></code>

=== modified file 'loggerhead/templates/breadcrumbs.pt'
--- loggerhead/templates/breadcrumbs.pt	2008-09-30 02:04:14 +0000
+++ loggerhead/templates/breadcrumbs.pt	2012-01-08 21:43:42 +0000
@@ -10,7 +10,7 @@
 
 <!-- Branch breadcrumbs (for the path within the branch) -->
 <span metal:define-macro="branch" class="breadcrumb">
-    <tal:block repeat="crumb branch_breadcrumbs">/<a tal:attributes="href python:url([crumb['suffix'], change.revno], file_id=crumb['file_id'])" tal:content="crumb/dir_name" tal:condition="not:repeat/crumb/end" /><span tal:replace="crumb/dir_name" tal:condition="repeat/crumb/end" /></tal:block>
+    <tal:block repeat="crumb branch_breadcrumbs">/<a tal:attributes="href python:url([crumb['suffix'], change.revno, crumb['path']])" tal:content="crumb/dir_name" tal:condition="not:repeat/crumb/end" /><span tal:replace="crumb/dir_name" tal:condition="repeat/crumb/end" /></tal:block>
 </span>
 
 <!-- Directory breadcrumbs (for the path leading up to the branch) -->

=== removed file 'loggerhead/templates/browse.pt'
--- loggerhead/templates/browse.pt	2009-05-13 13:14:04 +0000
+++ loggerhead/templates/browse.pt	1970-01-01 00:00:00 +0000
@@ -1,60 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml">
-  <head>
-    <title>loggerhead branches</title>
-    <link rel="stylesheet"
-          tal:attributes="href python:branch.static_url('/static/css/global.css')" />
-  </head>
-
-  <body>
-   <div id="loggerheadCont">
-    <div>
-      <div class="browse-project"
-           tal:repeat="project projects">
-        <h1 tal:content="project/friendly_name" />
-        <div class="browse-project-description"
-             tal:condition="python:len(project.long_description) > 0"
-             tal:content="structure project/long_description"></div>
-
-          <table id="logentries">
-            <tr class="logheader">
-              <th class="summarycell" colspan="2">Name</th>
-              <th class="summarycell">Description</th>
-              <th class="datecell" colspan="2">Last change</th>
-            </tr>
-            <tal:block repeat="view python:views_by_project[project]">
-              <tr tal:attributes="class string:blueRow1">
-                <td class="icocell">
-                  <img tal:attributes="src python:branch.static_url('/static/images/ico_branch.gif')" alt="Branch" />
-                </td>
-                <td class="autcell">
-                    <a tal:attributes="href python:branch.static_url(project.name + '/' +  view.name)" 
-                       tal:content="view/friendly_name" />
-                </td>
-                <td class="autcell"
-                    tal:content="view/description" />
-                <td class="date"
-                    tal:content="python:view.get_history().has_revisions and util._approximatedate(view.last_updated()) or '(no revisions)'" />
-                <td class="icocell">
-                  <a tal:attributes="href python:branch.static_url(project.name + '/' +  view.name + '/atom')"
-                     title="atom feed">
-                    <img tal:attributes="src python:branch.static_url('/static/images/ico_rss.gif')"
-                         alt="atom feed" />
-                  </a>
-                </td>
-              </tr>
-              <tr tal:condition="view/served_url">
-                <td class="name">
-                </td>
-                <td class="description url" colspan="2">
-                  <a tal:attributes="href view/served_url"
-                     tal:content="view/served_url" />
-                </td>
-              </tr>
-            </tal:block>
-          </table>
-      </div>
-    </div>
-  </div>
-  </body>
-</html>

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2009-06-25 03:35:43 +0000
+++ loggerhead/templates/changelog.pt	2012-02-02 07:42:24 +0000
@@ -16,26 +16,36 @@
   </head>
 
   <body>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+    <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
     <tal:block metal:fill-slot="heading">
-      <h1>
-        <tal:has-link condition="branch/branch_link">
-          <a tal:attributes="href branch/branch_link"
-             tal:content="branch/friendly_name">
-            nice/branch/name
-          </a>
-        </tal:has-link>
-        <tal:no-link condition="not: branch/branch_link">
-          <span metal:use-macro="breadcrumbs/directory"></span>
-        </tal:no-link>
-        <tal:block condition="changes">
-          : changes
-          <tal:block condition="filter_file_id">to <span tal:content="python:history.get_path(revid, filter_file_id)" /></tal:block>
-          <tal:block condition="start_revid">from revision
-            <span tal:content="python:history.get_revno(start_revid)"/>
-          </tal:block>
-          <tal:block condition="query">matching <tal:block content="query" /></tal:block>
-        </tal:block>
-      </h1>
+        <div id="breadcrumbs">
+            <tal:has-link condition="branch/branch_link">
+              <a tal:attributes="href branch/branch_link"
+                 tal:content="branch/friendly_name">
+                nice/branch/name
+              </a>
+            </tal:has-link>
+            <tal:no-link condition="not: branch/branch_link">
+              <span metal:use-macro="breadcrumbs/directory"></span>
+            </tal:no-link>
+            <tal:block condition="changes">
+              &#187; Changes
+              <tal:block condition="filter_file_id">to <span tal:content="python:history.get_path(revid, filter_file_id)" /></tal:block>
+              <tal:block condition="start_revid">from revision
+                <span tal:content="python:history.get_revno(start_revid)"/>
+              </tal:block>
+              <tal:block condition="query">matching <tal:block content="query" /></tal:block>
+            </tal:block>
+        </div>
     </tal:block>
 
     <div metal:fill-slot="content">
@@ -69,14 +79,14 @@
       <!-- Table -->
       <table id="logentries">
         <tr class="logheader">
-          <td class="revisionnumber">Rev</td>
-          <td class="expandcell show_if_js">&nbsp;</td>
-          <td class="summarycell">Summary</td>
-          <td class="authorcell">Authors</td>
-          <td tal:condition="show_tag_col" class="tagscell">Tags</td>
-          <td class="datecell">Date</td>
-          <td class="diffcell">Diff</td>
-          <td class="downloadcell">Files</td>
+          <th class="revisionnumber">Rev</th>
+          <th class="expandcell show_if_js">&nbsp;</th>
+          <th class="summarycell">Summary</th>
+          <th class="authorcell">Authors</th>
+          <th tal:condition="show_tag_col" class="tagscell">Tags</th>
+          <th class="datecell">Date</th>
+          <th class="diffcell">Diff</th>
+          <th class="downloadcell">Files</th>
         </tr>
         <tal:block tal:repeat="entry changes">
           <a tal:attributes="name string:entry-${entry/revno}"/>
@@ -95,12 +105,16 @@
             </td>
             <td class="summcell">
               <div class="short_description">
+                <img tal:condition="python:entry.parents[1:]"
+                     tal:attributes="src python:branch.static_url('/static/images/ico_mergefrom.gif')" />
                 <a tal:attributes="title python:'Show revision '+entry.revno;
                                    href  python:url(['/revision', entry.revno], clear=1);
                                    class string:link"
                    tal:content="entry/short_comment"></a>
               </div>
               <div class="long_description" style="display: none">
+                <img tal:condition="python:entry.parents[1:]"
+                     tal:attributes="src python:branch.static_url('/static/images/ico_mergefrom.gif')" />
                 <a tal:attributes="title python:'Show revision '+entry.revno;
                                    href  python:url(['/revision', entry.revno], clear=1);
                                    class string:link"
@@ -115,8 +129,8 @@
             <td tal:condition="show_tag_col" tal:content="string:${entry/tags}"
                 class="tagcell"></td>
             <td class="date">
-              <span tal:attributes="title python:util.date_time(entry.date)"
-                    tal:content="python:util._approximatedate(entry.date)"></span>
+              <span tal:attributes="title python:util.date_time(entry.utc_date)"
+                    tal:content="python:util._approximatedate(entry.utc_date)"></span>
             </td>
             <td class="diffr"><a tal:attributes="title python:'Show diff at revision '+entry.revno;
                                                  href python:url(['/revision', entry.revno], clear=1)">

=== modified file 'loggerhead/templates/directory.pt'
--- loggerhead/templates/directory.pt	2008-10-18 00:23:09 +0000
+++ loggerhead/templates/directory.pt	2011-02-17 19:51:25 +0000
@@ -4,6 +4,8 @@
     <title tal:content="string:Browsing ${name}"></title>
     <link rel="stylesheet"
           tal:attributes="href python:static_url('/static/css/global.css')" />
+    <link rel="shortcut icon"
+          tal:attributes="href python:branch.static_url('/static/images/favicon.ico')" /> 
   </head>
   <body>
     <div id="loggerheadCont">
@@ -19,13 +21,13 @@
             <th class="datecell">Latest Rev</th>
             <th class="datecell">Last Changed</th>
           </tr>
-          
+
           <tr class="blueRow0" tal:condition="python:name != '/'">
             <td class="icocell">
-                <a href=".."><img tal:attributes="src python:static_url('/static/images/ico_folder_up.gif')" /></a>
+                <a href="../"><img tal:attributes="src python:static_url('/static/images/ico_folder_up.gif')" /></a>
             </td>
             <td class="summcell" colspan="3">
-                <a href="..">..</a>
+                <a href="../">..</a>
             </td>
           </tr>
           <tal:block repeat="dir dirs">
@@ -50,12 +52,12 @@
             <tal:non-branch-row tal:condition="not:dir/branch">
               <tr tal:attributes="class string:blueRow${dir/parity}">
                 <td class="icocell">
-                    <a tal:attributes="href string:${dir/dirname}">
+                    <a tal:attributes="href string:${dir/dirname}/">
                         <img tal:attributes="src python:static_url('/static/images/ico_folder.gif')" alt="Folder" />
                     </a>
                 </td>
                 <td class="autcell">
-                  <a tal:attributes="href string:${dir/dirname}" tal:content="dir/dirname" /></td>
+                  <a tal:attributes="href string:${dir/dirname}/" tal:content="dir/dirname" /></td>
                 <td class="date"></td>
                 <td class="date" tal:content="dir/last_change"></td>
               </tr>

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2009-07-15 22:02:09 +0000
+++ loggerhead/templates/inventory.pt	2012-02-02 07:42:24 +0000
@@ -9,23 +9,30 @@
     </metal:block>
   </head>
   <body>
-
-    <h1 metal:fill-slot="heading">
-      <tal:has-link condition="branch/branch_link">
-        <a tal:attributes="href branch/branch_link"
-           tal:content="branch/friendly_name">
-          nice/branch/name
-        </a>
-      </tal:has-link>
-      <span tal:condition="not: branch/branch_link">
-        <span metal:use-macro="breadcrumbs/directory" />
-      </span>
-      <tal:block condition="python:change">
-        : viewing
-        <span metal:use-macro="breadcrumbs/branch" />
-        for revision <span tal:content="change/revno"/>
-      </tal:block>
-    </h1>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+      <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
+          <tal:block metal:fill-slot="heading">
+          <div id="breadcrumbs">
+          <tal:has-link condition="branch/branch_link">
+            <a tal:attributes="href branch/branch_link"
+               tal:content="branch/friendly_name">
+              nice/branch/name
+            </a>
+          </tal:has-link>
+          <tal:no-link condition="not: branch/branch_link">
+            <span metal:use-macro="breadcrumbs/directory"></span>
+          </tal:no-link>
+          <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
+        </div>
+      </tal:block>
 
     <div metal:fill-slot="content">
         <tal:branch-info replace="structure python:branchinfo(branch)" />
@@ -70,14 +77,14 @@
                                              title string:Go to ${file/filename}" />
                     </a>
 
-                 <a tal:attributes="href python:url(['/files', revno_url, file.absolutepath])" 
+                 <a tal:attributes="href python:url(['/files', revno_url, file.absolutepath])"
                                         tal:content="file/filename" class="link"></a>
                  </td>
                  <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
                                                      title string:Show revision ${file/change/revno}"
                                      tal:content="file/change/revno"></a>
                  </td>
-                 <td class="date" tal:content="python:util.date_time(file.change.date)"></td>
+                 <td class="date" tal:content="python:util.date_time(file.change.utc_date)"></td>
                  <td class="timedate2"></td>
                  <td class="expcell"><a tal:attributes="href python:url(['/changes'], start_revid=start_revid, filter_file_id=file.file_id);
                                                      title string:Show revision ${file/change/revno}">
@@ -89,18 +96,18 @@
 
             <!-- Show this if it's a symlink -->
             <tr tal:attributes="class string:blueRow${repeat/file/even}" tal:condition="python:file.kind=='symlink'">
-                <td class="autcell"><a tal:attributes="href python:url(['/annotate', change.revno, file.absolutepath])">
+                <td class="autcell"><a tal:attributes="href python:url(['/view', change.revno, file.absolutepath])">
                         <img tal:attributes="src python:branch.static_url('/static/images/ico_file_flecha.gif')" alt="Symlink" />
                     </a>
 
-                 <a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])" 
+                 <a tal:attributes="href python:url(['/view', revno_url, file.absolutepath])"
                                         tal:content="file/filename" class="link"></a>
                  </td>
                  <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
                                                      title string:Show revision ${file/change/revno}"
                                      tal:content="file/change/revno"></a>
                  </td>
-                 <td class="date" tal:content="python:util.date_time(file.change.date)"></td>
+                 <td class="date" tal:content="python:util.date_time(file.change.utc_date)"></td>
                  <td class="timedate2">.</td>
                  <td class="expcell"><a tal:attributes="href python:url(['/changes'], start_revid=start_revid, filter_file_id=file.file_id);
                                                         title string:Show revision ${file/change/revno}">
@@ -113,26 +120,26 @@
 
             <!-- Show this if it's a regular file -->
             <tr tal:attributes="class string:blueRow${repeat/file/even}" tal:condition="python:file.kind=='file'">
-                <td class="autcell"><a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])">
+                <td class="autcell"><a tal:attributes="href python:url(['/view', revno_url, file.absolutepath])">
                         <img tal:attributes="src python:branch.static_url('/static/images/ico_file.gif');
-                                             title string:Annotate ${file/filename}"
+                                             title string:View ${file/filename}"
                              tal:condition="python:file.executable is False" />
                         <!-- Show a different icon id the file executable -->
-                        <img tal:attributes="src python:branch.static_url('/static/images/ico_file_modify.gif'); 
-                                             title string:Annotate ${file/filename}"
+                        <img tal:attributes="src python:branch.static_url('/static/images/ico_file_modify.gif');
+                                             title string:View ${file/filename}"
                              tal:condition="python:file.executable is True" alt="File" />
                     </a>
 
-              <a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])"
+              <a tal:attributes="href python:url(['/view', revno_url, file.absolutepath])"
                                      tal:content="file/filename" class="link"></a></td>
               <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
                                                   title string:Show revision ${file/change/revno}"
                                   tal:content="file/change/revno"></a>
               </td>
-              <td class="date" tal:content="python:util.date_time(file.change.date)"></td>
+              <td class="date" tal:content="python:util.date_time(file.change.utc_date)"></td>
               <td class="timedate2" tal:content="python:util.human_size(file.size)"></td>
-              <td class="expcell"><a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath]);
-                                                     title string:Annotate ${file/filename}">
+              <td class="expcell"><a tal:attributes="href python:url(['/view', revno_url, file.absolutepath]);
+                                                     title string:View ${file/filename}">
                                       <img tal:attributes="src python:branch.static_url('/static/images/ico_planilla.gif')" alt="Diff" />
                                      </a>
               </td>

=== modified file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	2009-06-24 02:35:46 +0000
+++ loggerhead/templates/macros.pt	2011-11-04 18:16:32 +0000
@@ -9,6 +9,8 @@
     <title metal:define-slot="title"></title>
     <link rel="stylesheet"
           tal:attributes="href python:branch.static_url('/static/css/global.css')" />
+    <link rel="shortcut icon"
+          tal:attributes="href python:branch.static_url('/static/images/favicon.png')" />
     <tal:comment condition="nothing">
       <script type="text/javascript"
               src="http://getfirebug.com/releases/lite/1.2/firebug-lite-compressed.js"></script>
@@ -41,27 +43,23 @@
     <metal:block metal:define-slot="header_extras" />
   </head>
 
-<body>
+ <body tal:attributes="class python:branch.public_private_css()">
     <!-- Loggerhead Content Area -->
-      <div id="finderBox">
-         <tal:search-box define="navigation navigation"
-                        replace="structure python:search_box(branch,
-                                 navigation)" />
-        <div>
-          <tal:feed-link replace="structure python:feed_link(branch, url)" />
-        </div>
-      </div>
-
+      
+      <div metal:define-slot="backlink" />
+      
+      <h1 class="branch-name">
+        <span metal:define-slot="branchname" />
+      </h1>
       <tal:menu define="fileview_active fileview_active"
                 replace="structure python:menu(branch, url, fileview_active)" />
       <div id="loggerheadCont">
 
         <div id="search_terms"></div>
-
-        <h1 metal:define-slot="heading"></h1>
+        <div metal:define-slot="heading" />
         <div metal:define-slot="content"></div>
 
-        <p class="fl">Loggerhead <tal:version content="loggerhead_version" /> is a web-based interface for <a href="http://bazaar-vcs.org/">Bazaar</a> branches</p>
+        <p id="footer" class="fl">Loggerhead <tal:version content="loggerhead_version" /> is a web-based interface for <a href="http://bazaar-vcs.org/">Bazaar</a> branches</p>
       </div>
   </body>
 </html>

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2009-04-28 03:01:54 +0000
+++ loggerhead/templates/revision.pt	2011-11-23 08:33:55 +0000
@@ -20,9 +20,18 @@
     </head>
 
     <body>
-
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+      <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
       <tal:block metal:fill-slot="heading">
-        <h1>
+        <div id="breadcrumbs">
           <tal:has-link condition="branch/branch_link">
             <a tal:attributes="href branch/branch_link"
                tal:content="branch/friendly_name">
@@ -32,7 +41,7 @@
           <tal:no-link condition="not: branch/branch_link">
             <span metal:use-macro="breadcrumbs/directory"></span>
           </tal:no-link>
-          <span class="breadcrumb">:
+          <span class="breadcrumb">&#187; Revision
             <tal:revno condition="not:specific_path" content="change/revno"></tal:revno>
             <a tal:condition="specific_path" tal:content="change/revno"
                tal:attributes="href python:url(['/revision', change.revno])"
@@ -42,20 +51,20 @@
             </tal:compare-to>
           </span>
           <span class="breadcrumb" tal:condition="specific_path">
-            : <tal:annotate content="structure python:annotate_link(url, change.revno, specific_path)" />
+            : <tal:annotate content="structure python:view_link(url, change.revno, specific_path)" />
           </span>
-        </h1>
+        </div>
         <tal:branch-info replace="structure python:branchinfo(branch)" />
-        <p tal:condition="not:specific_path">
+        <h1 tal:condition="not:specific_path">
           Viewing all changes in revision <tal:revno content="change/revno" />.
-        </p>
+        </h1>
         <p tal:condition="specific_path">
           <a tal:attributes="href python:url(['/revision', change.revno])">
             &#xAB; back to all changes in this revision
           </a>
         </p>
         <p tal:condition="specific_path">
-          Viewing changes to <tal:annotate content="structure python:annotate_link(url, change.revno, specific_path)" />
+          Viewing changes to <tal:annotate content="structure python:view_link(url, change.revno, specific_path)" />
         </p>
         <ul id="submenuTabs">
           <li id="first"><a tal:attributes="href python:url(['/files', change.revno]);
@@ -64,18 +73,21 @@
           <li tal:condition="python:remember != change.revno">
             <a tal:attributes="href python:url(['/revision', change.revno], remember=change.revno, compare_revid=None);
                                title string:compare with another revision"
-               tal:content="string:Compare with another revision"></a></li>
+               tal:content="string:compare with another revision"></a></li>
           <li tal:condition="python:(remember is not None) and (compare_revid is None) and (change.revno != remember)" >
             <a tal:attributes="href python:url(['/revision', change.revno], compare_revid=history.get_revno(remember))">
               compare with revision <tal:b content="python:history.get_revno(remember)" />
             </a>
           </li>
           <li>
-            <a tal:condition="python:len(change.parents) > 0 and compare_revid is None"
+            <a tal:condition="python:compare_revid is None"
                tal:attributes="href python:url(['/diff', change.revno], clear=1)">download diff</a>
             <a tal:condition="python:compare_revid is not None"
                tal:attributes="href python:url(['/diff', change.revno, history.get_revno(compare_revid)], clear=1)">download diff</a>
-          </li>
+           </li>
+           <li tal:condition="python:can_export">
+            <a tal:attributes="href python:url(['/tarball', change.revno])">download tarball</a>
+           </li>
           <li id="last"><a tal:attributes="href python:url(['/changes', change.revno]);
                                            title string:view history from revision ${change/revno}"
                            tal:content="string:view history from revision ${change/revno}"></a></li>

=== modified file 'loggerhead/templates/revisionfilechanges.pt'
--- loggerhead/templates/revisionfilechanges.pt	2009-03-19 00:05:41 +0000
+++ loggerhead/templates/revisionfilechanges.pt	2012-01-08 23:01:10 +0000
@@ -36,6 +36,7 @@
       <tal:old-name condition="not:renamed/text_modified" content="renamed/new_filename" class="filename">
         new_filename
       </tal:old-name>
+      <tal:prop-change condition="renamed/exe_change" content="string:*" />
     </li>
   </ul>
 
@@ -44,7 +45,9 @@
       <li class="desc">files modified:</li>
     </tal:block>
     <li class="files">
-      <tal:revision content="structure python:file_link(item.filename)" />
+      <tal:revision condition="item/text_modified" content="structure python:file_link(item.filename)" />
+      <tal:revision condition="not:item/text_modified" content="item/filename" />
+      <tal:prop-change condition="item/exe_change" content="string:*" />
     </li>
   </ul>
 </tal:block>

=== modified file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	2009-06-11 21:16:27 +0000
+++ loggerhead/templates/revisioninfo.pt	2012-02-02 07:42:24 +0000
@@ -1,23 +1,49 @@
 <div class="infoContainer">
     <div id="infTxt">
-        <div class="committer" tal:content="python:', '.join(util.hide_emails(change.authors))"></div>
-        <div class="timer" tal:content="python:util.date_time(change.date)"></div>
-        <div class="mfrom" tal:repeat="parent python:change.parents[1:]">
-            <a tal:attributes="title string:Merged From ${parent/revno};
-                               href python:url(['/revision', parent.revno], clear=1)"
-               tal:content="python:revno_with_nick(parent)"></a>
-        </div>
-        <div class="mto" tal:repeat="merge_point change/merge_points">
-            <a tal:attributes="title string:Merged In ${merge_point/revno};
-                               href python:url(['/revision', merge_point.revno], clear=1)"
-               tal:content="python:revno_with_nick(merge_point)"></a>
-        </div>
-        <div class="mto" tal:condition="merged_in">
-          This revision was merged to the branch mainline in
-          <a tal:attributes="href python:url(['/revision', merged_in], clear=1)">revision
-            <tal:b content="merged_in"/></a>.
-        </div>
-        <div class="revid" tal:content="string:Revision ID: ${change/revid}"></div>
+        <ul>
+            <li class="committer">
+                <strong>Committer:</strong>
+                <span tal:content="python:util.hide_email(change.committer)"></span>
+            </li>
+            <li class="authors" tal:condition="python:len(change.authors) > 1 or change.authors[0] != change.committer">
+                <strong>Author(s):</strong>
+                <span tal:content="python:', '.join(util.hide_emails(change.authors))"></span>
+            </li>
+            <li class="timer">
+                <strong>Date:</strong>
+                <span tal:content="python:util.date_time(change.utc_date)"></span>
+            </li>
+            <li class="mfrom" tal:condition="python:change.parents[1:]">
+                <strong>mfrom:</strong>
+                <span tal:repeat="parent python:change.parents[1:]">
+                    <a tal:attributes="title string:Merged From ${parent/revno};
+                                       href python:url(['/revision', parent.revno], clear=1)"
+                       tal:content="python:revno_with_nick(parent)"></a>
+                </span>
+            </li>
+            <li class="mto" tal:condition="change/merge_points">
+                <strong>mto:</strong>
+                <span tal:repeat="merge_point change/merge_points">
+                    <a tal:attributes="title string:Merged In ${merge_point/revno};
+                                       href python:url(['/revision', merge_point.revno], clear=1)"
+                       tal:content="python:revno_with_nick(merge_point)"></a>
+                </span>
+            </li>
+            <li class="mto" tal:condition="merged_in">
+                <strong>mto:</strong>
+                  This revision was merged to the branch mainline in
+                  <a tal:attributes="href python:url(['/revision', merged_in], clear=1)">revision
+                    <tal:b content="merged_in"/></a>.
+            </li>
+            <li class="revid">
+                <strong>Revision ID:</strong>
+                <span tal:content="change/revid"></span>
+            </li>
+        </ul>
+	<div tal:attributes="class string:foreign-${change/foreign_vcs}"
+	     tal:condition="change/foreign_vcs" 
+	     tal:repeat="revid_part python:change.foreign_revid.keys()"
+	     tal:content="python:'%s: %s' % (revid_part, change.foreign_revid[revid_part])"/>
         <div class="tags" tal:condition="change/tags" tal:content="string:Tags: ${change/tags}"></div>
         <div class="clear"><!-- --></div>
         <div class="bug" tal:repeat="bug change/bugs">

=== renamed file 'loggerhead/templates/annotate.pt' => 'loggerhead/templates/view.pt'
--- loggerhead/templates/annotate.pt	2009-03-06 09:57:54 +0000
+++ loggerhead/templates/view.pt	2012-02-02 07:42:24 +0000
@@ -3,31 +3,42 @@
 <html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
   <head>
     <title metal:fill-slot="title"
-           tal:content="string:${branch/friendly_name} : contents of ${path}
+           tal:content="string:${branch/friendly_name} : contents of ${file_path}
                         at revision ${change/revno}">
     </title>
     <metal:block fill-slot="header_extras">
         <link rel="stylesheet" type="text/css" media="all"
-              tal:attributes="href python:branch.static_url('/static/css/annotate.css')"/>
+              tal:attributes="href python:branch.static_url('/static/css/view.css')"/>
         <link rel="stylesheet" type="text/css" media="all"
               tal:attributes="href python:branch.static_url('/static/css/highlight.css')"/>
     </metal:block>
   </head>
 
   <body>
-
-    <h1 metal:fill-slot="heading">
-      <tal:has-link condition="branch/branch_link">
-        <a tal:attributes="href branch/branch_link"
-           tal:content="branch/friendly_name">
-          nice/branch/name
-        </a>
-      </tal:has-link>
-      <tal:no-link condition="not: branch/branch_link">
-        <span metal:use-macro="breadcrumbs/directory"></span>
-      </tal:no-link>
-      <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
-    </h1>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+      <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
+          <tal:block metal:fill-slot="heading">
+          <div id="breadcrumbs">
+          <tal:has-link condition="branch/branch_link">
+            <a tal:attributes="href branch/branch_link"
+               tal:content="branch/friendly_name">
+              nice/branch/name
+            </a>
+          </tal:has-link>
+          <tal:no-link condition="not: branch/branch_link">
+            <span metal:use-macro="breadcrumbs/directory"></span>
+          </tal:no-link>
+          <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
+        </div>
+      </tal:block>
 
     <div metal:fill-slot="content">
       <tal:branch-info replace="structure python:branchinfo(branch)" />
@@ -36,6 +47,14 @@
             <li id="first">
                 <a tal:attributes="href python:url(['/files', change.revno], clear=1)">browse files</a>
             </li>
+            <li tal:condition="not:annotated">
+              <a tal:attributes="href python:url(['/annotate', revno_url, file_path], clear=1)"
+              >view with revision information</a>
+            </li>
+            <li tal:condition="annotated">
+              <a tal:attributes="href python:url(['/view', revno_url, file_path], clear=1)"
+              >view without revision information</a>
+            </li>
             <li>
                 <a tal:attributes="href python:url(['/revision', change.revno], clear=1)">view revision</a>
             </li>
@@ -50,25 +69,29 @@
             </li>
         </ul>
 
-      <div class="annotate">
+      <div class="view">
         <table id="logentries">
-          <tr class="logheader">
-            <td class="annoLineTit">Line</td>
-            <td class="annoRevTit">Revision</td>
-            <td class="annoContTit">Contents</td>
-          </tr>
+          <tal:rep tal:repeat="line contents">
+            <tr>
+              <td tal:condition="python:path('repeat/line/number') in annotated"
+                  tal:define="anno python:annotated.get(path('repeat/line/number'), None)"
+                  tal:attributes="rowspan python:anno.revspan"
+                  class="viewRev">
+                <a tal:content="python:'%s' % (anno.change.revno,)"
+                   tal:attributes="href python:url(['/revision', anno.change.revno], clear=1);
+                                   title python:'%s by %s, on %s (%s)' % (anno.change.revno, ', '.join(util.hide_emails(anno.change.authors)), anno.change.utc_date.strftime('%d %b %Y %H:%M UTC'), util.date_time(anno.change.utc_date))"></a>
+                <span class="viewAuthors" tal:content="python:'by ' + ', '.join(util.hide_emails(anno.change.authors))" />
+                <br />
 
-          <tr tal:repeat="line contents" 
-              tal:attributes="class string:blueRow${line/parity}">
-            <td class="annoLine"><a tal:attributes="id string:L${line/lineno}; href string:#L${line/lineno}" tal:content="line/lineno">1</a></td>
-            <td class="annoRev">
-              <a tal:condition="python:line.status=='changed'"
-                 tal:content="python:util.trunc(line.change.revno)"
-                 tal:attributes="href python:url(['/revision', line.change.revno], clear=1);
-                                 title python:'%s by %s, on %s (%s)'%(line.change.revno, ', '.join(util.hide_emails(line.change.authors)), line.change.date.strftime('%d %b %Y %H:%M'), util.date_time(line.change.date))"></a>
-            </td>
-            <td class="annoCont"><pre tal:content="structure line/text"></pre></td>
-          </tr>
+                <span tal:content="python:anno.message"></span>
+              </td>
+              <td tal:attributes="class python:path('repeat/line/number') in annotated and 'viewLine first' or 'viewLine'">
+                <a tal:attributes="id string:L${repeat/line/number}; href string:#L${repeat/line/number}"
+                   tal:content="repeat/line/number">1</a>
+              </td>
+              <td class="viewCont"><pre tal:content="structure line"></pre></td>
+            </tr>
+          </tal:rep>
         </table>
       </div>
 

=== modified file 'loggerhead/tests/__init__.py'
--- loggerhead/tests/__init__.py	2006-12-11 06:44:19 +0000
+++ loggerhead/tests/__init__.py	2011-09-08 00:33:28 +0000
@@ -0,0 +1,31 @@
+# Copyright 2006, 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+def load_tests(standard_tests, module, loader):
+    standard_tests.addTests(loader.loadTestsFromModuleNames([
+        (__name__ + '.' + x) for x in [
+            'test_controllers',
+            'test_corners',
+            'test_history',
+            'test_http_head',
+            'test_load_test',
+            'test_simple',
+            'test_revision_ui',
+            'test_templating',
+            'test_util',
+        ]]))
+    return standard_tests

=== added file 'loggerhead/tests/fixtures.py'
--- loggerhead/tests/fixtures.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/fixtures.py	2011-11-25 02:28:56 +0000
@@ -0,0 +1,43 @@
+# Copyright (C) 2007, 2008, 2009, 2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from __future__ import absolute_import
+
+from fixtures import Fixture
+
+
+class SampleBranch(Fixture):
+
+    def __init__(self, testcase):
+        # Must be a bzr TestCase to hook into branch creation, unfortunately.
+        self.testcase = testcase
+
+    def setUp(self):
+        Fixture.setUp(self)
+
+        self.tree = self.testcase.make_branch_and_tree('.')
+
+        self.filecontents = (
+            'some\nmultiline\ndata\n'
+            'with<htmlspecialchars\n')
+        filenames = ['myfilename', 'anotherfile<']
+        self.testcase.build_tree_contents(
+            (filename, self.filecontents) for filename in filenames)
+        for filename in filenames:
+            self.tree.add(filename, '%s-id' % filename)
+        self.fileid = self.tree.path2id('myfilename')
+        self.msg = 'a very exciting commit message <'
+        self.revid = self.tree.commit(message=self.msg)

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2009-03-19 18:48:15 +0000
+++ loggerhead/tests/test_controllers.py	2011-11-25 01:41:07 +0000
@@ -1,80 +1,369 @@
+# Copyright (C) 2008-2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+import logging
+import tarfile
+import tempfile
+
+from paste.fixture import (
+    AppError,
+    )
+from paste.httpexceptions import HTTPServerError
+
+from testtools.matchers import (
+    Matcher,
+    Mismatch,
+    )
+
+from bzrlib import errors
+import simplejson
+
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.inventory_ui import InventoryUI
-from loggerhead.controllers.revision_ui import RevisionUI
-from loggerhead.tests.test_simple import BasicTests
-from loggerhead import util
+from loggerhead.tests.test_simple import (
+    BasicTests,
+    consume_app,
+    TestWithSimpleTree,
+    )
 
 
 class TestInventoryUI(BasicTests):
 
+    def make_bzrbranch_for_tree_shape(self, shape):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(shape)
+        tree.smart_add([])
+        tree.commit('')
+        self.addCleanup(tree.branch.lock_read().unlock)
+        return tree.branch
+
     def make_bzrbranch_and_inventory_ui_for_tree_shape(self, shape):
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(shape)
-        tree.smart_add([])
-        tree.commit('')
-        tree.branch.lock_read()
-        self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch)
-        return tree.branch, InventoryUI(branch_app, branch_app.get_history)
+        branch = self.make_bzrbranch_for_tree_shape(shape)
+        branch_app = self.make_branch_app(branch)
+        return branch, InventoryUI(branch_app, branch_app.get_history)
 
     def test_get_filelist(self):
         bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
             ['filename'])
         inv = bzrbranch.repository.get_inventory(bzrbranch.last_revision())
-        self.assertEqual(1, len(inv_ui.get_filelist(inv, '', 'filename')))
+        self.assertEqual(1, len(inv_ui.get_filelist(inv, '', 'filename', 'head')))
+
+    def test_smoke(self):
+        bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
+            ['filename'])
+        start, content = consume_app(inv_ui,
+            {'SCRIPT_NAME': '/files', 'PATH_INFO': ''})
+        self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
+                         start)
+        self.assertContainsRe(content, 'filename')
+
+    def test_no_content_for_HEAD(self):
+        bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
+            ['filename'])
+        start, content = consume_app(inv_ui,
+            {'SCRIPT_NAME': '/files', 'PATH_INFO': '',
+             'REQUEST_METHOD': 'HEAD'})
+        self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
+                         start)
+        self.assertEqual('', content)
+
+    def test_get_values_smoke(self):
+        branch = self.make_bzrbranch_for_tree_shape(['a-file'])
+        branch_app = self.make_branch_app(branch)
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/files'}
+        inv_ui = branch_app.lookup_app(env)
+        inv_ui.parse_args(env)
+        values = inv_ui.get_values('', {}, {})
+        self.assertEqual('a-file', values['filelist'][0].filename)
+
+    def test_json_render_smoke(self):
+        branch = self.make_bzrbranch_for_tree_shape(['a-file'])
+        branch_app = self.make_branch_app(branch)
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/+json/files'}
+        inv_ui = branch_app.lookup_app(env)
+        self.assertOkJsonResponse(inv_ui, env)
 
 
 class TestRevisionUI(BasicTests):
 
-    def make_bzrbranch_and_revision_ui_for_tree_shapes(self, shape1, shape2):
+    def make_branch_app_for_revision_ui(self, shape1, shape2):
         tree = self.make_branch_and_tree('.')
         self.build_tree_contents(shape1)
         tree.smart_add([])
-        tree.commit('')
+        tree.commit('msg 1', rev_id='rev-1')
         self.build_tree_contents(shape2)
         tree.smart_add([])
-        tree.commit('')
-        tree.branch.lock_read()
-        self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch)
-        branch_app._environ = {
-            'wsgi.url_scheme':'',
-            'SERVER_NAME':'',
-            'SERVER_PORT':'80',
-            }
-        branch_app._url_base = ''
-        branch_app.friendly_name = ''
-        return tree.branch, RevisionUI(branch_app, branch_app.get_history)
+        tree.commit('msg 2', rev_id='rev-2')
+        branch = tree.branch
+        self.addCleanup(branch.lock_read().unlock)
+        return self.make_branch_app(branch)
 
     def test_get_values(self):
-        branch, rev_ui = self.make_bzrbranch_and_revision_ui_for_tree_shapes(
-            [], [])
-        rev_ui.args = ['2']
-        util.set_context({})
-        self.assertIsInstance(
-            rev_ui.get_values('', {}, []),
-            dict)
+        branch_app = self.make_branch_app_for_revision_ui([], [])
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/revision/2'}
+        rev_ui = branch_app.lookup_app(env)
+        rev_ui.parse_args(env)
+        self.assertIsInstance(rev_ui.get_values('', {}, []), dict)
+
+    def test_add_template_values(self):
+        branch_app = self.make_branch_app_for_revision_ui(
+                [('file', 'content\n')], [('file', 'new content\n')])
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/revision/1/non-existent-file',
+               'QUERY_STRING':'start_revid=1' }
+        revision_ui = branch_app.lookup_app(env)
+        path = revision_ui.parse_args(env)
+        values = revision_ui.get_values(path, revision_ui.kwargs, {})
+        revision_ui.add_template_values(values)
+        self.assertIs(values['diff_chunks'], None)
+
+    def test_get_values_smoke(self):
+        branch_app = self.make_branch_app_for_revision_ui(
+                [('file', 'content\n'), ('other-file', 'other\n')],
+                [('file', 'new content\n')])
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/revision/head:'}
+        revision_ui = branch_app.lookup_app(env)
+        revision_ui.parse_args(env)
+        values = revision_ui.get_values('', {}, {})
+
+        self.assertEqual(values['revid'], 'rev-2')
+        self.assertEqual(values['change'].comment, 'msg 2')
+        self.assertEqual(values['file_changes'].modified[0].filename, 'file')
+        self.assertEqual(values['merged_in'], None)
+
+    def test_json_render_smoke(self):
+        branch_app = self.make_branch_app_for_revision_ui(
+                [('file', 'content\n'), ('other-file', 'other\n')],
+                [('file', 'new content\n')])
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/+json/revision/head:'}
+        revision_ui = branch_app.lookup_app(env)
+        self.assertOkJsonResponse(revision_ui, env)
+
 
 
 class TestAnnotateUI(BasicTests):
 
     def make_annotate_ui_for_file_history(self, file_id, rev_ids_texts):
         tree = self.make_branch_and_tree('.')
-        open('filename', 'w').write('')
+        self.build_tree_contents([('filename', '')])
         tree.add(['filename'], [file_id])
-        for rev_id, text in rev_ids_texts:
-            open('filename', 'w').write(text)
-            tree.commit(rev_id=rev_id, message='.')
+        for rev_id, text, message in rev_ids_texts:
+            self.build_tree_contents([('filename', text)])
+            tree.commit(rev_id=rev_id, message=message)
         tree.branch.lock_read()
         self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch)
+        branch_app = BranchWSGIApp(tree.branch, friendly_name='test_name')
         return AnnotateUI(branch_app, branch_app.get_history)
 
     def test_annotate_file(self):
-        history = [('rev1', 'old\nold\n'), ('rev2', 'new\nold\n')]
+        history = [('rev1', 'old\nold\n', '.'), ('rev2', 'new\nold\n', '.')]
         ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
-        annotated = list(ann_ui.annotate_file('file_id', 'rev2'))
+        # A lot of this state is set up by __call__, but we'll do it directly
+        # here.
+        ann_ui.args = ['rev2']
+        annotate_info = ann_ui.get_values('filename',
+            kwargs={'file_id': 'file_id'}, headers={})
+        annotated = annotate_info['annotated']
         self.assertEqual(2, len(annotated))
-        self.assertEqual('2', annotated[0].change.revno)
-        self.assertEqual('1', annotated[1].change.revno)
+        self.assertEqual('2', annotated[1].change.revno)
+        self.assertEqual('1', annotated[2].change.revno)
+
+    def test_annotate_empty_comment(self):
+        # Testing empty comment handling without breaking
+        history = [('rev1', 'old\nold\n', '.'), ('rev2', 'new\nold\n', '')]
+        ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
+        ann_ui.args = ['rev2']
+        annotate_info = ann_ui.get_values('filename',
+            kwargs={'file_id': 'file_id'}, headers={})
+
+    def test_annotate_file_zero_sized(self):
+        # Test against a zero-sized file without breaking. No annotation must be present.
+        history = [('rev1', '' , '.')]
+        ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
+        ann_ui.args = ['rev1']
+        annotate_info = ann_ui.get_values('filename',
+            kwargs={'file_id': 'file_id'}, headers={})
+        annotated = annotate_info['annotated']
+        self.assertEqual(0, len(annotated))
+
+
+class TestFileDiffUI(BasicTests):
+
+    def make_branch_app_for_filediff_ui(self):
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('rev-1-id', None, [
+            ('add', ('', 'root-id', 'directory', '')),
+            ('add', ('filename', 'f-id', 'file', 'content\n'))],
+            message="First commit.")
+        builder.build_snapshot('rev-2-id', None, [
+            ('modify', ('f-id', 'new content\n'))])
+        builder.finish_series()
+        branch = builder.get_branch()
+        self.addCleanup(branch.lock_read().unlock)
+        return self.make_branch_app(branch)
+
+    def test_get_values_smoke(self):
+        branch_app = self.make_branch_app_for_filediff_ui()
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/+filediff/rev-2-id/rev-1-id/f-id'}
+        filediff_ui = branch_app.lookup_app(env)
+        filediff_ui.parse_args(env)
+        values = filediff_ui.get_values('', {}, {})
+        chunks = values['chunks']
+        self.assertEqual('insert', chunks[0].diff[1].type)
+        self.assertEqual('new content', chunks[0].diff[1].line)
+
+    def test_json_render_smoke(self):
+        branch_app = self.make_branch_app_for_filediff_ui()
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/+json/+filediff/rev-2-id/rev-1-id/f-id'}
+        filediff_ui = branch_app.lookup_app(env)
+        self.assertOkJsonResponse(filediff_ui, env)
+
+
+class TestRevLogUI(BasicTests):
+
+    def make_branch_app_for_revlog_ui(self):
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('rev-id', None, [
+            ('add', ('', 'root-id', 'directory', '')),
+            ('add', ('filename', 'f-id', 'file', 'content\n'))],
+            message="First commit.")
+        builder.finish_series()
+        branch = builder.get_branch()
+        self.addCleanup(branch.lock_read().unlock)
+        return self.make_branch_app(branch)
+
+    def test_get_values_smoke(self):
+        branch_app = self.make_branch_app_for_revlog_ui()
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/+revlog/rev-id'}
+        revlog_ui = branch_app.lookup_app(env)
+        revlog_ui.parse_args(env)
+        values = revlog_ui.get_values('', {}, {})
+        self.assertEqual(values['file_changes'].added[1].filename, 'filename')
+        self.assertEqual(values['entry'].comment, "First commit.")
+
+    def test_json_render_smoke(self):
+        branch_app = self.make_branch_app_for_revlog_ui()
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/+json/+revlog/rev-id'}
+        revlog_ui = branch_app.lookup_app(env)
+        self.assertOkJsonResponse(revlog_ui, env)
+
+
+class TestControllerHooks(BasicTests):
+
+    def test_dummy_hook(self):
+        return
+        # A hook that returns None doesn't influence the searching for
+        # a controller.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: None
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertRaises(KeyError, app.lookup_app, env)
+
+    def test_working_hook(self):
+        # A hook can provide an app to use for a particular request.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: "I am hooked"
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertEquals("I am hooked", app.lookup_app(env))
+
+
+class IsTarfile(Matcher):
+
+    def __init__(self, compression):
+        self.compression = compression
+
+    def match(self, content_bytes):
+        f = tempfile.NamedTemporaryFile()
+        try:
+            f.write(content_bytes)
+            f.flush()
+            tarfile.open(f.name, mode='r|' + self.compression)
+        finally:
+            f.close()
+
+
+class MatchesTarballHeaders(Matcher):
+
+    def __init__(self, expect_filename):
+        self.expect_filename = expect_filename
+
+    def match(self, response):
+        # Maybe the c-t should be more specific, but this is probably good for
+        # making sure it gets saved without the client trying to decompress it
+        # or anything.
+        if (response.header('Content-Type') == 'application/octet-stream'
+            and response.header('Content-Disposition') ==
+            "attachment; filename*=utf-8''" + self.expect_filename):
+            pass
+        else:
+            return Mismatch("wrong response headers: %r"
+                % response.headers)
+
+
+class TestDownloadTarballUI(TestWithSimpleTree):
+
+    def setUp(self):
+        super(TestDownloadTarballUI, self).setUp()
+
+    def test_download_tarball(self):
+        # Tarball downloads are enabled by default.
+        app = self.setUpLoggerhead()
+        response = app.get('/tarball')
+        self.assertThat(
+            response.body,
+            IsTarfile('gz'))
+        self.assertThat(
+            response,
+            MatchesTarballHeaders('branch.tgz'))
+
+    def test_download_tarball_of_version(self):
+        app = self.setUpLoggerhead()
+        response = app.get('/tarball/1')
+        self.assertThat(
+            response.body,
+            IsTarfile('gz'))
+        self.assertThat(
+            response,
+            MatchesTarballHeaders('branch-r1.tgz'))
+
+    def test_download_tarball_forbidden(self):
+        app = self.setUpLoggerhead(export_tarballs=False)
+        e = self.assertRaises(
+            AppError, 
+            app.get,
+            '/tarball')
+        self.assertContainsRe(
+            str(e),
+            '(?s).*403 Forbidden'
+            '.*Tarball downloads are not allowed')

=== added file 'loggerhead/tests/test_history.py'
--- loggerhead/tests/test_history.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_history.py	2011-03-25 13:09:10 +0000
@@ -0,0 +1,328 @@
+# Copyright (C) 2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+"""Direct tests of the loggerhead/history.py module"""
+
+from bzrlib.foreign import (
+    ForeignRevision,
+    ForeignVcs,
+    VcsMapping,
+    )
+
+from datetime import datetime
+from bzrlib import tag, tests
+
+from loggerhead import history as _mod_history
+
+
+class TestCaseWithExamples(tests.TestCaseWithMemoryTransport):
+
+    def make_linear_ancestry(self):
+        # Time goes up
+        # rev-3
+        #  |
+        # rev-2
+        #  |
+        # rev-1
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('rev-1', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        builder.build_snapshot('rev-2', ['rev-1'], [])
+        builder.build_snapshot('rev-3', ['rev-2'], [])
+        builder.finish_series()
+        b = builder.get_branch()
+        self.addCleanup(b.lock_read().unlock)
+        return _mod_history.History(b, {})
+
+    def make_long_linear_ancestry(self):
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('A', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        for r in "BCDEFGHIJKLMNOPQRSTUVWXYZ":
+            builder.build_snapshot(r, None, [])
+        builder.finish_series()
+        b = builder.get_branch()
+        self.addCleanup(b.lock_read().unlock)
+        return _mod_history.History(b, {})
+
+    def make_merged_ancestry(self):
+        # Time goes up
+        # rev-3
+        #  |  \
+        #  |  rev-2
+        #  |  /
+        # rev-1
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('rev-1', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        builder.build_snapshot('rev-2', ['rev-1'], [])
+        builder.build_snapshot('rev-3', ['rev-1', 'rev-2'], [])
+        builder.finish_series()
+        b = builder.get_branch()
+        self.addCleanup(b.lock_read().unlock)
+        return _mod_history.History(b, {})
+
+    def make_deep_merged_ancestry(self):
+        # Time goes up
+        # F
+        # |\
+        # | E
+        # | |\
+        # | | D
+        # | |/
+        # B C
+        # |/
+        # A
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('A', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        builder.build_snapshot('B', ['A'], [])
+        builder.build_snapshot('C', ['A'], [])
+        builder.build_snapshot('D', ['C'], [])
+        builder.build_snapshot('E', ['C', 'D'], [])
+        builder.build_snapshot('F', ['B', 'E'], [])
+        builder.finish_series()
+        b = builder.get_branch()
+        self.addCleanup(b.lock_read().unlock)
+        return _mod_history.History(b, {})
+
+    def assertRevidsFrom(self, expected, history, search_revs, tip_rev):
+        self.assertEqual(expected,
+                         list(history.get_revids_from(search_revs, tip_rev)))
+
+
+class _DictProxy(object):
+
+    def __init__(self, d):
+        self._d = d
+        self._accessed = set()
+        self.__setitem__ = d.__setitem__
+
+    def __getitem__(self, name):
+        self._accessed.add(name)
+        return self._d[name]
+
+    def __len__(self):
+        return len(self._d)
+
+
+def track_rev_info_accesses(h):
+    """Track __getitem__ access to History._rev_info,
+
+    :return: set of items accessed
+    """
+    h._rev_info = _DictProxy(h._rev_info)
+    return h._rev_info._accessed
+
+
+class TestHistoryGetRevidsFrom(TestCaseWithExamples):
+
+    def test_get_revids_from_simple_mainline(self):
+        history = self.make_linear_ancestry()
+        self.assertRevidsFrom(['rev-3', 'rev-2', 'rev-1'],
+                              history, None, 'rev-3')
+
+    def test_get_revids_from_merged_mainline(self):
+        history = self.make_merged_ancestry()
+        self.assertRevidsFrom(['rev-3', 'rev-1'],
+                              history, None, 'rev-3')
+
+    def test_get_revids_given_one_rev(self):
+        history = self.make_merged_ancestry()
+        # rev-3 was the first mainline revision to see rev-2.
+        self.assertRevidsFrom(['rev-3'], history, ['rev-2'], 'rev-3')
+
+    def test_get_revids_deep_ancestry(self):
+        history = self.make_deep_merged_ancestry()
+        self.assertRevidsFrom(['F'], history, ['F'], 'F')
+        self.assertRevidsFrom(['F'], history, ['E'], 'F')
+        self.assertRevidsFrom(['F'], history, ['D'], 'F')
+        self.assertRevidsFrom(['F'], history, ['C'], 'F')
+        self.assertRevidsFrom(['B'], history, ['B'], 'F')
+        self.assertRevidsFrom(['A'], history, ['A'], 'F')
+
+    def test_get_revids_doesnt_over_produce_simple_mainline(self):
+        # get_revids_from shouldn't walk the whole ancestry just to get the
+        # answers for the first few revisions.
+        history = self.make_long_linear_ancestry()
+        accessed = track_rev_info_accesses(history)
+        result = history.get_revids_from(None, 'Z')
+        self.assertEqual(set(), accessed)
+        self.assertEqual('Z', result.next())
+        # We already know 'Z' because we passed it in.
+        self.assertEqual(set(), accessed)
+        self.assertEqual('Y', result.next())
+        self.assertEqual(set([history._rev_indices['Z']]), accessed)
+
+    def test_get_revids_doesnt_over_produce_for_merges(self):
+        # get_revids_from shouldn't walk the whole ancestry just to get the
+        # answers for the first few revisions.
+        history = self.make_long_linear_ancestry()
+        accessed = track_rev_info_accesses(history)
+        result = history.get_revids_from(['X', 'V'], 'Z')
+        self.assertEqual(set(), accessed)
+        self.assertEqual('X', result.next())
+        # We access 'W' because we are checking that W wasn't merged into X.
+        # The important bit is that we aren't getting the whole ancestry.
+        self.assertEqual(set([history._rev_indices[x] for x in 'ZYXW']),
+                         accessed)
+        self.assertEqual('V', result.next())
+        self.assertEqual(set([history._rev_indices[x] for x in 'ZYXWVU']),
+                         accessed)
+        self.assertRaises(StopIteration, result.next)
+        self.assertEqual(set([history._rev_indices[x] for x in 'ZYXWVU']),
+                         accessed)
+
+
+
+class TestHistoryChangeFromRevision(tests.TestCaseWithTransport):
+
+    def make_single_commit(self):
+        tree = self.make_branch_and_tree('test')
+        rev_id = tree.commit('Commit Message', timestamp=1299838474.317,
+            timezone=3600, committer='Joe Example <joe@example.com>',
+            revprops={})
+        self.addCleanup(tree.branch.lock_write().unlock)
+        rev = tree.branch.repository.get_revision(rev_id)
+        history = _mod_history.History(tree.branch, {})
+        return history, rev
+
+    def test_simple(self):
+        history, rev = self.make_single_commit()
+        change = history._change_from_revision(rev)
+        self.assertEqual(rev.revision_id, change.revid)
+        self.assertEqual(datetime.fromtimestamp(1299838474.317),
+                         change.date)
+        self.assertEqual(datetime.utcfromtimestamp(1299838474.317),
+                         change.utc_date)
+        self.assertEqual(['Joe Example <joe@example.com>'],
+                         change.authors)
+        self.assertEqual('test', change.branch_nick)
+        self.assertEqual('Commit Message', change.short_comment)
+        self.assertEqual('Commit Message', change.comment)
+        self.assertEqual(['Commit&nbsp;Message'], change.comment_clean)
+        self.assertEqual([], change.parents)
+        self.assertEqual([], change.bugs)
+        self.assertEqual(None, change.tags)
+
+    def test_tags(self):
+        history, rev = self.make_single_commit()
+        b = history._branch
+        b.tags.set_tag('tag-1', rev.revision_id)
+        b.tags.set_tag('tag-2', rev.revision_id)
+        b.tags.set_tag('Tag-10', rev.revision_id)
+        change = history._change_from_revision(rev)
+        # If available, tags are 'naturally' sorted. (sorting numbers in order,
+        # and ignoring case, etc.)
+        if getattr(tag, 'sort_natural', None) is not None:
+            self.assertEqual('tag-1, tag-2, Tag-10', change.tags)
+        else:
+            self.assertEqual('Tag-10, tag-1, tag-2', change.tags)
+
+    def test_committer_vs_authors(self):
+        tree = self.make_branch_and_tree('test')
+        rev_id = tree.commit('Commit Message', timestamp=1299838474.317,
+            timezone=3600, committer='Joe Example <joe@example.com>',
+            revprops={'authors': u'A Author <aauthor@example.com>\n'
+                                 u'B Author <bauthor@example.com>'})
+        self.addCleanup(tree.branch.lock_write().unlock)
+        rev = tree.branch.repository.get_revision(rev_id)
+        history = _mod_history.History(tree.branch, {})
+        change = history._change_from_revision(rev)
+        self.assertEqual(u'Joe Example <joe@example.com>',
+                         change.committer)
+        self.assertEqual([u'A Author <aauthor@example.com>',
+                          u'B Author <bauthor@example.com>'],
+                         change.authors)
+
+
+class TestHistory_IterateSufficiently(tests.TestCase):
+
+    def assertIterate(self, expected, iterable, stop_at, extra_rev_count):
+        self.assertEqual(expected, _mod_history.History._iterate_sufficiently(
+            iterable, stop_at, extra_rev_count))
+
+    def test_iter_no_extra(self):
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(['a', 'b', 'c'], iter(lst), 'c', 0)
+        self.assertIterate(['a', 'b', 'c', 'd'], iter(lst), 'd', 0)
+
+    def test_iter_not_found(self):
+        # If the key in question isn't found, we just exhaust the list
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(lst, iter(lst), 'not-there', 0)
+
+    def test_iter_with_extra(self):
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(['a', 'b', 'c'], iter(lst), 'b', 1)
+        self.assertIterate(['a', 'b', 'c', 'd', 'e'], iter(lst), 'c', 2)
+
+    def test_iter_with_too_many_extra(self):
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(lst, iter(lst), 'y', 10)
+        self.assertIterate(lst, iter(lst), 'z', 10)
+
+    def test_iter_with_extra_None(self):
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(lst, iter(lst), 'z', None)
+
+
+
+class TestHistoryGetView(TestCaseWithExamples):
+
+    def test_get_view_limited_history(self):
+        # get_view should only load enough history to serve the result, not all
+        # history.
+        history = self.make_long_linear_ancestry()
+        accessed = track_rev_info_accesses(history)
+        revid, start_revid, revid_list = history.get_view('Z', 'Z', None,
+                                                      extra_rev_count=5)
+        self.assertEqual(['Z', 'Y', 'X', 'W', 'V', 'U'], revid_list)
+        self.assertEqual('Z', revid)
+        self.assertEqual('Z', start_revid)
+        self.assertEqual(set([history._rev_indices[x] for x in 'ZYXWVU']),
+                         accessed)
+
+
+class TestHistoryGetChangedUncached(TestCaseWithExamples):
+
+    def test_native(self):
+        history = self.make_linear_ancestry()
+        changes = history.get_changes_uncached(['rev-1', 'rev-2'])
+        self.assertEquals(2, len(changes))
+        self.assertEquals('rev-1', changes[0].revid)
+        self.assertEquals('rev-2', changes[1].revid)
+        self.assertIs(None, getattr(changes[0], 'foreign_vcs', None))
+        self.assertIs(None, getattr(changes[0], 'foreign_revid', None))
+
+    def test_foreign(self):
+        # Test with a mocked foreign revision, as it's not possible
+        # to rely on any foreign plugins being installed.
+        history = self.make_linear_ancestry()
+        foreign_vcs = ForeignVcs(None, "vcs")
+        foreign_vcs.show_foreign_revid = repr
+        foreign_rev = ForeignRevision(("uuid", 1234), VcsMapping(foreign_vcs),
+            "revid-in-bzr", message="message",
+            timestamp=234423423.3)
+        change = history._change_from_revision(foreign_rev)
+        self.assertEquals('revid-in-bzr', change.revid)
+        self.assertEquals("('uuid', 1234)", change.foreign_revid)
+        self.assertEquals("vcs", change.foreign_vcs)

=== added file 'loggerhead/tests/test_http_head.py'
--- loggerhead/tests/test_http_head.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_http_head.py	2011-02-10 02:33:15 +0000
@@ -0,0 +1,92 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the HeadMiddleware app."""
+
+from cStringIO import StringIO
+
+from bzrlib import tests
+
+from loggerhead.apps import http_head
+
+
+content = ["<html>",
+           "<head><title>Listed</title></head>",
+           "<body>Content</body>",
+           "</html>",
+          ]
+headers = {'X-Ignored-Header': 'Value'}
+
+def yielding_app(environ, start_response):
+    writer = start_response('200 OK', headers)
+    for chunk in content:
+        yield chunk
+
+
+def list_app(environ, start_response):
+    writer = start_response('200 OK', headers)
+    return content
+
+
+def writer_app(environ, start_response):
+    writer = start_response('200 OK', headers)
+    for chunk in content:
+        writer(chunk)
+    return []
+
+
+class TestHeadMiddleware(tests.TestCase):
+
+    def _trap_start_response(self, status, response_headers, exc_info=None):
+        self._write_buffer = StringIO()
+        self._start_response_passed = (status, response_headers, exc_info)
+        return self._write_buffer.write
+
+    def _consume_app(self, app, request_method):
+        environ = {'REQUEST_METHOD': request_method}
+        value = list(app(environ, self._trap_start_response))
+        self._write_buffer.writelines(value)
+
+    def _verify_get_passthrough(self, app):
+        app = http_head.HeadMiddleware(app)
+        self._consume_app(app, 'GET')
+        self.assertEqual(('200 OK', headers, None), self._start_response_passed)
+        self.assertEqualDiff(''.join(content), self._write_buffer.getvalue())
+
+    def _verify_head_no_body(self, app):
+        app = http_head.HeadMiddleware(app)
+        self._consume_app(app, 'HEAD')
+        self.assertEqual(('200 OK', headers, None), self._start_response_passed)
+        self.assertEqualDiff('', self._write_buffer.getvalue())
+
+    def test_get_passthrough_yielding(self):
+        self._verify_get_passthrough(yielding_app)
+
+    def test_head_passthrough_yielding(self):
+        self._verify_head_no_body(yielding_app)
+
+    def test_get_passthrough_list(self):
+        self._verify_get_passthrough(list_app)
+
+    def test_head_passthrough_list(self):
+        self._verify_head_no_body(list_app)
+
+    def test_get_passthrough_writer(self):
+        self._verify_get_passthrough(writer_app)
+
+    def test_head_passthrough_writer(self):
+        self._verify_head_no_body(writer_app)
+

=== added file 'loggerhead/tests/test_load_test.py'
--- loggerhead/tests/test_load_test.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_load_test.py	2011-04-14 14:10:04 +0000
@@ -0,0 +1,337 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#
+
+"""Tests for the load testing code."""
+
+import socket
+import time
+import threading
+import Queue
+
+from bzrlib import tests
+from bzrlib.tests import http_server
+
+from loggerhead import load_test
+
+
+empty_script = """{
+    "parameters": {},
+    "requests": []
+}"""
+
+class TestRequestDescription(tests.TestCase):
+
+    def test_init_from_dict(self):
+        rd = load_test.RequestDescription({'thread': '10', 'relpath': '/foo'})
+        self.assertEqual('10', rd.thread)
+        self.assertEqual('/foo', rd.relpath)
+
+    def test_default_thread_is_1(self):
+        rd = load_test.RequestDescription({'relpath': '/bar'})
+        self.assertEqual('1', rd.thread)
+        self.assertEqual('/bar', rd.relpath)
+
+
+_cur_time = time.time()
+def one_sec_timer():
+    """Every time this timer is called, it increments by 1 second."""
+    global _cur_time
+    _cur_time += 1.0
+    return _cur_time
+
+
+class NoopRequestWorker(load_test.RequestWorker):
+
+    # Every call to _timer will increment by one
+    _timer = staticmethod(one_sec_timer)
+
+    # Ensure that process never does anything
+    def process(self, url):
+        return True
+
+
+class TestRequestWorkerInfrastructure(tests.TestCase):
+    """Tests various infrastructure bits, without doing actual requests."""
+
+    def test_step_next_tracks_time(self):
+        rt = NoopRequestWorker('id')
+        rt.queue.put('item')
+        rt.step_next()
+        self.assertTrue(rt.queue.empty())
+        self.assertEqual([('item', True, 1.0)], rt.stats)
+
+    def test_step_multiple_items(self):
+        rt = NoopRequestWorker('id')
+        rt.queue.put('item')
+        rt.step_next()
+        rt.queue.put('next-item')
+        rt.step_next()
+        self.assertTrue(rt.queue.empty())
+        self.assertEqual([('item', True, 1.0), ('next-item', True, 1.0)],
+                         rt.stats)
+
+    def test_step_next_does_nothing_for_noop(self):
+        rt = NoopRequestWorker('id')
+        rt.queue.put('item')
+        rt.step_next()
+        rt.queue.put('<noop>')
+        rt.step_next()
+        self.assertEqual([('item', True, 1.0)], rt.stats)
+
+    def test_step_next_will_timeout(self):
+        # We don't want step_next to block forever
+        rt = NoopRequestWorker('id', blocking_time=0.001)
+        self.assertRaises(Queue.Empty, rt.step_next)
+
+    def test_run_stops_for_stop_event(self):
+        rt = NoopRequestWorker('id', blocking_time=0.001, _queue_size=2)
+        rt.queue.put('item1')
+        rt.queue.put('item2')
+        event = threading.Event()
+        t = threading.Thread(target=rt.run, args=(event,))
+        t.start()
+        # Wait for the queue to be processed
+        rt.queue.join()
+        # Now we can queue up another one, and wait for it
+        rt.queue.put('item3')
+        rt.queue.join()
+        # Now set the stopping event
+        event.set()
+        # Add another item to the queue, which might get processed, but the
+        # next item won't
+        rt.queue.put('item4')
+        rt.queue.put('item5')
+        t.join()
+        self.assertEqual([('item1', True, 1.0), ('item2', True, 1.0),
+                          ('item3', True, 1.0)],
+                         rt.stats[:3])
+        # The last event might be item4 or might be item3, the important thing
+        # is that even though there are still queued events, we won't
+        # process anything past the first
+        self.assertNotEqual('item5', rt.stats[-1][0])
+
+
+class TestRequestWorker(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestRequestWorker, self).setUp()
+        self.transport_readonly_server = http_server.HttpServer
+
+    def test_request_items(self):
+        rt = load_test.RequestWorker('id', blocking_time=0.01, _queue_size=2)
+        self.build_tree(['file1', 'file2'])
+        readonly_url1 = self.get_readonly_url('file1')
+        self.assertStartsWith(readonly_url1, 'http://')
+        readonly_url2 = self.get_readonly_url('file2')
+        rt.queue.put(readonly_url1)
+        rt.queue.put(readonly_url2)
+        rt.step_next()
+        rt.step_next()
+        self.assertEqual(readonly_url1, rt.stats[0][0])
+        self.assertEqual(readonly_url2, rt.stats[1][0])
+
+    def test_request_nonexistant_items(self):
+        rt = load_test.RequestWorker('id', blocking_time=0.01, _queue_size=2)
+        readonly_url1 = self.get_readonly_url('no-file1')
+        rt.queue.put(readonly_url1)
+        rt.step_next()
+        self.assertEqual(readonly_url1, rt.stats[0][0])
+        self.assertEqual(False, rt.stats[0][1])
+
+    def test_no_server(self):
+        s = socket.socket()
+        # Bind to a port, but don't listen on it
+        s.bind(('localhost', 0))
+        url = 'http://%s:%s/path' % s.getsockname()
+        rt = load_test.RequestWorker('id', blocking_time=0.01, _queue_size=2)
+        rt.queue.put(url)
+        rt.step_next()
+        self.assertEqual((url, False), rt.stats[0][:2])
+
+
+class NoActionScript(load_test.ActionScript):
+
+    _thread_class = NoopRequestWorker
+    _default_blocking_timeout = 0.01
+
+
+class TestActionScriptInfrastructure(tests.TestCase):
+
+    def test_parse_requires_parameters_and_requests(self):
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '')
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '{}')
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '{"parameters": {}}')
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '{"requests": []}')
+        load_test.ActionScript.parse(
+            '{"parameters": {}, "requests": [], "comment": "section"}')
+        script = load_test.ActionScript.parse(
+            empty_script)
+        self.assertIsNot(None, script)
+
+    def test_parse_default_base_url(self):
+        script = load_test.ActionScript.parse(empty_script)
+        self.assertEqual('http://localhost:8080', script.base_url)
+
+    def test_parse_find_base_url(self):
+        script = load_test.ActionScript.parse(
+            '{"parameters": {"base_url": "http://example.com"},'
+            ' "requests": []}')
+        self.assertEqual('http://example.com', script.base_url)
+
+    def test_parse_default_blocking_timeout(self):
+        script = load_test.ActionScript.parse(empty_script)
+        self.assertEqual(60.0, script.blocking_timeout)
+
+    def test_parse_find_blocking_timeout(self):
+        script = load_test.ActionScript.parse(
+            '{"parameters": {"blocking_timeout": 10.0},'
+            ' "requests": []}'
+        )
+        self.assertEqual(10.0, script.blocking_timeout)
+
+    def test_parse_finds_requests(self):
+        script = load_test.ActionScript.parse(
+            '{"parameters": {}, "requests": ['
+            ' {"relpath": "/foo"},'
+            ' {"relpath": "/bar"}'
+            ' ]}')
+        self.assertEqual(2, len(script._requests))
+        self.assertEqual("/foo", script._requests[0].relpath)
+        self.assertEqual("/bar", script._requests[1].relpath)
+
+    def test__get_worker(self):
+        script = NoActionScript()
+        # If an id is found, then we should create it
+        self.assertEqual({}, script._threads)
+        worker = script._get_worker('id')
+        self.assertTrue('id' in script._threads)
+        # We should have set the blocking timeout
+        self.assertEqual(script.blocking_timeout / 10.0,
+                         worker.blocking_time)
+
+        # Another request will return the identical object
+        self.assertIs(worker, script._get_worker('id'))
+
+        # And the stop event will stop the thread
+        script.stop_and_join()
+
+    def test__full_url(self):
+        script = NoActionScript()
+        self.assertEqual('http://localhost:8080/path',
+                         script._full_url('/path'))
+        self.assertEqual('http://localhost:8080/path/to/foo',
+                         script._full_url('/path/to/foo'))
+        script.base_url = 'http://example.com'
+        self.assertEqual('http://example.com/path/to/foo',
+                         script._full_url('/path/to/foo'))
+        script.base_url = 'http://example.com/base'
+        self.assertEqual('http://example.com/base/path/to/foo',
+                         script._full_url('/path/to/foo'))
+        script.base_url = 'http://example.com'
+        self.assertEqual('http://example.com:8080/path',
+                         script._full_url(':8080/path'))
+
+    def test_single_threaded(self):
+        script = NoActionScript.parse("""{
+            "parameters": {"base_url": ""},
+            "requests": [
+                {"thread": "1", "relpath": "first"},
+                {"thread": "1", "relpath": "second"},
+                {"thread": "1", "relpath": "third"},
+                {"thread": "1", "relpath": "fourth"}
+            ]}""")
+        script.run()
+        worker = script._get_worker("1")
+        self.assertEqual(["first", "second", "third", "fourth"],
+                         [s[0] for s in worker.stats])
+
+    def test_two_threads(self):
+        script = NoActionScript.parse("""{
+            "parameters": {"base_url": ""},
+            "requests": [
+                {"thread": "1", "relpath": "first"},
+                {"thread": "2", "relpath": "second"},
+                {"thread": "1", "relpath": "third"},
+                {"thread": "2", "relpath": "fourth"}
+            ]}""")
+        script.run()
+        worker = script._get_worker("1")
+        self.assertEqual(["first", "third"],
+                         [s[0] for s in worker.stats])
+        worker = script._get_worker("2")
+        self.assertEqual(["second", "fourth"],
+                         [s[0] for s in worker.stats])
+
+
+class TestActionScriptIntegration(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestActionScriptIntegration, self).setUp()
+        self.transport_readonly_server = http_server.HttpServer
+
+    def test_full_integration(self):
+        self.build_tree(['first', 'second', 'third', 'fourth'])
+        url = self.get_readonly_url()
+        script = load_test.ActionScript.parse("""{
+            "parameters": {"base_url": "%s", "blocking_timeout": 2.0},
+            "requests": [
+                {"thread": "1", "relpath": "first"},
+                {"thread": "2", "relpath": "second"},
+                {"thread": "1", "relpath": "no-this"},
+                {"thread": "2", "relpath": "or-this"},
+                {"thread": "1", "relpath": "third"},
+                {"thread": "2", "relpath": "fourth"}
+            ]}""" % (url,))
+        script.run()
+        worker = script._get_worker("1")
+        self.assertEqual([("first", True), ('no-this', False),
+                          ("third", True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])
+        worker = script._get_worker("2")
+        self.assertEqual([("second", True), ('or-this', False),
+                          ("fourth", True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])
+
+
+class TestRunScript(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestRunScript, self).setUp()
+        self.transport_readonly_server = http_server.HttpServer
+
+    def test_run_script(self):
+        self.build_tree(['file1', 'file2', 'file3', 'file4'])
+        url = self.get_readonly_url()
+        self.build_tree_contents([('localhost.script', """{
+    "parameters": {"base_url": "%s", "blocking_timeout": 0.1},
+    "requests": [
+        {"thread": "1", "relpath": "file1"},
+        {"thread": "2", "relpath": "file2"},
+        {"thread": "1", "relpath": "file3"},
+        {"thread": "2", "relpath": "file4"}
+    ]
+}""" % (url,))])
+        script = load_test.run_script('localhost.script')
+        worker = script._threads["1"][0]
+        self.assertEqual([("file1", True), ('file3', True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])
+        worker = script._threads["2"][0]
+        self.assertEqual([("file2", True), ("file4", True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])

=== added file 'loggerhead/tests/test_revision_ui.py'
--- loggerhead/tests/test_revision_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_revision_ui.py	2011-03-16 12:37:17 +0000
@@ -0,0 +1,46 @@
+# Copyright (C) 2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+
+from loggerhead.tests.test_simple import BasicTests
+
+
+class TestRevisionUI(BasicTests):
+
+    def test_authors_vs_committer(self):
+        self.createBranch()
+        self.tree.commit('First', committer="Joe Example <joe@example.com>",
+            revprops={'authors': u'A Author <aauthor@example.com>\n'
+                                 u'B Author <bauthor@example.com>'})
+        app = self.setUpLoggerhead()
+        res = app.get('/revision/1')
+        # We would like to assert that Joe Example is connected to Committer,
+        # and the Authors are connected. However, that requires asserting the
+        # exact HTML connections, which I wanted to avoid.
+        res.mustcontain('Committer', 'Joe Example',
+                        'Author(s)', 'A Author, B Author')
+
+    def test_author_is_committer(self):
+        self.createBranch()
+        self.tree.commit('First', committer="Joe Example <joe@example.com>")
+        app = self.setUpLoggerhead()
+        res = app.get('/revision/1')
+        # We would like to assert that Joe Example is connected to Committer,
+        # and the Authors are connected. However, that requires asserting the
+        # exact HTML connections, which I wanted to avoid.
+        res.mustcontain('Committer', 'Joe Example')
+        self.assertFalse('Author(s)' in res.body)

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2009-06-08 23:02:49 +0000
+++ loggerhead/tests/test_simple.py	2011-11-25 02:28:56 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009 Canonical Ltd.
+# Copyright (C) 2007, 2008, 2009, 2011 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,23 +17,25 @@
 
 import cgi
 import logging
+import re
+import simplejson
+from cStringIO import StringIO
 
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.util.configobj.configobj import ConfigObj
+try:
+    from bzrlib.util.configobj.configobj import ConfigObj
+except ImportError:
+    from configobj import ConfigObj
 from bzrlib import config
 
 from loggerhead.apps.branch import BranchWSGIApp
+from loggerhead.apps.http_head import HeadMiddleware
 from paste.fixture import TestApp
-from paste.httpexceptions import HTTPExceptionHandler
-
-
-
-def test_config_root():
-    from loggerhead.apps.config import Root
-    config = ConfigObj()
-    app = TestApp(HTTPExceptionHandler(Root(config)))
-    res = app.get('/')
-    res.mustcontain('loggerhead branches')
+from paste.httpexceptions import HTTPExceptionHandler, HTTPMovedPermanently
+
+from loggerhead.tests.fixtures import (
+    SampleBranch,
+    )
 
 
 class BasicTests(TestCaseWithTransport):
@@ -50,33 +52,62 @@
         branch_app = BranchWSGIApp(self.tree.branch, '', **kw).app
         return TestApp(HTTPExceptionHandler(branch_app))
 
+    def assertOkJsonResponse(self, app, env):
+        start, content = consume_app(app, env)
+        self.assertEqual('200 OK', start[0])
+        self.assertEqual('application/json', dict(start[1])['Content-Type'])
+        self.assertEqual(None, start[2])
+        simplejson.loads(content)
+
+    def make_branch_app(self, branch, **kw):
+        branch_app = BranchWSGIApp(branch, friendly_name='friendly-name', **kw)
+        branch_app._environ = {
+            'wsgi.url_scheme':'',
+            'SERVER_NAME':'',
+            'SERVER_PORT':'80',
+            }
+        branch_app._url_base = ''
+        return branch_app
+
 
 class TestWithSimpleTree(BasicTests):
 
     def setUp(self):
         BasicTests.setUp(self)
-        self.createBranch()
-
-        self.filecontents = ('some\nmultiline\ndata\n'
-                             'with<htmlspecialchars\n')
-        self.build_tree_contents(
-            [('myfilename', self.filecontents)])
-        self.tree.add('myfilename')
-        self.fileid = self.tree.path2id('myfilename')
-        self.msg = 'a very exciting commit message <'
-        self.revid = self.tree.commit(message=self.msg)
+        self.sample_branch_fixture = SampleBranch(self)
+
+        # XXX: This could be cleaned up more... -- mbp 2011-11-25
+        self.useFixture(self.sample_branch_fixture)
+        self.tree = self.sample_branch_fixture.tree
+        self.fileid = self.sample_branch_fixture.fileid
+        self.filecontents = self.sample_branch_fixture.filecontents
+        self.msg = self.sample_branch_fixture.msg
+
+    def test_public_private(self):
+        app = self.make_branch_app(self.tree.branch, private=True)
+        self.assertEqual(app.public_private_css(), 'private')
+        app = self.make_branch_app(self.tree.branch)
+        self.assertEqual(app.public_private_css(), 'public')
 
     def test_changes(self):
         app = self.setUpLoggerhead()
         res = app.get('/changes')
         res.mustcontain(cgi.escape(self.msg))
 
+    def test_changes_for_file(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/changes?filter_file_id=myfilename-id')
+        res.mustcontain(cgi.escape(self.msg))
+
     def test_changes_branch_from(self):
         app = self.setUpLoggerhead(served_url="lp:loggerhead")
         res = app.get('/changes')
         self.failUnless("To get this branch, use:" in res)
         self.failUnless("lp:loggerhead" in res)
-        app = self.setUpLoggerhead(served_url=None)
+
+    def test_no_empty_download_location(self):
+        """With no served_url, no instructions how to get it"""
+        app = self.setUpLoggerhead()
         res = app.get('/changes')
         self.failIf("To get this branch, use:" in res)
 
@@ -88,8 +119,20 @@
     def test_annotate(self):
         app = self.setUpLoggerhead()
         res = app.get('/annotate', params={'file_id': self.fileid})
+        # If pygments is installed, it inserts <span class="pyg" content into
+        # the output, to trigger highlighting. And it specifically highlights
+        # the &lt; that we are interested in seeing in the output.
+        # Without pygments we have a simple: 'with&lt;htmlspecialchars'
+        # With it, we have
+        # '<span class='pyg-n'>with</span><span class='pyg-o'>&lt;</span>'
+        # '<span class='pyg-n'>htmlspecialchars</span>
+        # So we pre-filter the body, to make sure remove spans of that type.
+        body_no_span = re.sub(r'<span class="pyg-.">', '', res.body)
+        body_no_span = body_no_span.replace('</span>', '')
         for line in self.filecontents.splitlines():
-            res.mustcontain(cgi.escape(line))
+            escaped = cgi.escape(line)
+            self.assertTrue(escaped in body_no_span,
+                            "did not find %r in %r" % (escaped, body_no_span))
 
     def test_inventory(self):
         app = self.setUpLoggerhead()
@@ -117,6 +160,8 @@
     def test_revision(self):
         app = self.setUpLoggerhead()
         res = app.get('/revision/1')
+        res.mustcontain(no=['anotherfile<'])
+        res.mustcontain('anotherfile&lt;')
         res.mustcontain('myfilename')
 
 
@@ -157,6 +202,71 @@
         res = app.get('/changes', status=404)
 
 
+class TestControllerRedirects(BasicTests):
+    """
+    Test that a file under /files redirects to /view,
+    and a directory under /view redirects to /files.
+    """
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.createBranch()
+        self.build_tree(('file', 'folder/', 'folder/file'))
+        self.tree.smart_add([])
+        self.tree.commit('')
+
+    def test_view_folder(self):
+        app = TestApp(BranchWSGIApp(self.tree.branch, '').app)
+
+        e = self.assertRaises(HTTPMovedPermanently, app.get, '/view/head:/folder')
+        self.assertEqual(e.location(), '/files/head:/folder')
+
+    def test_files_file(self):
+        app = TestApp(BranchWSGIApp(self.tree.branch, '').app)
+
+        e = self.assertRaises(HTTPMovedPermanently, app.get, '/files/head:/folder/file')
+        self.assertEqual(e.location(), '/view/head:/folder/file')
+        e = self.assertRaises(HTTPMovedPermanently, app.get, '/files/head:/file')
+        self.assertEqual(e.location(), '/view/head:/file')
+
+
+class TestHeadMiddleware(BasicTests):
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.createBranch()
+        self.msg = 'trivial commit message'
+        self.revid = self.tree.commit(message=self.msg)
+
+    def setUpLoggerhead(self, **kw):
+        branch_app = BranchWSGIApp(self.tree.branch, '', **kw).app
+        return TestApp(HTTPExceptionHandler(HeadMiddleware(branch_app)))
+
+    def test_get(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/changes')
+        res.mustcontain(self.msg)
+        self.assertEqual('text/html', res.header('Content-Type'))
+
+    def test_head(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/changes', extra_environ={'REQUEST_METHOD': 'HEAD'})
+        self.assertEqual('text/html', res.header('Content-Type'))
+        self.assertEqualDiff('', res.body)
+
+
+def consume_app(app, env):
+    body = StringIO()
+    start = []
+    def start_response(status, headers, exc_info=None):
+        start.append((status, headers, exc_info))
+        return body.write
+    extra_content = list(app(env, start_response))
+    body.writelines(extra_content)
+    return start[0], body.getvalue()
+
+
+
 #class TestGlobalConfig(BasicTests):
 #    """
 #    Test that global config settings are respected

=== added file 'loggerhead/tests/test_util.py'
--- loggerhead/tests/test_util.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_util.py	2011-09-08 00:33:28 +0000
@@ -0,0 +1,33 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib import tests
+
+from loggerhead.util import html_escape, html_format
+
+
+class TestHTMLEscaping(tests.TestCase):
+
+    def test_html_escape(self):
+        self.assertEqual(
+            "foo &quot;&#39;&lt;&gt;&amp;",
+            html_escape("foo \"'<>&"))
+
+    def test_html_format(self):
+        self.assertEqual(
+            '<foo bar="baz&quot;&#39;">&lt;baz&gt;&amp;</foo>',
+            html_format(
+                '<foo bar="%s">%s</foo>', "baz\"'", "<baz>&"))

=== removed file 'loggerhead/trace.py'
--- loggerhead/trace.py	2008-10-24 02:26:05 +0000
+++ loggerhead/trace.py	1970-01-01 00:00:00 +0000
@@ -1,75 +0,0 @@
-#
-# Copyright (C) 2008  Canonical Ltd.
-#                     (Authored by Martin Albisetti <argentina@gmail.com)
-# Copyright (C) 2008  Guillermo Gonzalez <guillo.gonzo@gmail.com>
-# Copyright (C) 2006  Robey Pointer <robey@lag.net>
-# Copyright (C) 2006  Goffredo Baroncelli <kreijack@inwind.it>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
-
-import os
-import logging
-import sys
-
-
-def make_handler(config, filename):
-    roll = config.get('log.roll', 'never')
-    if roll == 'daily':
-        h = logging.handlers.TimedRotatingFileHandler(filename,
-                                                      'midnight',
-                                                      1,
-                                                      100)
-    elif roll == 'weekly':
-        h = logging.handlers.TimedRotatingFileHandler(filename, 'W0', 1, 100)
-    else:
-        h = logging.FileHandler(filename)
-    return h
-
-
-def setup_logging(log_folder, config, foreground):
-    # i hate that stupid logging config format, so just set up logging here.
-
-    if not os.path.exists(log_folder):
-        os.mkdir(log_folder)
-
-    f = logging.Formatter('%(levelname)-.3s [%(asctime)s.%(msecs)03d]'
-                          ' %(name)s: %(message)s',
-                          '%Y%m%d-%H:%M:%S')
-    debug_log = make_handler(config, os.path.join(log_folder, 'debug.log'))
-    debug_log.setLevel(logging.DEBUG)
-    debug_log.setFormatter(f)
-    if foreground:
-        stdout_log = logging.StreamHandler(sys.stdout)
-        stdout_log.setLevel(logging.DEBUG)
-        stdout_log.setFormatter(f)
-    f = logging.Formatter('[%(asctime)s.%(msecs)03d] %(message)s',
-                          '%Y%m%d-%H:%M:%S')
-    access_log = make_handler(config, os.path.join(log_folder, 'access.log'))
-    access_log.setLevel(logging.INFO)
-    access_log.setFormatter(f)
-
-    error_log = make_handler(config, os.path.join(log_folder, 'error.log'))
-    error_log.setLevel(logging.ERROR)
-    error_log.setFormatter(f)
-
-    logging.getLogger('').setLevel(logging.DEBUG)
-    logging.getLogger('').addHandler(debug_log)
-    logging.getLogger('loggerhead.access').addHandler(access_log)
-    logging.getLogger('loggerhead').setLevel(logging.ERROR)
-    logging.getLogger('loggerhead').addHandler(error_log)
-
-    if foreground:
-        logging.getLogger('').addHandler(stdout_log)

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2009-10-17 06:35:33 +0000
+++ loggerhead/util.py	2012-02-02 07:42:24 +0000
@@ -20,7 +20,6 @@
 #
 
 import base64
-import cgi
 import datetime
 import logging
 import re
@@ -36,6 +35,8 @@
 except ImportError:
     from elementtree import ElementTree as ET
 
+from bzrlib import urlutils
+
 from simpletal.simpleTALUtils import HTMLStructureCleaner
 
 log = logging.getLogger("loggerhead.controllers")
@@ -51,13 +52,12 @@
 # Display of times.
 
 # date_day -- just the day
-# date_time -- full date with time
+# date_time -- full date with time (UTC)
 #
-# displaydate -- for use in sentences
 # approximatedate -- for use in tables
 #
-# displaydate and approximatedate return an elementtree <span> Element
-# with the full date in a tooltip.
+# approximatedate return an elementtree <span> Element
+# with the full date (UTC) in a tooltip.
 
 
 def date_day(value):
@@ -66,19 +66,12 @@
 
 def date_time(value):
     if value is not None:
-        return value.strftime('%Y-%m-%d %T')
+        # Note: this assumes that the value is UTC in some fashion.
+        return value.strftime('%Y-%m-%d %H:%M:%S UTC')
     else:
         return 'N/A'
 
 
-def _displaydate(date):
-    delta = abs(datetime.datetime.now() - date)
-    if delta > datetime.timedelta(1, 0, 0):
-        # far in the past or future, display the date
-        return 'on ' + date_day(date)
-    return _approximatedate(date)
-
-
 def _approximatedate(date):
     delta = datetime.datetime.now() - date
     if abs(delta) > datetime.timedelta(1, 0, 0):
@@ -125,16 +118,13 @@
     return _wrap_with_date_time_title(date, _approximatedate(date))
 
 
-def displaydate(date):
-    return _wrap_with_date_time_title(date, _displaydate(date))
-
-
 class Container(object):
     """
     Convert a dict into an object with attributes.
     """
 
     def __init__(self, _dict=None, **kw):
+        self._properties = {}
         if _dict is not None:
             for key, value in _dict.iteritems():
                 setattr(self, key, value)
@@ -151,6 +141,24 @@
         out += '}'
         return out
 
+    def __getattr__(self, attr):
+        """Used for handling things that aren't already available."""
+        if attr.startswith('_') or attr not in self._properties:
+            raise AttributeError('No attribute: %s' % (attr,))
+        val = self._properties[attr](self, attr)
+        setattr(self, attr, val)
+        return val
+
+    def _set_property(self, attr, prop_func):
+        """Set a function that will be called when an attribute is desired.
+
+        We will cache the return value, so the function call should be
+        idempotent. We will pass 'self' and the 'attr' name when triggered.
+        """
+        if attr.startswith('_'):
+            raise ValueError("Cannot create properties that start with _")
+        self._properties[attr] = prop_func
+
 
 def trunc(text, limit=10):
     if len(text) <= limit:
@@ -193,16 +201,47 @@
 # only do this if unicode turns out to be a problem
 #_BADCHARS_RE = re.compile(ur'[\u007f-\uffff]')
 
+# Can't be a dict; &amp; needs to be done first.
+html_entity_subs = [
+    ("&", "&amp;"),
+    ('"', "&quot;"),
+    ("'", "&#39;"), # &apos; is defined in XML, but not HTML.
+    (">", "&gt;"),
+    ("<", "&lt;"),
+    ]
+
+
+def html_escape(s):
+    """Transform dangerous (X)HTML characters into entities.
+
+    Like cgi.escape, except also escaping \" and '. This makes it safe to use
+    in both attribute and element content.
+
+    If you want to safely fill a format string with escaped values, use
+    html_format instead
+    """
+    for char, repl in html_entity_subs:
+        s = s.replace(char, repl)
+    return s
+
+
+def html_format(template, *args):
+    """Safely format an HTML template string, escaping the arguments.
+
+    The template string must not be user-controlled; it will not be escaped.
+    """
+    return template % tuple(html_escape(arg) for arg in args)
+
+
 # FIXME: get rid of this method; use fixed_width() and avoid XML().
 
-
 def html_clean(s):
     """
     clean up a string for html display.  expand any tabs, encode any html
     entities, and replace spaces with '&nbsp;'.  this is primarily for use
     in displaying monospace text.
     """
-    s = cgi.escape(s.expandtabs())
+    s = html_escape(s.expandtabs())
     s = s.replace(' ', '&nbsp;')
     return s
 
@@ -248,7 +287,7 @@
         except UnicodeDecodeError:
             s = s.decode('iso-8859-15')
 
-    s = cgi.escape(s).expandtabs().replace(' ', NONBREAKING_SPACE)
+    s = html_escape(s).expandtabs().replace(' ', NONBREAKING_SPACE)
 
     return HSC.clean(s).replace('\n', '<br/>')
 
@@ -324,6 +363,14 @@
     return out
 
 
+def local_path_from_url(url):
+    """Convert Bazaar URL to local path, ignoring readonly+ prefix"""
+    readonly_prefix = 'readonly+'
+    if url.startswith(readonly_prefix):
+        url = url[len(readonly_prefix):]
+    return urlutils.local_path_from_url(url)
+
+
 def fill_in_navigation(navigation):
     """
     given a navigation block (used by the template for the page header), fill
@@ -425,7 +472,7 @@
     for index, dir_name in enumerate(dir_parts):
         inner_breadcrumbs.append({
             'dir_name': dir_name,
-            'file_id': inv.path2id('/'.join(dir_parts[:index + 1])),
+            'path': '/'.join(dir_parts[:index + 1]),
             'suffix': '/' + view,
         })
     return inner_breadcrumbs
@@ -496,8 +543,8 @@
 #         for re-ordering an existing page by different sort
 
 t_context = threading.local()
-_valid = ('start_revid', 'file_id', 'filter_file_id', 'q', 'remember',
-          'compare_revid', 'sort')
+_valid = (
+    'start_revid', 'filter_file_id', 'q', 'remember', 'compare_revid', 'sort')
 
 
 def set_context(map):
@@ -587,3 +634,30 @@
             if exit_code != 3:
                 return exit_code
             print '-'*20, 'Restarting', '-'*20
+
+
+def convert_file_errors(application):
+    """WSGI wrapper to convert some file errors to Paste exceptions"""
+    def new_application(environ, start_response):
+        try:
+            return application(environ, start_response)
+        except (IOError, OSError), e:
+            import errno
+            from paste import httpexceptions
+            if e.errno == errno.ENOENT:
+                raise httpexceptions.HTTPNotFound()
+            elif e.errno == errno.EACCES:
+                raise httpexceptions.HTTPForbidden()
+            else:
+                raise
+    return new_application
+
+
+def convert_to_json_ready(obj):
+    if isinstance(obj, Container):
+        d = obj.__dict__.copy()
+        del d['_properties']
+        return d
+    elif isinstance(obj, datetime.datetime):
+        return tuple(obj.utctimetuple())
+    raise TypeError(repr(obj) + " is not JSON serializable")

=== modified file 'loggerhead/wholehistory.py'
--- loggerhead/wholehistory.py	2009-10-17 08:47:38 +0000
+++ loggerhead/wholehistory.py	2010-04-26 19:37:26 +0000
@@ -51,8 +51,8 @@
                             (branch.get_config().get_nickname(),))
 
     graph = branch.repository.get_graph()
-    parent_map = dict(((key, value) for key, value in
-         graph.iter_ancestry([last_revid]) if value is not None))
+    parent_map = dict((key, value) for key, value in
+        graph.iter_ancestry([last_revid]) if value is not None)
 
     _revision_graph = _strip_NULL_ghosts(parent_map)
 
@@ -80,6 +80,6 @@
             if revid not in c[1]:
                 c[1] = c[1] + (revid,)
 
-    log.info('built revision graph cache: %r secs' % (time.time() - z,))
+    log.info('built revision graph cache: %.3f secs' % (time.time() - z,))
 
     return (_rev_info, _rev_indices)

=== modified file 'loggerhead/zptsupport.py'
--- loggerhead/zptsupport.py	2009-03-02 05:44:20 +0000
+++ loggerhead/zptsupport.py	2010-05-12 15:01:44 +0000
@@ -23,10 +23,6 @@
 
 from simpletal import simpleTAL, simpleTALES
 
-logging.getLogger("simpleTAL").setLevel(logging.INFO)
-logging.getLogger("simpleTALES").setLevel(logging.INFO)
-
-
 _zpt_cache = {}
 
 

=== modified file 'loggerheadd'
--- loggerheadd	2008-11-24 20:26:26 +0000
+++ loggerheadd	2009-04-30 10:39:05 +0000
@@ -16,9 +16,9 @@
 
 LHUSER=loggerhead
 
-if [ `whoami` = "$LHUSER" ]; then                                                                                   
+if [ `whoami` = "$LHUSER" ]; then
     SUDO=""
-else                                                                                                                
+else
     SUDO="sudo -H -u $LHUSER"
 fi
 

=== modified file 'setup.py'
--- setup.py	2009-10-17 06:19:40 +0000
+++ setup.py	2012-02-02 04:58:46 +0000
@@ -30,7 +30,10 @@
     license = "GNU GPL v2 or later",
     maintainer = "Michael Hudson",
     maintainer_email = "michael.hudson@canonical.com",
-    scripts = ["start-loggerhead", "stop-loggerhead", "serve-branches"],
+    scripts = [
+        "serve-branches",
+        "loggerhead.wsgi",
+        ],
     packages = ["loggerhead",
                 "loggerhead/apps",
                 "loggerhead/controllers",
@@ -64,9 +67,9 @@
                                    "static/javascript/yui/build/yui-base/*",
                                    "static/images/*"]},
     data_files = [
-        ('share/man/man1', ['start-loggerhead.1',
-                            'stop-loggerhead.1',
-                            'serve-branches.1']),
-        ('share/doc/loggerhead', ['loggerhead.conf.example']),
+        ('share/man/man1', ['serve-branches.1']),
+        ('share/doc/loggerhead', ['loggerhead.conf.example',
+                                  'apache-loggerhead.conf',
+                                  'bazaar.conf']),
         ],
     )

=== removed file 'start-loggerhead'
--- start-loggerhead	2009-10-17 06:35:33 +0000
+++ start-loggerhead	1970-01-01 00:00:00 +0000
@@ -1,146 +0,0 @@
-#!/usr/bin/env python
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""
-WARNING! This script has been deprecated and will go away soon!
-A script for starting the loggerhead process.
-"""
-
-
-import logging
-from optparse import OptionParser
-import os
-import sys
-import urlparse
-
-from bzrlib.plugin import load_plugins
-from bzrlib.util.configobj.configobj import ConfigObj
-
-from paste import httpserver
-from paste.httpexceptions import make_middleware
-from paste.translogger import make_filter
-
-from loggerhead import daemon
-from loggerhead.apps.config import Root
-from loggerhead.trace import setup_logging
-from loggerhead.apps.error import ErrorHandlerApp
-
-
-def main():
-    sys.stderr.write('\n\n')
-    sys.stderr.write('WARNING!!! This script has been deprecated by '
-                     'serve-branches, and will be removed in the next '
-                     'release. Please migrate to serve-branches and report '
-                     'any missing features.\n')
-    sys.stderr.write('\n\n')
-
-    home = os.path.realpath(os.path.dirname(__file__))
-    default_pidfile = os.path.join(home, 'loggerhead.pid')
-    default_configfile = os.path.join(home, 'loggerhead.conf')
-    default_log_folder = os.path.join(home, 'logs')
-    parser = OptionParser(usage='usage: %prog [options]', version='%prog')
-    parser.add_option('-f', '--foreground',
-                      action='store_true', dest='foreground', default=False,
-                      help="run in the foreground; don't daemonize")
-    parser.add_option('-C', '--check', action='store_true',
-                      dest='check', default=False,
-                      help=("only start if not already running (useful for "
-                            "cron jobs)"))
-    parser.add_option('-p', '--pidfile', dest="pidfile",
-                      default=default_pidfile,
-                      type=str, help="override pidfile location")
-    parser.add_option('-c', '--config-file', dest="configfile",
-                      default=default_configfile,
-                      type=str, help="override configuration file location")
-    parser.add_option('-L', '--log-folder', dest="log_folder",
-                      default=default_log_folder,
-                      type=str, help="override log file directory")
-    options, args = parser.parse_args()
-    if len(args) > 0:
-        parser.error('No filename arguments are used, only options.')
-
-    if options.check:
-        if daemon.is_running(options.pidfile):
-            sys.exit(0)
-        sys.stderr.write('Did not find loggerhead running in %r;' % (
-                         options.pidfile))
-        sys.stderr.write(' restarting...\n')
-
-    # read loggerhead config
-
-    config = ConfigObj(options.configfile, encoding='utf-8')
-    extra_path = config.get('bzrpath', None)
-    if extra_path:
-        sys.path.insert(0, extra_path)
-
-    potential_overrides = [('server.socket_port', int),
-                           ('server.webpath', str),
-                           ('server.thread_pool', int),
-                           ('server.socket_host', str)]
-    server_port = int(config.get('server.socket_port', 8080))
-    nworkers = int(config.get('server.thread_pool', 10))
-    server_host = config.get('server.socket_host', '0.0.0.0')
-    webpath = config.get('server.webpath', None)
-
-    for key, keytype in potential_overrides:
-        value = config.get(key, None)
-        if value is not None:
-            value = keytype(value)
-
-    if not options.foreground:
-        sys.stderr.write('\n')
-        sys.stderr.write('Launching loggerhead into the background.\n')
-        sys.stderr.write('PID file: %s\n' % (options.pidfile,))
-        sys.stderr.write('\n')
-
-        daemon.daemonize(options.pidfile, home)
-
-    setup_logging(options.log_folder, config, foreground=options.foreground)
-
-    log = logging.getLogger('loggerhead')
-    log.info('Starting up...')
-
-    app = Root(config)
-
-    app = app
-    app = make_middleware(app)
-    app = make_filter(app, None, logger_name=log.name+'.access')
-    app = ErrorHandlerApp(app)
-
-    if webpath:
-        scheme, netloc, path, blah, blah, blah = urlparse.urlparse(webpath)
-
-        def app(environ, start_response, orig=app):
-            environ['SCRIPT_NAME'] = path
-            environ['HTTP_HOST'] = netloc
-            environ['wsgi.url_scheme'] = scheme
-            return orig(environ, start_response)
-
-    load_plugins()
-
-    try:
-        httpserver.serve(
-            app, host=server_host, port=server_port,
-            threadpool_workers=nworkers)
-    finally:
-        log.info('Shutdown.')
-        try:
-            os.remove(options.pidfile)
-        except OSError:
-            pass
-
-
-if __name__ == '__main__':
-    main()

=== removed file 'start-loggerhead.1'
--- start-loggerhead.1	2008-08-06 18:33:20 +0000
+++ start-loggerhead.1	1970-01-01 00:00:00 +0000
@@ -1,38 +0,0 @@
-.TH START-LOGGERHEAD "1" "July 2008" "start-loggerhead 1.2.1" "User Commands"
-.SH NAME
-start-loggerhead \- start a loggerhead server
-.SH SYNOPSIS
-.B start-loggerhead
-[\fIoptions\fR]
-.SH DESCRIPTION
-Loggerhead is a web viewer for projects in bazaar. It can be used to navigate 
-a branch history, annotate files, view patches, perform searches, etc. It's 
-heavily based on bazaar-webserve, which is itself based on hgweb for Mercurial.
-.PP
-The start-loggerhead command starts a new standalone loggerhead server.
-.SH OPTIONS
-.TP
-\fB\-\-version\fR
-show program's version number and exit
-.TP
-\fB\-h\fR, \fB\-\-help\fR
-show this help message and exit
-.TP
-\fB\-f\fR, \fB\-\-foreground\fR
-run in the foreground; don't daemonize
-.TP
-\fB\-C\fR, \fB\-\-check\fR
-only start if not already running (useful for cron jobs)
-.TP
-\fB\-p\fR, \fB\-\-pidfile=PIDFILE\fR
-override pid file location
-.TP
-\fB\-c\fR, \fB\-\-config-file=CONFIGFILE\fR
-override configuration file location
-.TP
-\fB\-L\fR, \fB\-\-log-folder=LOG-FOLDER\fR
-override log directory
-.SH "LICENSE"
-GNU GPLv2 or later.
-.SH "SEE ALSO"
-https://launchpad.net/loggerhead

=== removed file 'stop-loggerhead'
--- stop-loggerhead	2009-05-13 14:03:48 +0000
+++ stop-loggerhead	1970-01-01 00:00:00 +0000
@@ -1,51 +0,0 @@
-#!/usr/bin/env python
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""A script for stopping the loggerhead process."""
-
-import os
-import sys
-from optparse import OptionParser
-
-home = os.path.realpath(os.path.dirname(__file__))
-default_pidfile = os.path.join(home, 'loggerhead.pid')
-parser = OptionParser(usage='usage: %prog [options]', version='%prog')
-parser.add_option('-p', '--pidfile', dest="pidfile", default=default_pidfile,
-                  help="override pidfile location")
-
-options, args = parser.parse_args()
-if len(args) > 0:
-    parser.error('No filename arguments are used, only options.')
-
-try:
-    f = open(options.pidfile, 'r')
-except IOError, e:
-    print 'Pid file %s not found.' % (options.pidfile,)
-    sys.exit(1)
-
-pid = int(f.readline())
-
-try:
-    os.kill(pid, 0)
-except OSError, e:
-    print 'Stale pid file; server is not running.'
-    sys.exit(1)
-
-print
-print 'Shutting down previous server @ pid %d.' % (pid,)
-print
-
-import signal
-os.kill(pid, signal.SIGINT)

=== removed file 'stop-loggerhead.1'
--- stop-loggerhead.1	2008-08-06 18:39:15 +0000
+++ stop-loggerhead.1	1970-01-01 00:00:00 +0000
@@ -1,19 +0,0 @@
-.TH STOP-LOGGERHEAD "1" "July 2008" "stop-loggerhead 1.2.1" "User Commands"
-.SH NAME
-stop-loggerhead \- stop a loggerhead server
-.SH SYNOPSIS
-.B stop-loggerhead
-.SH DESCRIPTION
-Loggerhead is a web viewer for projects in bazaar. It can be used to navigate 
-a branch history, annotate files, view patches, perform searches, etc. It's 
-heavily based on bazaar-webserve, which is itself based on hgweb for Mercurial.
-.SH OPTIONS
-.TP
-\fB\-p\fR, \fB\-\-pidfile=PIDFILE\fR
-override pid file location
-.PP
-The stop-loggerhead command stops the currently running loggerhead server.
-.SH "LICENSE"
-GNU GPLv2 or later.
-.SH "SEE ALSO"
-https://launchpad.net/loggerhead

